
/*************************************************************************
 Project Name: Project9.prj
 
 Purpose: This project demonstrates plotting data from a file on an OGL 
		  control using the color map and manipulating the OGL controls 
		  to view the data in different orientations.

Project Name: Project9.prj
File Name:    Project9.c
***************************************************************************/
#include <userint.h>
#include <ansi_c.h>
#include "Project9.h"
#include "cviogl.h"
#include <formatio.h>

#define POINTS_X  21
#define POINTS_Y  21

#define PI  3.1415927

static int OGLPanelHandle;
static int OGLControl_ID;
static int SetOGLAttributes(void);                   
static int PlotOGLData(void);

static char buf[256];
double ReadArray[POINTS_Y][POINTS_X];
char  FileName[MAX_FILENAME_LEN];
static ColorMapEntry MapColor[5];

//Function prototypes
void CreateColorMap (int lowColor, int MediumLow, int medColor,int MediumHigh, int highColor);
int CreateColorScale (void);

//Help Message
#define HELP_MSG "\
\n\
\n\
Position the mouse on the OGL control.\n\
The following buttons and keys are used to control the OGL plot:\n\
\n\
To Rotate - Hold the left mouse button and move the OGL object \n\
To Zoom In - Hold the Ctrl key, and move the mouse closer while holding the left mouse button \n\
To Zoom Out - Hold the Ctrl key, and move the mouse further while holding the left mouse button\n\
To Pan - Hold the Shift key, and move the mouse while holding the mouse button \n\
\n\
To view and change the OGL properties, right-click on the OGL control.    \n\
\n\
"
//Program entry point
int main (int argc, char *argv[])
{
    int error = 0;
    if (InitCVIRTE (0, argv, 0) == 0)   /* Initialize CVI libraries */
        return -1;  /* out of memory */
        
    if ((OGLPanelHandle = LoadPanel (0, "Project9.uir", OGLPANEL)) < 0)
        return -1;
        
    // Convert to OpenGL control
    OGLControl_ID = OGLConvertCtrl(OGLPanelHandle,OGLPANEL_OGL_CTRL);
    if (OGLControl_ID<0) 
    {
        OGLGetErrorString (OGLControl_ID, buf, 255);
        MessagePopup("OGLConvertCtrl Error", buf);
        goto Error;
    }    
    
    DisplayPanel (OGLPanelHandle); 
    //Setup CVIOGL control
	SetOGLAttributes();
    RunUserInterface();
    
    //Dicard CVIOGL control
    OGLDiscardCtrl(OGLPanelHandle,OGLControl_ID);  
    return 0;
    
Error :
	//Dicard CVIOGL control
    OGLDiscardCtrl(OGLPanelHandle,OGLControl_ID);
    DiscardPanel(OGLPanelHandle);
    return error;
}//main

//Select the data file, set the attributes, and plot on the OGL control
int CVICALLBACK LoadPlotCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int Index;
	
	switch (event)
		{
		case EVENT_COMMIT:
			//Get filename from ring control
			GetCtrlIndex (OGLPanelHandle, OGLPANEL_LOAD, &Index);
			GetLabelFromIndex (OGLPanelHandle,OGLPANEL_LOAD , Index, FileName );
			//Plot OGL using selected file data
    		PlotOGLData();
			break;
		}
	return 0;
} //LoadPlotCB

//Set OGL attributes
static int SetOGLAttributes(void)
{
    //Enable Properties Pop-up panel
    OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID,
						 OGLATTR_ENABLE_PROPERTY_POPUP, 1);
	//Setup the OGL control background color
	OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID, OGLATTR_BGCOLOR,
							OGLVAL_DK_GRAY);
						 
    //Set up Lighting attributes
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_LIGHTING_ENABLE, 1);
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_LIGHT_SELECT, 1);
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_LIGHT_ENABLE, 1);
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_LIGHT_DISTANCE, 3.0);
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_VIEW_DISTANCE,2.5);
	OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID, OGLATTR_PROJECTION_TYPE,
						  								    OGLVAL_PERSPECTIVE);
    
    //Setup Axis Labels    
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_XNAME_VISIBLE,1);
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_YNAME_VISIBLE,1);
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_ZNAME_VISIBLE,1);
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_XNAME,"X Axis");
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_YNAME,"Y Axis");
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_ZNAME,"Z Axis");
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_XLABEL_VISIBLE,1);
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_YLABEL_VISIBLE,1);
    OGLSetCtrlAttribute(OGLPanelHandle,OGLControl_ID,OGLATTR_ZLABEL_VISIBLE,1);
	
	//Setup grid attributes
	OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID, OGLATTR_XY_GRID_COLOR,
						 OGLVAL_LT_GRAY);
	OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID, OGLATTR_XY_GRID_VISIBLE,
						 1);
	
	OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID, OGLATTR_YZ_GRID_COLOR,
						 OGLVAL_LT_GRAY);
	OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID, OGLATTR_YZ_GRID_VISIBLE,
						 1);
						 
	OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID, OGLATTR_XZ_GRID_COLOR,
						 OGLVAL_LT_GRAY);
	OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID, OGLATTR_XZ_GRID_VISIBLE,
						 1);
    return 0;
    
} //SetOGLAttributes


//Plot OGL using data from the selected data file
static int PlotOGLData(void)
{
    
    int numPlots, i;
    int plotHandle;
    double X_Gain, Y_Gain, X_Offset,Y_Offset;
 
    // Setup control
    OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID,OGLATTR_PLOTAREA_ZSTART, 0.0);
    OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID,OGLATTR_PLOTAREA_ZSIZE,0.75);
    OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID,OGLATTR_VIEW_AUTO_DISTANCE, 0);
    OGLSetCtrlAttribute (OGLPanelHandle, OGLControl_ID,OGLATTR_VIEW_DISTANCE, 2.5);
    OGLGetCtrlAttribute (OGLPanelHandle, OGLControl_ID,OGLATTR_NUM_PLOTHANDLES,&numPlots);   
    
    //Delete any existing plots
    for (i=0;i<numPlots;i++)
    {
        OGLGetCtrlAttribute(OGLPanelHandle,OGLControl_ID, OGLATTR_FIRST_PLOTHANDLE, &plotHandle);  
        OGLDeletePlot(OGLPanelHandle,OGLControl_ID,plotHandle,0);
    }
    
		//Read data from the file into an array
		FileToArray (FileName, ReadArray, VAL_DOUBLE, POINTS_X * POINTS_Y, 1,
					 VAL_GROUPS_TOGETHER, VAL_GROUPS_AS_COLUMNS, VAL_ASCII);
		
		//Specify the gains and offsets
		X_Gain = PI/(POINTS_X-1);
		Y_Gain = PI/(POINTS_Y-1);
		X_Offset =0;
		Y_Offset =0;
		
		//Make the color map 
    	CreateColorMap (VAL_DK_GREEN, VAL_CYAN, VAL_BLUE,VAL_RED, VAL_MAGENTA);  
    	
    	// Plot the color scale next to 3D plot to show this color map
	    if (CreateColorScale() != 0)
	    {
	        MessagePopup ("Initialization Error", "Not able to plot on Color Map scale");
	        return -1;
	    }
	    
		//Plot the OGL data
		plotHandle = OGLPlot3DUniform (OGLPanelHandle, OGLControl_ID,
									   ReadArray, POINTS_X, POINTS_Y,
									   OGLVAL_DOUBLE, X_Gain, X_Offset,
									   					Y_Gain, Y_Offset);
		//Set color scheme of the plot							   					
		OGLSetPlotColorScheme (OGLPanelHandle, OGLControl_ID, plotHandle,
							   OGLVAL_COLORMAP, MapColor, 5, VAL_YELLOW, 1,
							   NULL, 0, 0);
					   
        //Setup plot attributes                                         
        OGLSetPlotAttribute(OGLPanelHandle,OGLControl_ID,plotHandle,OGLATTR_SURFACE_STYLE,OGLVAL_SMOOTH);
        OGLSetPlotAttribute(OGLPanelHandle,OGLControl_ID,plotHandle,OGLATTR_SURFACE_SPECULAR_FACTOR,1.0);
        OGLSetPlotAttribute(OGLPanelHandle,OGLControl_ID,plotHandle,OGLATTR_SURFACE_SHININESS,50);
        OGLSetPlotAttribute(OGLPanelHandle,OGLControl_ID,plotHandle,OGLATTR_WIRE_STYLE,OGLVAL_SOLID);
      	OGLSetPlotAttribute(OGLPanelHandle,OGLControl_ID,plotHandle,OGLATTR_WIRE_COLOR,OGLVAL_BLACK);
    
    	//Display plotHandle
    	OGLRefreshGraph(OGLPanelHandle, OGLControl_ID);
    
    return 0;
}//PlotOGLData

//Create a color map for data magnitude values
void CreateColorMap (int lowColor, int MediumLow, int medColor,int MediumHigh, int highColor)
{
    MapColor[0].dataValue.valDouble = -10.0;
    MapColor[0].color = lowColor;
    MapColor[1].dataValue.valDouble = -5.0;
    MapColor[1].color = MediumLow;
    MapColor[2].dataValue.valDouble = 0.0;
    MapColor[2].color = medColor;
    MapColor[3].dataValue.valDouble = 5.0;
    MapColor[3].color = MediumHigh;
    MapColor[4].dataValue.valDouble = 10.0;
    MapColor[4].color = highColor;
} //CreateColorMap

//Display color map on the Color Map scale
int CreateColorScale (void)
{
    
    double ColorMapArray[5][2];
    int NumberOfXpoints, NumberOfYpoints, NumberOfColors, 
    	InterpolateColors,InterpolatePixels ;
    
    ColorMapArray[0][0] = -10.0;
    ColorMapArray[0][1] = -10.0;
    ColorMapArray[1][0] = -5.0;
    ColorMapArray[1][1] = -5.0;
    ColorMapArray[2][0] =  0.0;
    ColorMapArray[2][1] =  0.0;
    ColorMapArray[3][0] =  5.0;
    ColorMapArray[3][1] =  5.0;
    ColorMapArray[4][0] =  10.0;
    ColorMapArray[4][1] =  10.0;
    
    // Plot this array on the color scale graph control 
    NumberOfXpoints =2;
    NumberOfYpoints =5; 
    NumberOfColors =5; 
    InterpolateColors =0;
    InterpolatePixels =0;
    
    if (PlotIntensity (OGLPanelHandle, OGLPANEL_COLORMAP, ColorMapArray,NumberOfXpoints,
					   	NumberOfYpoints, VAL_DOUBLE, MapColor, VAL_YELLOW, NumberOfColors, 
					   	                 InterpolateColors, InterpolatePixels) <= 0)
        return -1;
    return 0;   
}//CreateColorScale

//Invoked from EXIT command button
int CVICALLBACK ExitCB (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
    }
    return 0;
}//ExitCB

//Invoked from PRINT command button
int CVICALLBACK PrintCB (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int error = OGLNoError, response;
    char PrintFileName[MAX_PATHNAME_LEN];
    switch (event) {
        case EVENT_COMMIT:
            SetWaitCursor (1);
            OGLCopyScaledCtrlBitmap (OGLPanelHandle, OGLControl_ID, 0, -1, -1, -1, -1);
			response = ConfirmPopup ("PRINT LOCATION",
									 "Do you want to print to a file");
			if (response ==0) //Send output to printer
            	PrintPanel (OGLPanelHandle, "", 1, VAL_FULL_PANEL, 1);
            else //Output to File
            {
			   PromptPopup ("PRINT FILE NAME", "Enter Print File Name",
							PrintFileName, MAX_PATHNAME_LEN-1);
			   PrintPanel (OGLPanelHandle, PrintFileName, 0, VAL_FULL_PANEL, 1);
            }
            SetWaitCursor (0);
            break;
    }
    if (error != OGLNoError) 
    {
        char buf[256];
        OGLGetErrorString (error, buf, 255);
        MessagePopup ("OGL Error", buf);
    }    
    return error;
}//PrintCB

//Invoked from Help command button
int CVICALLBACK HelpCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			MessagePopup ("Using OpenGL Controls", HELP_MSG);  
			break;
		}
	return 0;
} //HelpCB

//Invoked by right-clicking on the OpenGL control
int CVICALLBACK PropertiesCB (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            OGLPropertiesPopup(OGLPanelHandle, OGLControl_ID);
            break;
    }
    return 0;
}//PropertiesCB


