#ifndef _CVIOGL_H
#define _CVIOGL_H

#include <cvidef.h>
#include <stdarg.h>

#ifdef __cplusplus
    extern "C" {
#endif

#ifdef __LIBPROT__  
#pragma pack(push, 4)
typedef struct
{
    union
    {
        char            valChar;
        int             valInt;
        short           valShort;
        float           valFloat;
        double          valDouble;
        unsigned char   valUChar;
        unsigned long   valULong;
        unsigned short  valUShort;
    } dataValue;
    int color;
} ColorMapEntry;
#pragma pack(pop)
#else
#include <userint.h>
#endif

// Typedefs
// Typedefs for vertices to be created to a scatter plot.
typedef struct _OGLVertexC
{
    char x,y,z;
} OGLVertexC;

typedef struct _OGLVertexI
{
    int x,y,z;
} OGLVertexI;

typedef struct _OGLVertexSI
{
    short int x,y,z;
} OGLVertexSI;

typedef struct _OGLVertexUC
{
    unsigned char x,y,z;
} OGLVertexUC;

typedef struct _OGLVertexUI
{
    unsigned int x,y,z;
} OGLVertexUI;

typedef struct _OGLVertexUSI
{
    unsigned short int x,y,z;
} OGLVertexUSI;

typedef struct _OGLVertexF
{
    float x,y,z;
} OGLVertexF;

typedef struct _OGLVertexD
{
    double x,y,z;
} OGLVertexD;

//------------------------------------------------------------------------------------------------
// defines
//------------------------------------------------------------------------------------------------

// Defaults and Maximum values
#define OGL_NUM_LIGHTS                      4
#define OGL_MAX_DIVISIONS                   100
#define OGL_MAX_PRECISION                   15
#define OGL_MAX_POINTSIZE                   100.0
#define OGL_DEFAULT_FONT                    "Arial"
#define OGL_MAX_TRANSPARENCY                100
#define OGL_MAX_LIGHT_SPOT_EXPONENT         128
#define OGL_MAX_SURFACE_SHININESS           128
#define OGL_MAX_POINT_SIZE                  48
#define OGL_MAX_FONTNAME_SIZE               32
                                          
// ATTRIBUTES                             
#define CTRL_ATTR_BASE                      10000L

// Common to controls and plots
#define OGLATTR_COPY_ORIGINAL_DATA          (CTRL_ATTR_BASE +   1)   
#define OGLATTR_DATA_MODE                   (CTRL_ATTR_BASE +   2)


#define OGLATTR_VISIBLE                     (CTRL_ATTR_BASE +   3)
#define OGLATTR_ENABLE                      (CTRL_ATTR_BASE +   4)
#define OGLATTR_PLOTTING_ENABLE             (CTRL_ATTR_BASE +   5)

// Windows specific
#define OGLATTR_HWND                        (CTRL_ATTR_BASE +   6)
#define OGLATTR_HDC                         (CTRL_ATTR_BASE +   7)
#define OGLATTR_HRC                         (CTRL_ATTR_BASE +   8)

// For controls
// Present in userint.h
#define OGLATTR_LEFT                        (CTRL_ATTR_BASE +  10)
#define OGLATTR_TOP                         (CTRL_ATTR_BASE +  11)
#define OGLATTR_WIDTH                       (CTRL_ATTR_BASE +  12)
#define OGLATTR_HEIGHT                      (CTRL_ATTR_BASE +  13)
                                        
// Control Label                        
// Present in userint.h
#define OGLATTR_LABEL_COLOR                 (CTRL_ATTR_BASE +  15) 
#define OGLATTR_LABEL_TEXT                  (CTRL_ATTR_BASE +  16) 
#define OGLATTR_LABEL_FONT                  (CTRL_ATTR_BASE +  17)
#define OGLATTR_LABEL_ITALIC                (CTRL_ATTR_BASE +  18)
#define OGLATTR_LABEL_UNDERLINE             (CTRL_ATTR_BASE +  19)
#define OGLATTR_LABEL_STRIKEOUT             (CTRL_ATTR_BASE +  20)
#define OGLATTR_LABEL_POINT_SIZE            (CTRL_ATTR_BASE +  21)
#define OGLATTR_LABEL_BOLD                  (CTRL_ATTR_BASE +  22)
#define OGLATTR_LABEL_JUSTIFY               (CTRL_ATTR_BASE +  23)
#define OGLATTR_LABEL_TEXT_LENGTH           (CTRL_ATTR_BASE +  24)
#define OGLATTR_LABEL_FONT_NAME_LENGTH      (CTRL_ATTR_BASE +  25)
#define OGLATTR_LABEL_VISIBLE               (CTRL_ATTR_BASE +  26) 
#define OGLATTR_LABEL_SIZE_TO_TEXT          (CTRL_ATTR_BASE +  27)                               

// 3D view attributes
#define OGLATTR_PROJECTION_TYPE             (CTRL_ATTR_BASE +  30) // OGLVAL_ORHTOGRAPHIC, OGLVAL_PERSPECTIVES
#define OGLATTR_VIEW_DIRECTION              (CTRL_ATTR_BASE +  31)
#define OGLATTR_VIEW_LONGITUDE              (CTRL_ATTR_BASE +  32)  
#define OGLATTR_VIEW_LATITUDE               (CTRL_ATTR_BASE +  33)  
#define OGLATTR_VIEW_DISTANCE               (CTRL_ATTR_BASE +  34)  
#define OGLATTR_VIEW_AUTO_DISTANCE          (CTRL_ATTR_BASE +  35)  
#define OGLATTR_VIEW_CENTERX                (CTRL_ATTR_BASE +  36)  
#define OGLATTR_VIEW_CENTERY                (CTRL_ATTR_BASE +  37)  
#define OGLATTR_VIEW_CENTERZ                (CTRL_ATTR_BASE +  38)  

// Background color
#define OGLATTR_BGCOLOR               (CTRL_ATTR_BASE +  45) 

// Zoom-Pan-Rotate
#define OGLATTR_FAST_PAN_ZOOM_ROTATE        (CTRL_ATTR_BASE +  46) // int boolean
#define OGLATTR_ENABLE_PAN_ZOOM_ROTATE      (CTRL_ATTR_BASE +  47) // int bitfields
#define OGLATTR_ENABLE_PROPERTY_POPUP       (CTRL_ATTR_BASE +  48) // int boolean

// Lighting attributes
#define OGLATTR_AMBIENT_COLOR               (CTRL_ATTR_BASE +  50) // Ambient when 4 lights are disabled

// These are for individual lights selected using light select
#define OGLATTR_LIGHTING_ENABLE             (CTRL_ATTR_BASE +  51) // Enables 4 lights
#define OGLATTR_LIGHT_SELECT                (CTRL_ATTR_BASE +  52)
#define OGLATTR_LIGHT_ENABLE                (CTRL_ATTR_BASE +  53) // Individual light enable
#define OGLATTR_LIGHT_COLOR                 (CTRL_ATTR_BASE +  54)
#define OGLATTR_LIGHT_LONGITUDE             (CTRL_ATTR_BASE +  55)
#define OGLATTR_LIGHT_LATITUDE              (CTRL_ATTR_BASE +  56)
#define OGLATTR_LIGHT_DISTANCE              (CTRL_ATTR_BASE +  57)
#define OGLATTR_LIGHT_ATTENUATION           (CTRL_ATTR_BASE +  58)
#define OGLATTR_LIGHT_AMBIENT               (CTRL_ATTR_BASE +  59)
#define OGLATTR_LIGHT_DIFFUSE               (CTRL_ATTR_BASE +  60)
#define OGLATTR_LIGHT_SPECULAR              (CTRL_ATTR_BASE +  61)
#define OGLATTR_LIGHT_SPOT_ENABLE           (CTRL_ATTR_BASE +  62)
#define OGLATTR_LIGHT_SPOT_ANGLE            (CTRL_ATTR_BASE +  63)
#define OGLATTR_LIGHT_SPOT_EXPONENT         (CTRL_ATTR_BASE +  64)
                                                               
#define OGLATTR_PLOTAREA_XSTART           (CTRL_ATTR_BASE +  70)
#define OGLATTR_PLOTAREA_XSIZE            (CTRL_ATTR_BASE +  71)
#define OGLATTR_PLOTAREA_YSTART           (CTRL_ATTR_BASE +  72)
#define OGLATTR_PLOTAREA_YSIZE            (CTRL_ATTR_BASE +  73)
#define OGLATTR_PLOTAREA_ZSTART           (CTRL_ATTR_BASE +  74)
#define OGLATTR_PLOTAREA_ZSIZE            (CTRL_ATTR_BASE +  75)
                                                               
// Axes
// X Axis attributes
#define OGLATTR_XNAME_VISIBLE               (CTRL_ATTR_BASE + 100)      
#define OGLATTR_XNAME_FONT                  (CTRL_ATTR_BASE + 101)      
#define OGLATTR_XNAME_COLOR                 (CTRL_ATTR_BASE + 102)   
#define OGLATTR_XNAME_ITALIC                (CTRL_ATTR_BASE + 103)
#define OGLATTR_XNAME_UNDERLINE             (CTRL_ATTR_BASE + 104)
#define OGLATTR_XNAME_STRIKEOUT             (CTRL_ATTR_BASE + 105)
#define OGLATTR_XNAME_POINT_SIZE            (CTRL_ATTR_BASE + 106)
#define OGLATTR_XNAME_BOLD                  (CTRL_ATTR_BASE + 107)

#define OGLATTR_XLABEL_FONT                 (CTRL_ATTR_BASE + 110)   
#define OGLATTR_XLABEL_COLOR                (CTRL_ATTR_BASE + 111)   
#define OGLATTR_XLABEL_ITALIC               (CTRL_ATTR_BASE + 112)
#define OGLATTR_XLABEL_UNDERLINE            (CTRL_ATTR_BASE + 113)
#define OGLATTR_XLABEL_STRIKEOUT            (CTRL_ATTR_BASE + 114)
#define OGLATTR_XLABEL_POINT_SIZE           (CTRL_ATTR_BASE + 115)
#define OGLATTR_XLABEL_BOLD                 (CTRL_ATTR_BASE + 116)
#define OGLATTR_XLABEL_VISIBLE              (CTRL_ATTR_BASE + 117)  
                                    
// Present in userint.h
#define OGLATTR_XNAME                       (CTRL_ATTR_BASE + 120)  
#define OGLATTR_XNAME_LENGTH                (CTRL_ATTR_BASE + 121)  
#define OGLATTR_XMAP_MODE                   (CTRL_ATTR_BASE + 122)  
#define OGLATTR_XFORMAT                     (CTRL_ATTR_BASE + 123)  
#define OGLATTR_XDIVISIONS                  (CTRL_ATTR_BASE + 124) 
#define OGLATTR_XPRECISION                  (CTRL_ATTR_BASE + 125) 
#define OGLATTR_XAXIS_GAIN                  (CTRL_ATTR_BASE + 126) 
#define OGLATTR_XAXIS_OFFSET                (CTRL_ATTR_BASE + 127)                                         
#define OGLATTR_XREVERSE                    (CTRL_ATTR_BASE + 128)
#define OGLATTR_XMIN                        (CTRL_ATTR_BASE + 129)  
#define OGLATTR_XMAX                        (CTRL_ATTR_BASE + 130)  
#define OGLATTR_XAXIS_SCALING               (CTRL_ATTR_BASE + 131)  

// Y Axis attribute
#define OGLATTR_YNAME_VISIBLE               (CTRL_ATTR_BASE + 200)    
#define OGLATTR_YNAME_FONT                  (CTRL_ATTR_BASE + 201)    
#define OGLATTR_YNAME_COLOR                 (CTRL_ATTR_BASE + 202) 
#define OGLATTR_YNAME_ITALIC                (CTRL_ATTR_BASE + 203)
#define OGLATTR_YNAME_UNDERLINE             (CTRL_ATTR_BASE + 204)
#define OGLATTR_YNAME_STRIKEOUT             (CTRL_ATTR_BASE + 205)
#define OGLATTR_YNAME_POINT_SIZE            (CTRL_ATTR_BASE + 206)
#define OGLATTR_YNAME_BOLD                  (CTRL_ATTR_BASE + 207)
                                                                  
#define OGLATTR_YLABEL_FONT                 (CTRL_ATTR_BASE + 210) 
#define OGLATTR_YLABEL_COLOR                (CTRL_ATTR_BASE + 211)  
#define OGLATTR_YLABEL_ITALIC               (CTRL_ATTR_BASE + 212)
#define OGLATTR_YLABEL_UNDERLINE            (CTRL_ATTR_BASE + 213)
#define OGLATTR_YLABEL_STRIKEOUT            (CTRL_ATTR_BASE + 214)
#define OGLATTR_YLABEL_POINT_SIZE           (CTRL_ATTR_BASE + 215)
#define OGLATTR_YLABEL_BOLD                 (CTRL_ATTR_BASE + 216)
#define OGLATTR_YLABEL_VISIBLE              (CTRL_ATTR_BASE + 217)
                                                                  
// Present in userint.h                                           
#define OGLATTR_YNAME                       (CTRL_ATTR_BASE + 220)
#define OGLATTR_YNAME_LENGTH                (CTRL_ATTR_BASE + 221)
#define OGLATTR_YMAP_MODE                   (CTRL_ATTR_BASE + 222)
#define OGLATTR_YFORMAT                     (CTRL_ATTR_BASE + 223)
#define OGLATTR_YDIVISIONS                  (CTRL_ATTR_BASE + 224)
#define OGLATTR_YPRECISION                  (CTRL_ATTR_BASE + 225)  
#define OGLATTR_YAXIS_GAIN                  (CTRL_ATTR_BASE + 226)
#define OGLATTR_YAXIS_OFFSET                (CTRL_ATTR_BASE + 227)
#define OGLATTR_YREVERSE                    (CTRL_ATTR_BASE + 228)
#define OGLATTR_YMIN                        (CTRL_ATTR_BASE + 229)
#define OGLATTR_YMAX                        (CTRL_ATTR_BASE + 230)
#define OGLATTR_YAXIS_SCALING               (CTRL_ATTR_BASE + 231)

// Z Axis attribute
#define OGLATTR_ZNAME_FONT                  (CTRL_ATTR_BASE + 300) 
#define OGLATTR_ZNAME_COLOR                 (CTRL_ATTR_BASE + 301)  
#define OGLATTR_ZNAME_ITALIC                (CTRL_ATTR_BASE + 302)
#define OGLATTR_ZNAME_UNDERLINE             (CTRL_ATTR_BASE + 303)
#define OGLATTR_ZNAME_STRIKEOUT             (CTRL_ATTR_BASE + 304)
#define OGLATTR_ZNAME_POINT_SIZE            (CTRL_ATTR_BASE + 305)
#define OGLATTR_ZNAME_BOLD                  (CTRL_ATTR_BASE + 306)
#define OGLATTR_ZNAME_VISIBLE               (CTRL_ATTR_BASE + 307)     
                                                                  
#define OGLATTR_ZLABEL_VISIBLE              (CTRL_ATTR_BASE + 310)     
#define OGLATTR_ZLABEL_FONT                 (CTRL_ATTR_BASE + 311)  
#define OGLATTR_ZLABEL_COLOR                (CTRL_ATTR_BASE + 312)   
#define OGLATTR_ZLABEL_ITALIC               (CTRL_ATTR_BASE + 313)
#define OGLATTR_ZLABEL_UNDERLINE            (CTRL_ATTR_BASE + 314)
#define OGLATTR_ZLABEL_STRIKEOUT            (CTRL_ATTR_BASE + 315)
#define OGLATTR_ZLABEL_POINT_SIZE           (CTRL_ATTR_BASE + 316)
#define OGLATTR_ZLABEL_BOLD                 (CTRL_ATTR_BASE + 317)
                                                                  
#define OGLATTR_ZNAME                       (CTRL_ATTR_BASE + 320)
#define OGLATTR_ZNAME_LENGTH                (CTRL_ATTR_BASE + 321)
#define OGLATTR_ZMAP_MODE                   (CTRL_ATTR_BASE + 322)
#define OGLATTR_ZFORMAT                     (CTRL_ATTR_BASE + 323)
#define OGLATTR_ZDIVISIONS                  (CTRL_ATTR_BASE + 324)
#define OGLATTR_ZPRECISION                  (CTRL_ATTR_BASE + 325)
#define OGLATTR_ZAXIS_GAIN                  (CTRL_ATTR_BASE + 326)
#define OGLATTR_ZAXIS_OFFSET                (CTRL_ATTR_BASE + 327)
#define OGLATTR_ZREVERSE                    (CTRL_ATTR_BASE + 328)
#define OGLATTR_ZMIN                        (CTRL_ATTR_BASE + 329)
#define OGLATTR_ZMAX                        (CTRL_ATTR_BASE + 330)
#define OGLATTR_ZAXIS_SCALING               (CTRL_ATTR_BASE + 331)
                                                                  
#define OGLATTR_XY_PLANE_VISIBLE            (CTRL_ATTR_BASE + 400)
#define OGLATTR_YZ_PLANE_VISIBLE            (CTRL_ATTR_BASE + 401)
#define OGLATTR_XZ_PLANE_VISIBLE            (CTRL_ATTR_BASE + 402)
#define OGLATTR_XY_GRID_VISIBLE             (CTRL_ATTR_BASE + 403)
#define OGLATTR_YZ_GRID_VISIBLE             (CTRL_ATTR_BASE + 404)
#define OGLATTR_XZ_GRID_VISIBLE             (CTRL_ATTR_BASE + 405)
#define OGLATTR_XY_GRID_COLOR               (CTRL_ATTR_BASE + 406)
#define OGLATTR_YZ_GRID_COLOR               (CTRL_ATTR_BASE + 407)
#define OGLATTR_XZ_GRID_COLOR               (CTRL_ATTR_BASE + 408)
#define OGLATTR_XY_PLANE_ZLOCATION_OVERRIDE (CTRL_ATTR_BASE + 409)
#define OGLATTR_XY_PLANE_ZLOCATION          (CTRL_ATTR_BASE + 410)
#define OGLATTR_YZ_PLANE_XLOCATION_OVERRIDE (CTRL_ATTR_BASE + 411)
#define OGLATTR_YZ_PLANE_XLOCATION          (CTRL_ATTR_BASE + 412)
#define OGLATTR_XZ_PLANE_YLOCATION_OVERRIDE (CTRL_ATTR_BASE + 413)
#define OGLATTR_XZ_PLANE_YLOCATION          (CTRL_ATTR_BASE + 414)
#define OGLATTR_TICKS_X_ON_XY               (CTRL_ATTR_BASE + 415)
#define OGLATTR_TICKS_X_ON_XZ               (CTRL_ATTR_BASE + 416)
#define OGLATTR_TICKS_Y_ON_XY               (CTRL_ATTR_BASE + 417)
#define OGLATTR_TICKS_Y_ON_YZ               (CTRL_ATTR_BASE + 418)
#define OGLATTR_TICKS_Z_ON_YZ               (CTRL_ATTR_BASE + 419)
#define OGLATTR_TICKS_Z_ON_XZ               (CTRL_ATTR_BASE + 420)

// Plot Handles
#define OGLATTR_NUM_PLOTHANDLES             (CTRL_ATTR_BASE + 450) // returns the number of valid plot handles
#define OGLATTR_FIRST_PLOTHANDLE            (CTRL_ATTR_BASE + 451) // returns the first valid plot handle
#define OGLATTR_NEXT_PLOTHANDLE             (CTRL_ATTR_BASE + 452) // returns the next valid plot handle, returns 0 if no next

// Plot Attributes
#define OGLATTR_PLOT_VISIBLE                (CTRL_ATTR_BASE + 460)
#define OGLATTR_TRANSPARENCY                (CTRL_ATTR_BASE + 461)

// Color map.
#define OGLATTR_COLOR_SCHEME              (CTRL_ATTR_BASE + 430)
#define OGLATTR_COLORMAP                  (CTRL_ATTR_BASE + 431) 
#define OGLATTR_COLORMAP_NUMCOLORS        (CTRL_ATTR_BASE + 432)
#define OGLATTR_COLORMAP_HICOLOR          (CTRL_ATTR_BASE + 433)
#define OGLATTR_COLORMAP_INTERP           (CTRL_ATTR_BASE + 434)
#define OGLATTR_COLORARRAY                (CTRL_ATTR_BASE + 435)
#define OGLATTR_COLORARRAY_XSIZE          (CTRL_ATTR_BASE + 436)
#define OGLATTR_COLORARRAY_YSIZE          (CTRL_ATTR_BASE + 437)

#define OGLATTR_POINT_COLOR               (CTRL_ATTR_BASE + 470)
#define OGLATTR_POINT_SIZE                (CTRL_ATTR_BASE + 471)
#define OGLATTR_POINT_SMOOTH              (CTRL_ATTR_BASE + 473)                      
                                            
#define OGLATTR_WIRE_COLOR                (CTRL_ATTR_BASE + 480)
#define OGLATTR_WIRE_STYLE                (CTRL_ATTR_BASE + 481)
#define OGLATTR_WIRE_SMOOTH               (CTRL_ATTR_BASE + 483)                      
                                            
#define OGLATTR_SURFACE_STYLE             (CTRL_ATTR_BASE + 490)
#define OGLATTR_SURFACE_COLOR             (CTRL_ATTR_BASE + 491)
#define OGLATTR_SURFACE_SPECULAR_FACTOR   (CTRL_ATTR_BASE + 492)
#define OGLATTR_SURFACE_SHININESS         (CTRL_ATTR_BASE + 493)

// Projections                                                 
#define OGLATTR_PROJECT_TO_XY_PLANE         (CTRL_ATTR_BASE + 500)  
#define OGLATTR_PROJECT_TO_XZ_PLANE         (CTRL_ATTR_BASE + 501)  
#define OGLATTR_PROJECT_TO_YZ_PLANE         (CTRL_ATTR_BASE + 502)  
#define OGLATTR_PROJECTIONS_ONLY            (CTRL_ATTR_BASE + 503)  

#define OGLATTR_HIDE_PROPERTY_TAB           (CTRL_ATTR_BASE + 510) 

// ATTRIBUTE VALUES
#define OGLVAL_AUTO                       -1L
#define OGLVAL_NONE                       -2L
#define OGLVAL_USER_DEFINED               -3L
#define OGLVAL_MANUAL                     -4L

#define OGLVAL_TRUE                        1L   
#define OGLVAL_FALSE                       0L   

// Data Types
#define OGLVAL_CHAR                        0L
#define OGLVAL_INTEGER                     1L
#define OGLVAL_SHORT_INTEGER               2L
#define OGLVAL_FLOAT                       3L
#define OGLVAL_DOUBLE                      4L
#define OGLVAL_STRING                      5L
#define OGLVAL_UNSIGNED_SHORT_INTEGER      6L
#define OGLVAL_UNSIGNED_INTEGER            7L
#define OGLVAL_UNSIGNED_CHAR               8L

// Data Mode
#define OGLVAL_RETAIN                      1L
#define OGLVAL_DISCARD                     2L
                                         
// Projection type                       
#define OGLVAL_ORTHOGRAPHIC                1L
#define OGLVAL_PERSPECTIVE                 2L
// View position                         
#define OGLVAL_AUTO_XY                     1L
#define OGLVAL_AUTO_YZ                     2L
#define OGLVAL_AUTO_XZ                     3L
// Attenuation                           
#define OGLVAL_CONSTANT_ATTN               1L
#define OGLVAL_LINEAR_ATTN                 2L
#define OGLVAL_QUADRATIC_ATTN              3L
#define OGLVAL_ATTN_MIN                    OGLVAL_CONSTANT_ATTN
#define OGLVAL_ATTN_MAX                    OGLVAL_QUADRATIC_ATTN
// Wire styles                           
#define OGLVAL_SOLID                       0L
#define OGLVAL_DASH                        1L
#define OGLVAL_DOT                         2L
#define OGLVAL_DASH_DOT                    3L
#define OGLVAL_DASH_DOT_DOT                4L
#define OGLVAL_WIRE_MIN                    OGLVAL_SOLID
#define OGLVAL_WIRE_MAX                    OGLVAL_DASH_DOT_DOT
//Surface styles                         
#define OGLVAL_SMOOTH                      1L
#define OGLVAL_FLAT                        2L
#define OGLVAL_SURFACE_MIN                 OGLVAL_SMOOTH
#define OGLVAL_SURFACE_MAX                 OGLVAL_FLAT
// Surface Color                         
#define OGLVAL_BGCOLOR                     0xff000000
// 3dColorMap                            
#define OGLVAL_SHADED                      1L
#define OGLVAL_COLOR_SPECTRUM              2L
#define OGLVAL_GRAYSCALE                   3L
#define OGLVAL_COLORMAP                    4L
#define OGLVAL_COLORARRAY                  5L
#define OGLVAL_COLORMAP_MIN                OGLVAL_SHADED
#define OGLVAL_COLORMAP_MAX                OGLVAL_COLORARRAY
// grid location                         
#define OGLVAL_CENTER                      1L
#define OGLVAL_MIN                         2L
#define OGLVAL_MAX                         3L
// label locations                       
#define OGLVAL_GRID_TOP                    0x0001
#define OGLVAL_GRID_LEFT                   0x0002
#define OGLVAL_GRID_BOTTOM                 0x0004
#define OGLVAL_GRID_RIGHT                  0x0008
// Label formats
#define OGLVAL_FLOATING_PT_FORMAT          0xFFF0
#define OGLVAL_SCIENTIFIC_FORMAT           0xFFF1
#define OGLVAL_ENGINEERING_FORMAT          0xFFF2
// Map Modes
#define OGLVAL_LINEAR                      1L
#define OGLVAL_LOG                         2L
//Pan, Zoom and Rotate
#define OGLVAL_ROTATE                      1L
#define OGLVAL_ZOOM                        (OGLVAL_ROTATE << 1)
#define OGLVAL_PAN                         (OGLVAL_ZOOM << 1)
#define OGLVAL_PAN_ZOOM_ROTATE_MIN         OGLVAL_ROTATE
#define OGLVAL_PAN_ZOOM_ROTATE_MAX         (OGLVAL_ROTATE | OGLVAL_ZOOM | OGLVAL_PAN)
// Colors
#define OGLVAL_RED                         0xFF0000L /* 16 standard colors */
#define OGLVAL_GREEN                       0x00FF00L                         
#define OGLVAL_BLUE                        0x0000FFL                         
#define OGLVAL_CYAN                        0x00FFFFL                         
#define OGLVAL_MAGENTA                     0xFF00FFL                         
#define OGLVAL_YELLOW                      0xFFFF00L                         
#define OGLVAL_DK_RED                      0x800000L                         
#define OGLVAL_DK_BLUE                     0x000080L                         
#define OGLVAL_DK_GREEN                    0x008000L                         
#define OGLVAL_DK_CYAN                     0x008080L                         
#define OGLVAL_DK_MAGENTA                  0x800080L                         
#define OGLVAL_DK_YELLOW                   0x808000L                         
#define OGLVAL_LT_GRAY                     0xC0C0C0L                         
#define OGLVAL_DK_GRAY                     0x808080L                         
#define OGLVAL_BLACK                       0x000000L                         
#define OGLVAL_WHITE                       0xFFFFFFL                         

// Data Modes
#define OGLVAL_RETAIN                       1L
#define OGLVAL_DISCARD                      2L            
                                            
#define OGLVAL_PROPERTY_CTRL                1L
#define OGLVAL_PROPERTY_3D                  (OGLVAL_PROPERTY_CTRL << 1)
#define OGLVAL_PROPERTY_AXIS                (OGLVAL_PROPERTY_3D << 1)
#define OGLVAL_PROPERTY_LIGHTING            (OGLVAL_PROPERTY_AXIS << 1)
#define OGLVAL_PROPERTY_PLOTS               (OGLVAL_PROPERTY_LIGHTING << 1)
#define OGLVAL_PROPERTY_MIN                 0L
#define OGLVAL_PROPERTY_MAX                 ((OGLVAL_PROPERTY_PLOTS << 1) - 1)

//Error Codes
#define ERROR_BASE                                      -1000
#define OGLNoError                                      0
#define OGLErrorChildPanel                            (ERROR_BASE - 1)
#define OGLErrorNoHwndParent                          (ERROR_BASE - 2) // Should never occur
#define OGLErrorCanvasOrPictureRequired               (ERROR_BASE - 3)
#define OGLErrorWndRegistration                       (ERROR_BASE - 4)
#define OGLErrorWndCreation                           (ERROR_BASE - 5)
#define OGLErrorCtrlNotFound                          (ERROR_BASE - 6)
#define OGLErrorTooManyCtrls                          (ERROR_BASE - 7)
#define OGLErrorPlotNotFound                          (ERROR_BASE - 8)
#define OGLErrorTooManyPlots                          (ERROR_BASE - 9)
#define OGLErrorLabelAttributeCouldNotBeGot           (ERROR_BASE - 10)       
#define OGLErrorCouldNotHideControl                   (ERROR_BASE - 11)
#define OGLErrorCouldNotUnhideControl                 (ERROR_BASE - 12)
#define OGLErrorCouldNotMoveControl                   (ERROR_BASE - 13)
#define OGLErrorCouldNotResizeControl                 (ERROR_BASE - 14)
#define OGLErrorInvalidParameters                     (ERROR_BASE - 15)
#define OGLErrorInvalidOGLCtrlAttribute               (ERROR_BASE - 16)
#define OGLErrorInvalidOGLPlotAttribute               (ERROR_BASE - 17)
#define OGLErrorOpenGLError                           (ERROR_BASE - 18)
#define OGLErrorWindowsError                          (ERROR_BASE - 19)
#define OGLErrorFontNameTooLong                       (ERROR_BASE - 20)
#define OGLErrorAttributeNotSettable                  (ERROR_BASE - 21)
#define OGLErrorNotEnoughMemory                       (ERROR_BASE - 22)
#define OGLErrorValueIsInvalidOrOutOfRange            (ERROR_BASE - 23)
#define OGLErrorCouldNotSetContext                    (ERROR_BASE - 24)
#define OGLErrorCouldNotRestoreContext                (ERROR_BASE - 25)
#define OGLErrorInvalidDataType                       (ERROR_BASE - 26)
#define OGLErrorUninitializedOrZeroLen                (ERROR_BASE - 27)
#define OGLErrorInvalidThread                         (ERROR_BASE - 28) 
#define OGLErrorAlreadyRefreshing                     (ERROR_BASE - 29) 
#define OGLErrorUnsupportedPixelDepth                 (ERROR_BASE - 30) 

//----------------------------------------------------------------------------
// Events
//----------------------------------------------------------------------------
#define EVENT_BASE          1000
#define OGLEVENT_REFRESH    (EVENT_BASE + 1)

//----------------------------------------------------------------------------
// Prototypes
//----------------------------------------------------------------------------
#ifdef _CVI_
#pragma EnableLibraryRuntimeChecking
#endif

int DLLEXPORT CVIFUNC OGLNewCtrl(int panel, char *label, int top, int left);
int DLLEXPORT CVIFUNC OGLConvertCtrl(int panel, int control);
int DLLEXPORT CVIFUNC OGLDiscardCtrl(int panel,int controlId);
int DLLEXPORT CVIFUNC OGLGetCtrlAttribute(int panel, int controlId, int attribute, void *value);
int DLLEXPORT CVICDECL OGLSetCtrlAttribute(int panel, int controlId, int attribute, ...);
int DLLEXPORT CVIFUNC OGLRefreshGraph(int panel,int controlId);
int DLLEXPORT CVIFUNC OGLPropertiesPopup(int panel,int controlId);
int DLLEXPORT CVIFUNC OGLGetScaledCtrlBitmap(int panel, int controlId,int fullSize,int width,int height,int *bitmapId);
int DLLEXPORT CVIFUNC OGLCopyScaledCtrlBitmap(int panel,int controlId,int fullSize,int width,int height,int destPanel,int destControl);
int DLLEXPORT CVIFUNC OGLGetLastOpenGLError(int panel,int controlId,int *openGLError);
void DLLEXPORT CVIFUNC OGLGetErrorString(int errorCode,char *errorBuf,int length);

int DLLEXPORT CVIFUNC OGLPlot3DScatter(   int panel, int controlId, void *XYZArray,
                        int numXPoints, int numYPoints, int dataType);
int DLLEXPORT CVIFUNC OGLPlot3DUniform(   int panel, int controlId, void *ZArray,
                        int numXPoints, int numYPoints, int dataType,
                        double xGain, double xOffset,
                        double yGain, double yOffset);
int DLLEXPORT CVIFUNC OGLDeletePlot (int panel, int controlId, int plotHandle, int refresh);
int DLLEXPORT CVIFUNC OGLGetPlotAttribute(int panel, int controlId, int plotHandle, int attribute, void *value);
int DLLEXPORT CVICDECL OGLSetPlotAttribute(int panel, int controlId, int plotHandle, int attribute, ...);
int DLLEXPORT CVIFUNC OGLSetPlotColorScheme(  int panel, int controlId, int plotHandle, 
                                int colorScheme,ColorMapEntry *colormap,
                                int numColors,int hiColor,int interp,
                                int *colorArray,int xSize,int ySize);
                                
int DLLEXPORT CVIFUNC OGLSetCtrlAttributeFromParmInfo(int panel, int controlId, int attribute, va_list parmInfo);
int DLLEXPORT CVIFUNC OGLSetPlotAttributeFromParmInfo(int panel, int controlId, int plotHandle, int attribute, va_list parmInfo);

#ifdef __cplusplus
    }
#endif

#endif  /* _CVIOGL_H */
