/****************************************************************

Purpose:    This project shows you how to create a user interface
 			programmatically without creating controls using the 
 			User Interface Editor.
 			Various graphs and command button controls are created and 
 			their attributes set and modified programmatically.
 			
 			Note:  This project uses the advanced analysis library functions: 
 			SinePattern,Uniform, and WhiteNoise
 			The function declarations for these functions is in the header file
 			"<analysis.h>".   If you do not have advanced analysis library functions 
 			in your package, substitute the advanced library functions with your 
 			functions.

 Project Name: project1-1.prj
 File Name:    project1-1.c
 
*******************************************************************/

#include <formatio.h>
#include <analysis.h>
#include <utility.h>
#include <userint.h>
#include <ansi_c.h> 

/******************Function Prototypes ************************/

void CreateGUI(void);	 //Create the GUI programmatically
void CreateDisplayDataPanel(void); //Create Data panel

//Invoked when any item on Ring control is selected 
int CVICALLBACK RingCB (int panel, int control, int event,          
					void *callbackData, int eventData1, int eventData2);	
//Invoked by "Start" command button  		
int CVICALLBACK StartGUICB (int panel, int control, int event,
		   			void *callbackData, int eventData1, int eventData2);
//Invoked by "Stop" command button
int CVICALLBACK StopGUICB (int panel, int control, int event,
					void *callbackData, int eventData1, int eventData2); 
//Invoked by "Exit" command button					
int CVICALLBACK ExitGUICB (int panel, int control, int event,
					void *callbackData, int eventData1, int eventData2);   
//Invoked by "CLEAR" command button 					
int CVICALLBACK ClearGraphCB (int panel, int control, int event,
					void *callbackData, int eventData1, int eventData2);
//Plot Uniform random numbers 		
int CVICALLBACK PlotUniformCB (int panel, int control, int event,
					void *callbackData, int eventData1, int eventData2);
//Done entering data
int CVICALLBACK DoneGUICB (int panel, int control, int event,
					void *callbackData, int eventData1, int eventData2);
//Change Samples Callback					
int CVICALLBACK SamplesControlCB(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2);

/**************************************************************/
static int GUI_Handle, Child_Handle;

//User Interface control IDs
int GUI_START_ID, GRAPH_ID, TEXT_BOX_ID, CLEAR_PLOT_ID, PLOT_UNI_ID, AMPLITUDE_ID,
	PHASE_ID, CYCLES_ID, SAMPLES_ID,TEXT_MSG, SELECTED_ITEM, TABLE_MSG, RING_BOX; 

double amp, Y_axis[101], phase;
int seed, samples, PlotHandle, cycles, status;

//main
int main (int argc, char *argv[])  
{
	 
	if (InitCVIRTE (0, argv, 0) == 0)   //Used for DLLs or external compilers
		return -1;

	CreateGUI();   //Create the GUI 
	DisplayPanel (GUI_Handle);  

	RunUserInterface();
	DiscardPanel(GUI_Handle); 

return 0;
} //main


//Create the GUI    
void CreateGUI(void)
{

int DEC_BOX_CMDS,DEC_BOX_GRAPH, DEC_BOX_FUNC,TEXT_MSG_FUNC,GUI_EXIT_ID;

	//Create a new panel
	GUI_Handle = NewPanel (0, "PROGRAMMATICALLY CREATED GUI",
						   VAL_AUTO_CENTER, VAL_AUTO_CENTER, 500, 800);

	//Make the Panel movable
	SetPanelAttribute (GUI_Handle, ATTR_MOVABLE, 1);
	//Create Recessed decoration box for Commands
	DEC_BOX_CMDS = NewCtrl (GUI_Handle, CTRL_RECESSED_BOX, "", 180, 75);

	//Set dimensions of Decoration box
	SetCtrlAttribute (GUI_Handle, DEC_BOX_CMDS, ATTR_LEFT, 75); 
	SetCtrlAttribute (GUI_Handle, DEC_BOX_CMDS, ATTR_HEIGHT, 125);
	SetCtrlAttribute (GUI_Handle, DEC_BOX_CMDS, ATTR_WIDTH, 100);
	//Set frame color of Decoration box
	SetCtrlAttribute (GUI_Handle, DEC_BOX_CMDS, ATTR_FRAME_COLOR,
					VAL_OFFWHITE);

	//Create a "START" square command button 
	GUI_START_ID = NewCtrl (GUI_Handle, CTRL_SQUARE_COMMAND_BUTTON, "START\n<F1>",
										  200, 100);
								  
	//Set the width of the command button									  
	SetCtrlAttribute (GUI_Handle, GUI_START_ID, ATTR_WIDTH, 50);
	//Create a callback function for the start button
	InstallCtrlCallback (GUI_Handle, GUI_START_ID,  StartGUICB, 0);
	//Set the command button color
	SetCtrlAttribute (GUI_Handle, GUI_START_ID, ATTR_CMD_BUTTON_COLOR,
				   VAL_DK_GREEN);
	//Set the label color to WHITE
	SetCtrlAttribute (GUI_Handle, GUI_START_ID, ATTR_LABEL_COLOR,
				   VAL_WHITE);
	//Set Fonts to "Meta Fonts"
	SetCtrlAttribute (GUI_Handle, GUI_START_ID, ATTR_LABEL_FONT,
				   VAL_APP_META_FONT);
	//Assign "F1" as the shortcut key to this button
	SetCtrlAttribute (GUI_Handle, GUI_START_ID, ATTR_SHORTCUT_KEY,
				   VAL_F1_VKEY);
			
	//Disable the START command button
	SetCtrlAttribute (GUI_Handle, GUI_START_ID, ATTR_DIMMED, 1);

	//Create a "EXIT" square command button 
	GUI_EXIT_ID = NewCtrl (GUI_Handle, CTRL_SQUARE_COMMAND_BUTTON, "EXIT\n<F10>",
										  250, 100);
	//Set the width of the command button									  
	SetCtrlAttribute (GUI_Handle, GUI_EXIT_ID, ATTR_WIDTH, 50);										  
	//Create a callback function for the EXIT button
	InstallCtrlCallback (GUI_Handle, GUI_EXIT_ID,  ExitGUICB, 0);
	//Set the command button color
	SetCtrlAttribute (GUI_Handle, GUI_EXIT_ID, ATTR_CMD_BUTTON_COLOR,
				   VAL_RED);
	//Set the label color to yellow
	SetCtrlAttribute (GUI_Handle, GUI_EXIT_ID, ATTR_LABEL_COLOR,
				   VAL_YELLOW);
	//Set Fonts to "Meta Fonts"
	SetCtrlAttribute (GUI_Handle, GUI_EXIT_ID, ATTR_LABEL_FONT,
				   VAL_APP_META_FONT);
	//Assign "F10" as shortcut to this button
	SetCtrlAttribute (GUI_Handle, GUI_EXIT_ID, ATTR_SHORTCUT_KEY,
				   VAL_F10_VKEY);

	//Create Decoration box for Functions
	DEC_BOX_FUNC = NewCtrl (GUI_Handle, CTRL_RAISED_BOX, 0, 85, 75);
	
	//Set dimensions of Decoration box
	SetCtrlAttribute (GUI_Handle, DEC_BOX_FUNC, ATTR_LEFT, 65); 
	SetCtrlAttribute (GUI_Handle, DEC_BOX_FUNC, ATTR_HEIGHT, 65);
	SetCtrlAttribute (GUI_Handle, DEC_BOX_FUNC, ATTR_WIDTH, 130);
	//Add Label to Function box and center it over the ring control
	TEXT_MSG_FUNC = NewCtrl (GUI_Handle, CTRL_TEXT_MSG, "Select Function", 90, 85);

	//Create Ring Box.  Label for the control box could be 
	//inserted within quotes if you wish
	RING_BOX = NewCtrl (GUI_Handle, CTRL_RING, "", 110, 75);
	SetCtrlAttribute (GUI_Handle, RING_BOX, ATTR_WIDTH, 110);
	
	//Create a callback function for the Ring button
	InstallCtrlCallback (GUI_Handle, RING_BOX, RingCB, 0);
	//Add the list items
	InsertListItem (GUI_Handle, RING_BOX, 0, "Sine Wave",0);
	InsertListItem (GUI_Handle, RING_BOX, 1, "Uniform Random Numbers",1);  
	InsertListItem (GUI_Handle, RING_BOX, 2, "White Noise",2); 
	
	//Set the Fonts style for the ring control items
	SetCtrlAttribute (GUI_Handle, RING_BOX, ATTR_TEXT_COLOR, VAL_DK_BLUE);
	SetCtrlAttribute (GUI_Handle, RING_BOX, ATTR_TEXT_FONT, VAL_APP_META_FONT);
	
	//Create Raised decoration box for Graph
	DEC_BOX_GRAPH = NewCtrl (GUI_Handle, CTRL_RAISED_BOX, "", 60, 250);

	//Set dimensions of Decoration box
	SetCtrlAttribute (GUI_Handle, DEC_BOX_GRAPH, ATTR_HEIGHT, 375);
	SetCtrlAttribute (GUI_Handle, DEC_BOX_GRAPH, ATTR_WIDTH, 500);
	//Set frame color of Decoration box
	SetCtrlAttribute (GUI_Handle, DEC_BOX_GRAPH, ATTR_FRAME_COLOR,
					VAL_LT_GRAY);

	//Create Graph control
	GRAPH_ID = NewCtrl (GUI_Handle, CTRL_GRAPH, "", 100, 280);
	SetCtrlAttribute (GUI_Handle, GRAPH_ID, ATTR_HEIGHT, 310);
	SetCtrlAttribute (GUI_Handle, GRAPH_ID, ATTR_WIDTH, 450);

	//Create Graph Labels
	SetCtrlAttribute (GUI_Handle, GRAPH_ID, ATTR_XNAME,"Frequency");
	SetCtrlAttribute (GUI_Handle, GRAPH_ID, ATTR_YNAME,"Calculated Data");
	SetCtrlAttribute (GUI_Handle, GRAPH_ID, ATTR_XYNAME_BOLD, 1); 

	SetCtrlAttribute (GUI_Handle, GRAPH_ID, ATTR_XYNAME_COLOR, VAL_BLACK);
	
	//Create a "CLEAR" square command button 
	CLEAR_PLOT_ID = NewCtrl (GUI_Handle, CTRL_SQUARE_COMMAND_BUTTON, "CLEAR\n<F3>",
										  250, 190);
	//Set the width of the command button									  
	SetCtrlAttribute (GUI_Handle, CLEAR_PLOT_ID, ATTR_WIDTH, 40);										  
	//Create a callback function for the CLEAR  button
	InstallCtrlCallback (GUI_Handle, CLEAR_PLOT_ID,  ClearGraphCB, 0);
	//Set the command button color
	SetCtrlAttribute (GUI_Handle, CLEAR_PLOT_ID, ATTR_CMD_BUTTON_COLOR,
					  VAL_GREEN);
	//Set the label color to black
	SetCtrlAttribute (GUI_Handle, CLEAR_PLOT_ID, ATTR_LABEL_COLOR,
					  VAL_BLACK);
	//Set Fonts to "Meta Fonts"
	SetCtrlAttribute (GUI_Handle, CLEAR_PLOT_ID, ATTR_LABEL_FONT,
				   VAL_APP_META_FONT);
	//Assign "F3" as shortcut to this button
	SetCtrlAttribute (GUI_Handle, CLEAR_PLOT_ID, ATTR_SHORTCUT_KEY,
				   VAL_F3_VKEY);
	//Disable the CLEAR command button
	SetCtrlAttribute (GUI_Handle, CLEAR_PLOT_ID, ATTR_DIMMED, 1);
	//Close the panel by clicking on the "X" on the panel box          				   
	SetPanelAttribute (GUI_Handle, ATTR_CLOSE_CTRL, GUI_EXIT_ID);
	
}//CreateGUI

//Callback function assigned to "Ring" control
int CVICALLBACK RingCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			//Enable the START command button
			SetCtrlAttribute (GUI_Handle, GUI_START_ID, ATTR_DIMMED, 0);
			break;
		}
	return 0;
}//RingCB

//Callback function assigned to "START" command button
int CVICALLBACK StartGUICB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

char SelectedLabel[260], buf [500];	
int SelectedIndex,i;

	switch (event)
		{
		case EVENT_COMMIT:
			//Add Label to Graph
			TEXT_MSG = NewCtrl (GUI_Handle, CTRL_TEXT_MSG, "PLOT FOR: ", 70, 450);
			
			//Obtain Ring item index and label
			GetCtrlIndex (GUI_Handle, RING_BOX, &SelectedIndex);
			GetLabelFromIndex(GUI_Handle, RING_BOX, SelectedIndex, SelectedLabel);
			//Add selected Ring item to Graph header
			SELECTED_ITEM = NewCtrl (GUI_Handle, CTRL_TEXT_MSG,SelectedLabel,70, 520);
			switch(SelectedIndex)
			{
				case 0:  //Sine Wave
						CreateDisplayDataPanel();  //Create panel for inputing sine data
						break;

				case 1:	//Uniform Random numbers
						 //Add Label to Graph
						TABLE_MSG = NewCtrl (GUI_Handle, CTRL_TEXT_MSG, "DATA FOR: ", 70, 450);
						//Hide the Graph control
						SetCtrlAttribute (GUI_Handle, GRAPH_ID, ATTR_VISIBLE, 0);
						//Create Text Box control
						TEXT_BOX_ID = NewCtrl (GUI_Handle, CTRL_TEXT_BOX, "", 100, 280);
						//Set the text box control mode
						SetCtrlAttribute (GUI_Handle, TEXT_BOX_ID, ATTR_CTRL_MODE,
										  VAL_INDICATOR);
						//Set Text box dimensions
						SetCtrlAttribute (GUI_Handle, TEXT_BOX_ID, ATTR_HEIGHT, 310);
						SetCtrlAttribute (GUI_Handle, TEXT_BOX_ID, ATTR_WIDTH, 450);
						//Include both scroll bars
						SetCtrlAttribute (GUI_Handle, TEXT_BOX_ID, ATTR_SCROLL_BARS,
										  VAL_BOTH_SCROLL_BARS);
						//Set text background color 
						SetCtrlAttribute (GUI_Handle, TEXT_BOX_ID, ATTR_TEXT_BGCOLOR,
										  VAL_BLACK);
						//Set text color to yellow
						SetCtrlAttribute (GUI_Handle, TEXT_BOX_ID, ATTR_TEXT_COLOR,
										  VAL_YELLOW);
						//Hide the CLEAR button
						SetCtrlAttribute (GUI_Handle, CLEAR_PLOT_ID, ATTR_VISIBLE, 0);
						
						//Create a "PLOT" square command button 
						PLOT_UNI_ID = NewCtrl (GUI_Handle, CTRL_SQUARE_COMMAND_BUTTON,
																"PLOT\n<F4>", 250, 190);
						//Set the width of the command button									  
						SetCtrlAttribute (GUI_Handle, PLOT_UNI_ID, ATTR_WIDTH, 40);										  
						//Create a callback function for the PLOT button
						InstallCtrlCallback (GUI_Handle, PLOT_UNI_ID,  PlotUniformCB, 0);
						//Set the command button color
						SetCtrlAttribute (GUI_Handle, PLOT_UNI_ID, ATTR_CMD_BUTTON_COLOR,
										  									VAL_OFFWHITE);
						//Set the label color to black
						SetCtrlAttribute (GUI_Handle, PLOT_UNI_ID, ATTR_LABEL_COLOR,
										  									VAL_BLACK);
						//Set Fonts to "Meta Fonts"
						SetCtrlAttribute (GUI_Handle, PLOT_UNI_ID, ATTR_LABEL_FONT,
									   VAL_APP_META_FONT);
						//Assign "F4" as shortcut to this button
						SetCtrlAttribute (GUI_Handle, PLOT_UNI_ID, ATTR_SHORTCUT_KEY,
									   VAL_F4_VKEY);
						
						//Create Uniform Random Values
						samples=100;
						seed=20;
						Uniform(samples, seed,Y_axis); 
						//Display data on Text Box
						Fmt(buf,"%s<Samples       Value");
						InsertTextBoxLine(GUI_Handle, TEXT_BOX_ID,-1, buf);  //Display Header
						
						for (i=0;i<samples;i++)
						{   
							
							Fmt(buf,"%s<    %d           %f", i,  Y_axis[i]);
							InsertTextBoxLine(GUI_Handle, TEXT_BOX_ID,-1, buf);
						}
						
						break;
				case 2:  // White Noise
						amp=5.0;
						samples=50;
						seed=15;
						WhiteNoise(samples, amp, seed, Y_axis);
						PlotHandle = PlotY (GUI_Handle, GRAPH_ID, Y_axis, samples, 
											VAL_DOUBLE, VAL_CONNECTED_POINTS, 
											VAL_ASTERISK, VAL_SOLID, 1, VAL_WHITE);
						break;
				default:
						printf("Error in Ring control selection\n");
						break;
		
			} //switch(SelectedIndex)  
			//Enable the CLEAR command button
			SetCtrlAttribute (GUI_Handle, CLEAR_PLOT_ID, ATTR_DIMMED, 0);
			break;
		} 
	return 0;
}//StartGUICB


//Create and display the child panel
void CreateDisplayDataPanel(void)
{
#define SAMPLES_VAL 25
#define	MAX_SAMPLES 1000

int DONE_ID;

	    //Create a child panel
		Child_Handle = NewPanel (GUI_Handle, "SELECT DATA", VAL_AUTO_CENTER,  
								 VAL_AUTO_CENTER,300, 300);

		InstallPopup(Child_Handle);	

		//Create a Numeric control to enter samples
		SAMPLES_ID = NewCtrl (Child_Handle, CTRL_NUMERIC, "Samples", 60, 40);
		//Set data type to integer
		SetCtrlAttribute (Child_Handle, SAMPLES_ID, ATTR_DATA_TYPE, VAL_INTEGER);
						  
		//Set the Control Mode
		SetCtrlAttribute (Child_Handle, SAMPLES_ID, ATTR_CTRL_MODE, VAL_HOT);
		//Set number of default samples
		SetCtrlAttribute (Child_Handle, SAMPLES_ID, ATTR_DFLT_VALUE, SAMPLES_VAL);
		SetCtrlVal(Child_Handle, SAMPLES_ID, SAMPLES_VAL);	 
		//Specify minimum value of control
		SetCtrlAttribute (Child_Handle, SAMPLES_ID, ATTR_MIN_VALUE, 1);
		//Specify maximum value of control 
		SetCtrlAttribute (Child_Handle, SAMPLES_ID, ATTR_MAX_VALUE, MAX_SAMPLES);
		InstallCtrlCallback (Child_Handle, SAMPLES_ID,  SamplesControlCB, 0); 
		
		//Create a Numeric control to enter amplitude
		AMPLITUDE_ID = NewCtrl (Child_Handle, CTRL_NUMERIC, "Amplitude", 60, 150);
		//Set data type to double
		SetCtrlAttribute (Child_Handle, AMPLITUDE_ID, ATTR_DATA_TYPE, VAL_DOUBLE);
						  
		//Set the Control Mode
		SetCtrlAttribute (Child_Handle, AMPLITUDE_ID, ATTR_CTRL_MODE, VAL_HOT);
		//Set default amplitude
		SetCtrlAttribute (Child_Handle, AMPLITUDE_ID, ATTR_DFLT_VALUE, 5.0);
		SetCtrlVal(Child_Handle, AMPLITUDE_ID, 5.0);
		//Specify minimum value of control
		SetCtrlAttribute (Child_Handle, AMPLITUDE_ID, ATTR_MIN_VALUE, 0.0);
		
		//Create a Slider control to enter phase value
		PHASE_ID = NewCtrl (Child_Handle, CTRL_NUMERIC_POINTER_HSLIDE,
															"Phase", 170, 40);
		//Set data type
		SetCtrlAttribute (Child_Handle, PHASE_ID, ATTR_DATA_TYPE, VAL_DOUBLE);
						  
		//Set the Control Mode
		SetCtrlAttribute (Child_Handle, PHASE_ID, ATTR_CTRL_MODE, VAL_HOT);
		//Set default 
		SetCtrlAttribute (Child_Handle, PHASE_ID, ATTR_DFLT_VALUE, 0.0);
		DefaultCtrl (Child_Handle, PHASE_ID);
		//Specify minimum value of control
		SetCtrlAttribute (Child_Handle, PHASE_ID, ATTR_MIN_VALUE, 0.0);
		//Specify maximum value of control
		SetCtrlAttribute (Child_Handle, PHASE_ID, ATTR_MAX_VALUE, 180.0);
		
		//Create a Knob control to enter cyles value
		CYCLES_ID = NewCtrl (Child_Handle, CTRL_NUMERIC_KNOB, "Cycles", 140, 200);  
		//Set data type
		SetCtrlAttribute (Child_Handle, CYCLES_ID, ATTR_DATA_TYPE, VAL_INTEGER);
						  
		//Set the Control Mode
		SetCtrlAttribute (Child_Handle, CYCLES_ID, ATTR_CTRL_MODE, VAL_HOT);
		 
		//Set default cycles to 2
		SetCtrlAttribute (Child_Handle, CYCLES_ID, ATTR_DFLT_VALUE, 2);
		DefaultCtrl (Child_Handle, CYCLES_ID);
		//Specify minimum value of control
		SetCtrlAttribute (Child_Handle, CYCLES_ID, ATTR_MIN_VALUE, 1);
		//Specify maximum value of control
		SetCtrlAttribute (Child_Handle, CYCLES_ID, ATTR_MAX_VALUE, SAMPLES_VAL);
		//Create a "DONE" square command button 
		DONE_ID = NewCtrl (Child_Handle, CTRL_SQUARE_COMMAND_BUTTON, "DONE",
						  											 250, 150);
		//Set the width of the command button									  
		SetCtrlAttribute (Child_Handle, DONE_ID, ATTR_WIDTH, 30);										  
		//Create a callback function for the DONE button
		InstallCtrlCallback (Child_Handle, DONE_ID,  DoneGUICB, 0);
		//Set the command button color
		SetCtrlAttribute (Child_Handle, DONE_ID, ATTR_CMD_BUTTON_COLOR,
						  										VAL_DK_GREEN);
		//Set the label color
		SetCtrlAttribute (Child_Handle, DONE_ID, ATTR_LABEL_COLOR, VAL_WHITE);
		//Set Fonts to "Meta Fonts"
		SetCtrlAttribute (Child_Handle, DONE_ID, ATTR_LABEL_FONT, VAL_APP_META_FONT);
}//CreateDisplayDataPanel

//Set the Cycles control based on Samples selected
int CVICALLBACK SamplesControlCB(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
		
{
	switch (event)
		{
		case EVENT_COMMIT:
		   //Get the samples value 
			GetCtrlVal(Child_Handle, SAMPLES_ID, &samples);
		 	//Cycles must be less than samples
		 	GetCtrlAttribute (Child_Handle, SAMPLES_ID, ATTR_CTRL_VAL, &samples);
		 	//Set Maximum value of Cycles control to samples value
		  	SetCtrlAttribute (Child_Handle, CYCLES_ID, ATTR_MAX_VALUE, samples);
		  	//Set Cycles to 2
		  	// Ofcourse, you can select the cycles by changing the value on the 
		  	// user interface 
			SetCtrlVal(Child_Handle, CYCLES_ID,2); 
		break;
   }
	return 0;
}//SamplesControlCB			
		
//Done entering data
int CVICALLBACK DoneGUICB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
		
{
	switch (event)
		{
		case EVENT_COMMIT:
			//Obtain the values of the control
	    	GetCtrlAttribute (Child_Handle, SAMPLES_ID, ATTR_CTRL_VAL, &samples);
			GetCtrlAttribute (Child_Handle, AMPLITUDE_ID, ATTR_CTRL_VAL, &amp);
			GetCtrlAttribute (Child_Handle, PHASE_ID, ATTR_CTRL_VAL, &phase);
			GetCtrlAttribute (Child_Handle, CYCLES_ID, ATTR_CTRL_VAL, &cycles);
			
			SinePattern (samples, amp, phase,cycles, Y_axis);//Create sine wave
			DiscardPanel(Child_Handle);
			//Plot Sine Pattern with the select parameters
			PlotHandle = PlotY (GUI_Handle, GRAPH_ID, Y_axis, samples, VAL_DOUBLE,
								VAL_THIN_LINE, VAL_SIMPLE_DOT, VAL_SOLID, 1,
								VAL_BLACK);
			break;
		}
	return 0;
}//DoneGUICB	

//Delete the last plot
int CVICALLBACK ClearGraphCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
		
{
	switch (event)
		{
		case EVENT_COMMIT:
			status = DeleteGraphPlot (GUI_Handle, GRAPH_ID, -1, VAL_IMMEDIATE_DRAW);
			//Hide the message boxes
			SetCtrlAttribute (GUI_Handle, TEXT_MSG, ATTR_VISIBLE, 0);
			SetCtrlAttribute (GUI_Handle, SELECTED_ITEM, ATTR_VISIBLE, 0); 
			break;
		}
	return 0;
}//ClearGraphCB 	

//Invoked from PLOT command button
int CVICALLBACK PlotUniformCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
		
{
	switch (event)
		{
		case EVENT_COMMIT:
			//Display the CLEAR button
			SetCtrlAttribute (GUI_Handle, CLEAR_PLOT_ID, ATTR_VISIBLE, 1);
			//Delete the PLOT button
			DiscardCtrl (GUI_Handle, PLOT_UNI_ID);
			//Delete the Label on Graph
			DiscardCtrl (GUI_Handle, TABLE_MSG);
			//Make Graph visible
			SetCtrlAttribute (GUI_Handle, GRAPH_ID, ATTR_VISIBLE, 1);
			//Make Text Box invisible
			SetCtrlAttribute (GUI_Handle, TEXT_BOX_ID, ATTR_VISIBLE, 0);
			//Plot the Uniform random numbers 
			samples=100;
			seed=20;
			Uniform(samples, seed,Y_axis); 
			PlotHandle = PlotY (GUI_Handle, GRAPH_ID, Y_axis, samples, VAL_DOUBLE,
										VAL_CONNECTED_POINTS, VAL_SMALL_X, VAL_SOLID, 1,
																			VAL_DK_RED);
			break;
		}
	return 0;
}//PlotUniformCB		


//Callback function assigned to "EXIT" command button
int CVICALLBACK ExitGUICB(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			QuitUserInterface (0);
			break;
		}
	return 0;
}//ExitGUICB
