/*****************************************************************************

Purpose: This project demonstrates creating and manipulating graph legends 
		 using the legend instrument driver "legend.fp".
		 
Project Name: project2-2.prj
File Name:    project2-2.c

/**************************Include files*************************************/

#include <cvirte.h> 
#include <userint.h>
#include <ansi_c.h>
#include <analysis.h>
#include "project2-2.h"
#include "legend.h" 

/***************** Function prototype **************************************/ 
                                                    
void DisplayCursorPosition(void); 

/***************************************************************************/
static int GraphPanelHandle; 
double X_StartMin,X_StartMax,Y_StartMin, Y_StartMax;
static int counter=0; 
int PlotLegend, SinePlot,OffsetPlot, Offset30Plot, Offset60Plot, NumberOfItems,tankLG, 
    HeaderLegend, ScalingMode;
 
//Main function
int main (int argc, char *argv[])
{
  
    if (InitCVIRTE (0, argv, 0) == 0)   
        return -1;
    //Load panel and obtain panel handle
    if ((GraphPanelHandle =LoadPanel (0, "project2-2.uir", CURSORS)) < 0)
        return -1;
     
    //Display the panel
    DisplayPanel (GraphPanelHandle);
   
    //Store the Starting Axis values for display before any zoom
    GetAxisScalingMode (GraphPanelHandle, CURSORS_GRAPH, VAL_XAXIS, &ScalingMode,
							&X_StartMin, &X_StartMax);
							
	GetAxisScalingMode (GraphPanelHandle, CURSORS_GRAPH, VAL_LEFT_YAXIS, &ScalingMode,
							&Y_StartMin, &Y_StartMax);
    
    RunUserInterface();

    //Discard panel and release resources
    DiscardPanel (GraphPanelHandle);
    CloseCVIRTE ();
 
    return 0;
} //main

//Plot SinePattern wave.  
int CVICALLBACK PlotData (int panel, int control, int event,
                          void *callbackData, int eventData1, int eventData2)
{
    double Y_axis[100],  Offset30Data[100], Offset60Data[100];
    

    if (event == EVENT_COMMIT)
    {
        //Clear the graph
        DeleteGraphPlot (panel, CURSORS_GRAPH, -1, 1);
        
         //Create the Sine Pattern array in Y_axis
		SinePattern (100, 10.0, 0.0, 5, Y_axis);
		//Plot the graph for 100 points for SinePattern
		SinePlot = PlotY (panel, CURSORS_GRAPH, Y_axis, 100, VAL_DOUBLE,
						  VAL_CONNECTED_POINTS, VAL_EMPTY_DIAMOND, VAL_SOLID,
						  1, VAL_DK_BLUE);
	
		//Create another SinePattern with -30 degrees phase offset
		SinePattern (100, 10.0, -30.0, 5, Offset30Data);
		
        //Plot the graph for 100 points for SinePattern using 30 degrees offset phase
		Offset30Plot=PlotY (panel, CURSORS_GRAPH, Offset30Data, 100, VAL_DOUBLE,
			   VAL_CONNECTED_POINTS, VAL_SOLID_CIRCLE, VAL_SOLID, 1,
			   												VAL_MAGENTA);
			   												
		//Create another SinePattern with -60 degrees phase offset
		SinePattern (100, 10.0, -60.0, 5, Offset60Data);
		
        //Plot the graph for 100 points for SinePattern using 60 degrees offset phase
		Offset60Plot = PlotY (panel, CURSORS_GRAPH, Offset60Data, 100,
							  VAL_DOUBLE, VAL_CONNECTED_POINTS,
							  VAL_EMPTY_CIRCLE, VAL_SOLID, 1, VAL_DK_GREEN);
							  
		//Enable the ADD command button
		SetInputMode (panel, CURSORS_ADD, 1);

      }
        
    return 0;
} //PlotData

//Invoked by the ADD ALL button to create the Legend
int CVICALLBACK AddLegendCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	switch (event)
	{
		case EVENT_COMMIT:
			//Create Legend control
			LGCreateLegendControl (panel, CURSORS_GRAPH, LG_POS_BELOW_CENTER, 1,
								   1, &PlotLegend);
			//Move the legend control down 25 pixels from the anchor position
			LGSetLegendCtrlAttribute (panel, PlotLegend,
									  LG_ATTR_OFFSET_FROM_ANCHOR, 25);
			//Make the Legend control border visible
			SetCtrlAttribute(panel, PlotLegend, ATTR_BORDER_VISIBLE,1);
			//Automatically re-size the legend box ad items are added or deleted
			LGSetLegendCtrlAttribute (panel, PlotLegend, LG_ATTR_AUTO_SIZE, 1);
			//Legend item for Sine Plot with no phase offset				   
			LGInsertLegendItemForPlot (panel, PlotLegend,1,
									   "Phase Offset:  0 degrees", VAL_BLACK,
									   SinePlot);
			//Legend item for Sine plot with -30 degrees phase offset					   
			LGInsertLegendItemForPlot (panel,PlotLegend , 0, "Phase Offset:-30 degrees",
								   VAL_BLACK, Offset30Plot);					   
			//Legend item for Sine plot with -60 degrees phase offset
			LGInsertLegendItemForPlot (panel, PlotLegend,0, "Phase Offset:-60 degrees",
								   VAL_BLACK, Offset60Plot);
			//Set background color the Legend control					   
			LGSetLegendCtrlAttribute (panel, PlotLegend, LG_ATTR_PLOT_BG_COLOR,
									  VAL_CYAN);
			//Change the Fonts in the Legend box
			LGSetLegendCtrlAttribute (panel, PlotLegend, LG_ATTR_META_FONT,VAL_SYSTEM_META_FONT);

			//Enable the INSERT and DELETE buttons
			SetInputMode (panel, CURSORS_INSERT_ITEM, 1);
			SetInputMode (panel, CURSORS_DELETE_ITEM, 1);
			SetInputMode (panel, CURSORS_ADD, 0);//Disable ADD command button
			SetInputMode (panel, CURSORS_REMOVE, 1);//Enable REMOVE command button
			break;
	}
	return 0;
} //AddLegendCB

  
//Callback Function to insert legend items when INSERT command button is selected
int CVICALLBACK InsertLegendItemCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
   char buf[260];
   
	switch (event)
	{
		case EVENT_COMMIT:
			//Increment the counter for inserted item
			sprintf(buf, "Insert Legend Item %d", counter);
			counter++;
			//Insert a generic legend item 				   
			LGInsertLegendItemForPlot (panel, PlotLegend, 0, buf,
								   					VAL_RED, SinePlot);
		break;
	}
	return 0;
}//InsertLegendItemCB

//Callback Function to remove legend items when REMOVE is selected
int CVICALLBACK DeleteLegendItemCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			counter --;   //Decrement the counter for inserted item
			//Delete legend item 
			LGDeleteLegendItem(panel, PlotLegend, 0); 
			break;
	}
	return 0;
} //DeleteLegendItemCB

//Invoked by the REMOVE button to remove the Legends
int CVICALLBACK RemoveLegendCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			//Clear the legend items
			LGClearLegendCtrl (panel, PlotLegend);
			//Hide the Legend control box
			SetCtrlAttribute(panel, PlotLegend,ATTR_VISIBLE,0);
			//Disable the INSERT and DELETE buttons
			SetInputMode (panel, CURSORS_INSERT_ITEM, 0);
			SetInputMode (panel, CURSORS_DELETE_ITEM, 0);
			//Enable ADD command button
			SetInputMode (panel, CURSORS_ADD, 1); 
			counter=0;
			break;
	}
	return 0;
}//RemoveLegendCB

//Update the marker position 
int CVICALLBACK ContinuousUpdate (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_TIMER_TICK:
			//Update the marker values on the panel on every Timer tick
        	DisplayCursorPosition();  
			break;
	}
	return 0;
}//ContinuousUpdate

//Invoked when the zoom ring is selected
int CVICALLBACK SelectZoomCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	#define FirstCursor 1
	#define SecondCursor 3

    double X_Pos_First_Cursor;
    double X_Pos_Second_Cursor;
    double Y_Pos_First_Cursor;
    double Y_Pos_Second_Cursor;
    double temp;
    
    double ZoomValue, ZoomValue_X1, ZoomValue_X2, ZoomValue_Y1,ZoomValue_Y2, 
    X_axisMin, X_axisMax, Y_axisMin, Y_axisMax;
      
    int ZoomIndex;
    
    if (event == EVENT_COMMIT)
    {
       
        //Get the current position of both cursors
        GetGraphCursor (panel, CURSORS_GRAPH, FirstCursor, &X_Pos_First_Cursor, &Y_Pos_First_Cursor);
        GetGraphCursor (panel, CURSORS_GRAPH, SecondCursor, &X_Pos_Second_Cursor, &Y_Pos_Second_Cursor);
        
        // Swap values to obtain positive range values
        if (X_Pos_First_Cursor > X_Pos_Second_Cursor)
        {
            temp = X_Pos_First_Cursor;
            X_Pos_First_Cursor = X_Pos_Second_Cursor;
            X_Pos_Second_Cursor = temp;
        }
        if (Y_Pos_First_Cursor > Y_Pos_Second_Cursor)
        {
            temp = Y_Pos_First_Cursor;
            Y_Pos_First_Cursor = Y_Pos_Second_Cursor;
            Y_Pos_Second_Cursor = temp;
        }

/*********Here the scaling of the graph control is shown using the ring control selection*******/
		//Obtain the selected zoom value index
		GetCtrlIndex (panel, CURSORS_ZOOM_RING, &ZoomIndex);
		//Obtain the selected zoom value from the index  
		GetValueFromIndex (panel, CURSORS_ZOOM_RING, ZoomIndex, &ZoomValue);
		//Obtain the current X axis minimum and maximum values
		GetAxisScalingMode (panel, CURSORS_GRAPH, VAL_XAXIS, &ScalingMode,
							&X_axisMin, &X_axisMax);
		//Obtain the current Y axis minimum and maximum values					
		GetAxisScalingMode (panel, CURSORS_GRAPH, VAL_LEFT_YAXIS, &ScalingMode,
							&Y_axisMin, &Y_axisMax);					
		
		//Use the panel min and maximum values for normal mode (100%) scaling in main function
		//Ratio the graph control using the zoom factor selected
		ZoomValue_X1= (X_StartMin/ZoomValue);
		ZoomValue_X2= (X_StartMax/ZoomValue); 
		
		ZoomValue_Y1= (Y_StartMin/ZoomValue);
		ZoomValue_Y2= (Y_StartMax/ZoomValue);
		
		 //Set the X and Y axis to the new zoom values
		SetAxisScalingMode (panel, CURSORS_GRAPH, VAL_XAXIS, VAL_MANUAL, ZoomValue_X1, ZoomValue_X2);
		SetAxisScalingMode (panel, CURSORS_GRAPH, VAL_LEFT_YAXIS, VAL_MANUAL, ZoomValue_Y1, ZoomValue_Y2); 
		//Update the marker values on the panel
		DisplayCursorPosition();  
		
     }
    return 0;
} //SelectZoomCB

//Get cursor position and display on the GUI
void DisplayCursorPosition(void)
{
#define Marker 2

	double X_Marker_Position;
    double Y_Marker_Position;
    int    plothandle;
    int    index;

    //Get Marker's position 
    GetGraphCursor (GraphPanelHandle, CURSORS_GRAPH, Marker, &X_Marker_Position, &Y_Marker_Position);
        
    //Obtain graph handle and array index of the point to which the marker is attached
    GetGraphCursorIndex (GraphPanelHandle, CURSORS_GRAPH, 2, &plothandle, &index);
        
    //Update the controls on the panel
    SetCtrlVal (GraphPanelHandle, CURSORS_XREADOUT, X_Marker_Position);
    SetCtrlVal (GraphPanelHandle, CURSORS_YREADOUT, Y_Marker_Position);
    SetCtrlVal (GraphPanelHandle, CURSORS_INDEX, index);
}  //DisplayCursorPosition

//Exit the project
int CVICALLBACK Shutdown (int panel, int control, int event,
                          void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface (0);
        
    return 0;
} //Shutdown





