#ifndef LEGEND_INCLUDE
#define LEGEND_INCLUDE 1

#ifdef __cplusplus
    extern "C" {
#endif

/* Cannot be used with CVI 3.0 or CVI 3.1 */
#if defined(_CVI_) && ((_CVI_==1) || (_CVI_==301))
    #error The Legend Instrument Driver must be used with CVI version 3.1 or greater
#endif

/* Enable runtime checking for CVI */
#ifdef _CVI_
#pragma EnableLibraryRuntimeChecking
#endif

/* Make all possible predefined macros agree */
#if (!defined(_NI_mswin16_)) && (defined(_NI_mswin32_)||defined(__WIN32__)||defined(__NT__)||defined(_WIN32)||defined(WIN32))
    #ifndef _NI_mswin32_
        #define _NI_mswin32_ 1
    #endif
#endif

/* Define LEGENDAPI for Windows 95 and Windows NT exporting of functions */
#ifdef _NI_mswin32_
    #define LEGENDAPI __stdcall
#else     
    #define LEGENDAPI
#endif    

/* Define LEGENDAPI_CDECL for Windows 95 and Windows NT exporting of    */
/* functions with a variable number of arguments                        */
#ifndef LEGENDAPI_CDECL
    #ifdef _NI_mswin16_
        #define LEGENDAPI_CDECL
    #else
        #if defined(__WATCOMC__) || defined(_NI_WC_)
            #define LEGENDAPI_CDECL
        #else
            #define LEGENDAPI_CDECL __cdecl
        #endif
    #endif
#endif

/* This may be used in CVI 3.1, define CVICALLBACK for example programs to work */
#ifndef _NI_mswin32_
    #if (!defined(_CVI_)) || (defined(_CVI_) && (_CVI_==310))
        #ifndef CVICALLBACK
            #define CVICALLBACK
        #endif
    #endif    
#endif



#define LG_SUCCESS 0
#define LG_ERR_INVALID_PANEL -15000
#define LG_ERR_INVALID_CTRL -15001
#define LG_ERR_OUT_OF_MEM -15002
#define LG_ERR_INVALID_POSITION -15003
#define LG_ERR_INVALID_LEGEND_NUMBER -15004
#define LG_ERR_INVALID_GRAPH_CONTROL -15005
#define LG_ERR_MUST_RETAIN_DATA -15006
#define LG_ERR_CANNOT_GET_LEGEND_DATA -15007
#define LG_ERR_UNEXPECTED_NULL_PARAMETER -15008

#define LG_POS_DONT_MOVE 0
#define LG_POS_ABOVE_LEFT 1
#define LG_POS_ABOVE_CENTER 2
#define LG_POS_ABOVE_RIGHT 3
#define LG_POS_RIGHT_TOP 4
#define LG_POS_RIGHT_CENTER 5
#define LG_POS_RIGHT_BOTTOM 6
#define LG_POS_BELOW_RIGHT 7
#define LG_POS_BELOW_CENTER 8
#define LG_POS_BELOW_LEFT 9
#define LG_POS_LEFT_BOTTOM 10
#define LG_POS_LEFT_CENTER 11
#define LG_POS_LEFT_TOP 12

/* legend control attributes */
#define LG_ATTR_META_FONT 1
#define LG_ATTR_META_FONT_LEN 2
#define LG_ATTR_AUTO_SIZE 3
#define LG_ATTR_OFFSET_FROM_ANCHOR 4
#define LG_ATTR_ANCHOR_CTRL 5
#define LG_ATTR_SHOW_SAMPLES 6
#define LG_ATTR_REL_POS 7
#define LG_ATTR_GRAPH_BG_COLOR 8
#define LG_ATTR_PLOT_BG_COLOR 9
#define LG_ATTR_DELAY_UPDATE 10
/* legend attributes */
#define LG_ATTR_LEGEND_TEXT 11
#define LG_ATTR_LEGEND_TEXT_LEN 12
#define LG_ATTR_TEXT_COLOR 13
#define LG_ATTR_PLOT_STYLE 14
#define LG_ATTR_POINT_STYLE 15
#define LG_ATTR_LINE_STYLE 16
#define LG_ATTR_PLOT_COLOR 17

extern int LEGENDAPI LGCreateLegendControl (int panelHandle, int anchorControl, int relativePosition,
        int autoSize, int showSamples, int *legendControl);
extern int LEGENDAPI LGConvertGraphToLegend (int panelHandle, int legendControl, int anchorControl,
        int relativePosition, int autoSize, int showSamples);
extern int LEGENDAPI LGInsertLegendItemForPlot (int pan, int legendCtrl, int insertPosition, 
        char legendText[], int textColor, int plotHndl);
extern int LEGENDAPI LGInsertLegendItem (int panelHandle, int legendControl, int insertPosition,
        char legendText[], int textColor, int plotStyle, int pointStyle,
        int lineStyle, int plotColor);
extern int LEGENDAPI LGDeleteLegendItem (int panelHandle, int legendControl, int deletePosition);
#include <stdarg.h> /* for variable arg stuff */
extern int LEGENDAPI LGClearLegendCtrl (int pan, int legendCtrl);
extern int LEGENDAPI LGNumberOfLegendItems (int pan, int legendCtrl, int *numItems);

extern int LEGENDAPI LGDisplayLegendItems (int panelHandle, int legendControl);
extern int LEGENDAPI_CDECL LGSetLegendCtrlAttribute(int pan, int legendCtrl, int attribute, ...);
extern int LEGENDAPI LGGetLegendCtrlAttribute(int pan, int legendCtrl, int attribute,
        void *value);
extern int LGSetLegendItemAttribute(int pan, int legendCtrl, int legendNumber, 
        int attribute, ...);
extern int LEGENDAPI LGGetLegendItemAttribute(int pan, int legendCtrl, int legendNumber,
        int attribute, void *value);
extern char * LEGENDAPI LGGetErrorText(int errCode);

/* required for user protection */
extern int LEGENDAPI LGSetLegendItemAttributeFromParmInfo(int pan, int legendCtrl, 
        int legendNumber, int attribute, va_list parmInfo);
extern int LEGENDAPI LGSetLegendCtrlAttributeFromParmInfo(int pan, int legendCtrl, 
        int attribute, va_list argList);

#ifdef __cplusplus
    }
#endif

#endif /* #ifndef LEGEND_INCLUDE */
