/*****************************************************************************

Purpose: This project demonstrates the use of graph cursors.  It uses a marker
		 allowing you to obtain the coordinates of the marker position as it 
		 is moved on the plotted graph.  This project also shows you how to 
		 zoom the visible area of the graph using various scaling factors.

Project Name:  project 2-1.prj 
File Name:     project 2-1.c

/**************************Include files*************************************/

#include <cvirte.h> 
#include <userint.h>
#include <ansi_c.h>
#include <analysis.h>
#include "project2-1.h"

/***************** Function prototype **************************************/ 
                                                    
void DisplayCursorPosition(void); 

/***************************************************************************/
static int GraphPanelHandle; 
double X_StartMin,X_StartMax,Y_StartMin, Y_StartMax;

//Main function
int main (int argc, char *argv[])
{

    if (InitCVIRTE (0, argv, 0) == 0)   
        return -1;
    //Load panel and obtain panel handle
    if ((GraphPanelHandle =LoadPanel (0, "project2-1.uir", CURSORS)) < 0)
        return -1;
     
    //Display the panel
    DisplayPanel (GraphPanelHandle);
   
    //Store the Starting Axis values for display before any zoom
    GetAxisScalingMode (GraphPanelHandle, CURSORS_GRAPH, VAL_XAXIS, VAL_MANUAL,
														&X_StartMin, &X_StartMax);
							
	GetAxisScalingMode (GraphPanelHandle, CURSORS_GRAPH, VAL_LEFT_YAXIS, VAL_MANUAL,
															&Y_StartMin, &Y_StartMax);
    RunUserInterface();

    //Discard panel and release resources
    DiscardPanel (GraphPanelHandle);
    CloseCVIRTE ();
 
    return 0;
} //main

//Plot SinePattern wave.  
int CVICALLBACK PlotData (int panel, int control, int event,
                          void *callbackData, int eventData1, int eventData2)
{
    double Y_axis[100];

    if (event == EVENT_COMMIT)
    {
        //Clear the graph
		DeleteGraphPlot (panel, CURSORS_GRAPH, -1, VAL_DELAYED_DRAW);
        
         //Create the Sine Pattern array in Y_axis
		SinePattern (100, 10.0, 0.0, 5, Y_axis);
		//Plot the graph for 100 points
        PlotY (panel, CURSORS_GRAPH, Y_axis, 100, VAL_DOUBLE, VAL_THIN_LINE,
               						VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
    }
        
    return 0;
} //PlotData


//Calls the routine to update graph cursors
int CVICALLBACK UpdateMarker (int panel, int control, int event,  
						void *callbackData, int eventData1, int eventData2)
{
   
    if (event == EVENT_COMMIT)
    {
		//Update the marker values on the panel
        DisplayCursorPosition();  
    }
    return 0;
}//UpdateMarker


//Invoked when the zoom ring is selected
int CVICALLBACK SelectZoomCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
#define FirstCursor 1
#define SecondCursor 3

    double X_Pos_First_Cursor;
    double X_Pos_Second_Cursor;
    double Y_Pos_First_Cursor;
    double Y_Pos_Second_Cursor;
    double temp;
    
    double ZoomValue, ZoomValue_X1, ZoomValue_X2, ZoomValue_Y1,ZoomValue_Y2, 
    	   X_axisMin, X_axisMax, Y_axisMin, Y_axisMax;
      
    int ZoomIndex, ScalingMode;
    
    if (event == EVENT_COMMIT)
    {
        //Get the current position of both cursors
        GetGraphCursor (panel, CURSORS_GRAPH, FirstCursor,  &X_Pos_First_Cursor, 
        														&Y_Pos_First_Cursor);
        GetGraphCursor (panel, CURSORS_GRAPH, SecondCursor, &X_Pos_Second_Cursor, 
        														&Y_Pos_Second_Cursor);
        
        // Swap values to obtain positive range values
        if (X_Pos_First_Cursor > X_Pos_Second_Cursor)
        {
            temp = X_Pos_First_Cursor;
            X_Pos_First_Cursor = X_Pos_Second_Cursor;
            X_Pos_Second_Cursor = temp;
        }
        if (Y_Pos_First_Cursor > Y_Pos_Second_Cursor)
        {
            temp = Y_Pos_First_Cursor;
            Y_Pos_First_Cursor = Y_Pos_Second_Cursor;
            Y_Pos_Second_Cursor = temp;
        }

/**Here the scaling of the graph control is shown using the ring control selection***/
		//Obtain the selected zoom value index
		GetCtrlIndex (panel, CURSORS_ZOOM_RING, &ZoomIndex);
		//Obtain the selected zoom value from the index  
		GetValueFromIndex (panel, CURSORS_ZOOM_RING, ZoomIndex, &ZoomValue);
		//Obtain the current X axis minimum and maximum values
		GetAxisScalingMode (panel, CURSORS_GRAPH, VAL_XAXIS, &ScalingMode,
												&X_axisMin, &X_axisMax);
		//Obtain the current Y axis minimum and maximum values					
		GetAxisScalingMode (panel, CURSORS_GRAPH, VAL_LEFT_YAXIS, &ScalingMode,
													&Y_axisMin, &Y_axisMax);					
		
		//Use the panel min and maximum values for normal mode (100%) scaling 
		//in main function
		//Ratio the graph control using the zoom factor selected
		ZoomValue_X1= (X_StartMin/ZoomValue);
		ZoomValue_X2= (X_StartMax/ZoomValue); 
		
		ZoomValue_Y1= (Y_StartMin/ZoomValue);
		ZoomValue_Y2= (Y_StartMax/ZoomValue);
		
		//Set the X and Y axis to the new zoom values
		SetAxisScalingMode (panel, CURSORS_GRAPH, VAL_XAXIS, VAL_MANUAL, 
													ZoomValue_X1, ZoomValue_X2);
		SetAxisScalingMode (panel, CURSORS_GRAPH, VAL_LEFT_YAXIS, VAL_MANUAL, 
													ZoomValue_Y1, ZoomValue_Y2); 
		//Update the marker values on the panel
		DisplayCursorPosition();  
     }
    return 0;
} //SelectZoomCB

//Get cursor position and display on the GUI
void DisplayCursorPosition(void)
{
	#define Marker 2

	double X_Marker_Position;
    double Y_Marker_Position;
    int    plothandle;
    int    index;

    //Get Marker's position 
    GetGraphCursor (GraphPanelHandle, CURSORS_GRAPH, Marker, &X_Marker_Position, 
    															&Y_Marker_Position);
        
    //Obtain plot handle and array index of the point to 
    //which the marker is attached
    GetGraphCursorIndex (GraphPanelHandle, CURSORS_GRAPH, Marker, &plothandle, &index);
        
    //Update the controls on the panel
    SetCtrlVal (GraphPanelHandle, CURSORS_XREADOUT, X_Marker_Position);
    SetCtrlVal (GraphPanelHandle, CURSORS_YREADOUT, Y_Marker_Position);
    SetCtrlVal (GraphPanelHandle, CURSORS_INDEX, index);
}  //DisplayCursorPosition

//Exit the project
int CVICALLBACK Shutdown (int panel, int control, int event,
                          void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface (0);
    return 0;
}//Shutdown
