/****************************************************************

Purpose: This project demonstrates the use of plotting various
		 geometric shapes on the graph control and annotating them 
		 with text. 

Project Name:   project 2-3.prj
File Name:  	project 2-3.c

****************************************************************/

/*******************Include files***************************/
#include <formatio.h> 
#include <cvirte.h> 
#include <userint.h>
#include <ansi_c.h>
#include <analysis.h>
#include "project2-3.h"
#include "TextEntry.h"
#include "angleselect.h"

/***************** Function prototype **********************/ 
                                                    
void DisplayCursorPosition(void); 

/**********************************************************/
static int GraphPanelHandle, ArcPanelHandle, TextPanelHandle; 
double X_StartMin,X_StartMax,Y_StartMin, Y_StartMax;
static double X1_Position, X2_Position;
static double Y1_Position, Y2_Position;
int SelectedColor, FillColor, StartPlotFlag=0;

#define Marker 1

//Main function
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)   
        return -1;
    //Load panel and obtain panel handle
    if ((GraphPanelHandle =LoadPanel (0, "project2-3.uir", PLOTS)) < 0)
        return -1;
    //Display the panel
    DisplayPanel (GraphPanelHandle);
    RunUserInterface();
    //Discard panel and release resources
    DiscardPanel (GraphPanelHandle);
    CloseCVIRTE ();
    return 0;
} //main

//Plot SinePattern wave  
int CVICALLBACK PlotData (int panel, int control, int event,
                          void *callbackData, int eventData1, int eventData2)
{
    double Y_axis[100];
    int SinePlot;

    if (event == EVENT_COMMIT)
    {
        //Create the Sine Pattern array in Y_axis
		SinePattern (100, 10.0, 0.0, 2, Y_axis);
		//Plot the graph for 100 points for SinePattern
		SinePlot = PlotY (panel, PLOTS_GRAPH, Y_axis, 100, VAL_DOUBLE,
						  VAL_CONNECTED_POINTS, VAL_NO_POINT, VAL_SOLID, 1,
						  										VAL_BLACK);
    }
    return 0;
} //PlotData

//Calls the routine to plot selected shapes on graph control
int CVICALLBACK UpdateMarkerCB (int panel, int control, int event,
                              void *callbackData, int eventData1,
                              int eventData2)
{
    int    Fill;
    int    PlotIndex;

   	if (event == EVENT_RIGHT_CLICK) 
    {
		GetCtrlIndex (GraphPanelHandle, PLOTS_PLOT_RING, &PlotIndex);
		GetCtrlVal(GraphPanelHandle,PLOTS_COLOR_SELECT,&SelectedColor);
		GetCtrlVal(GraphPanelHandle,PLOTS_FILL_COLOR,&Fill);
		
		if (!Fill) //Do you want to fill with selected color?
			FillColor =VAL_TRANSPARENT;
		else
			 FillColor= SelectedColor;
		//Get the starting position of the marker when Text is not selected
		if ( (StartPlotFlag==0) && ( PlotIndex !=4))
		{
    		GetGraphCursor (GraphPanelHandle, PLOTS_GRAPH, Marker, 
    	            				&X1_Position, &Y1_Position);
    		StartPlotFlag=1;
    	}
    	else 
    	{
    	   //Get Marker's ending position 
    		GetGraphCursor (GraphPanelHandle, PLOTS_GRAPH, Marker, 
    	                                 &X2_Position, &Y2_Position);
	
			switch (PlotIndex)
			{
			 case 0:  //Draw a Line
	   			PlotLine (GraphPanelHandle, PLOTS_GRAPH, X1_Position, Y1_Position,
				  						   X2_Position, Y2_Position, SelectedColor);
				break; 
			 case 1: //Draw an Arc
			 	ArcPanelHandle =LoadPanel (GraphPanelHandle, "AngleSelect.uir", ARC);     
				InstallPopup (ArcPanelHandle);
	    		break; 
	    	case 2:  //Draw a Rectangle
				PlotRectangle (GraphPanelHandle, PLOTS_GRAPH, X1_Position,
							   Y1_Position, X2_Position, Y2_Position, SelectedColor,
							   												FillColor);
				break;
			case 3:  //Draw an Oval
				PlotOval(GraphPanelHandle, PLOTS_GRAPH, X1_Position,
							   Y1_Position, X2_Position, Y2_Position, SelectedColor,
	    					   												FillColor);
				break;
			case 4:   //Enter Text
				TextPanelHandle =LoadPanel (GraphPanelHandle, "TextEntry.uir", TEXT); 
				//Get Marker's position 
	    		GetGraphCursor (GraphPanelHandle, PLOTS_GRAPH, Marker, 
	    		 										 &X1_Position, &Y1_Position);
				InstallPopup (TextPanelHandle);
				break;
   		 }
   		 StartPlotFlag=0;  
   		}
   	}	 
    return 0;
}//UpdateMarkerCB

//Select the angles for the arc
int CVICALLBACK AnglesSelectedCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	double BeginAngle, ArcAngle; 
	switch (event)
		{
		case EVENT_COMMIT:
			//Obtain the Begin and the Arc angles
			GetCtrlVal(ArcPanelHandle,ARC_BEGIN_ANGLE,&BeginAngle);
			GetCtrlVal(ArcPanelHandle,ARC_BEGIN_ANGLE,&ArcAngle); 
			RemovePopup(ArcPanelHandle); 
			//Plot the arc with the selected values
			PlotArc (GraphPanelHandle, PLOTS_GRAPH, X1_Position, Y1_Position,
					 	       X2_Position, Y2_Position, BeginAngle, ArcAngle, 
					               				     SelectedColor, FillColor);
			break;
		}
	return 0;
}//AnglesSelectedCB

//Selections for Text input
int CVICALLBACK TextOKCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char TextBuffer[260], ExistingFontLabel[260], PointSizeLabel[260], NewMetaFont[260]={'\0'};
	int FontIndex, FontColor, PointSizeIndex, PointSize, BoldVal, ItalicsVal, UnderlineVal, 
	    StrikeoutVal;
	
	switch (event)
	{													  
		case EVENT_COMMIT:
			//Get the string entered in the text buffer
			GetCtrlVal(TextPanelHandle,TEXT_TEXT_STRING,TextBuffer);
			//Get existing font label
			GetCtrlIndex(TextPanelHandle,TEXT_RING, &FontIndex);
			GetLabelFromIndex(TextPanelHandle, TEXT_RING,FontIndex,
			                                         ExistingFontLabel);
			
			//Get Point Size index and label.
			GetCtrlIndex(TextPanelHandle,TEXT_POINT_SIZE, &PointSizeIndex);
			GetLabelFromIndex(TextPanelHandle, TEXT_POINT_SIZE,PointSizeIndex,
			                                                    PointSizeLabel); 
			Scan (PointSizeLabel, "%s>%i", &PointSize); //String to integer
			//Check if Bold selected
			GetCtrlVal(TextPanelHandle, TEXT_BOLD, &BoldVal);
			//Check if Italics selected
			GetCtrlVal(TextPanelHandle, TEXT_ITALICS, &ItalicsVal);
			//Check if Underline selected
			GetCtrlVal(TextPanelHandle, TEXT_UNDERLINE, &UnderlineVal);
			//Check if Strileout selected
			GetCtrlVal(TextPanelHandle, TEXT_STRIKEOUT, &StrikeoutVal);
			
			//Create a new meta-font based on the selections
			CreateMetaFont(NewMetaFont, ExistingFontLabel, PointSize,BoldVal,
			                              ItalicsVal, UnderlineVal,StrikeoutVal);
			//Obtain Font color													
			GetCtrlVal(TextPanelHandle, TEXT_COLOR_SELECT,&FontColor); 													
			HidePanel(TextPanelHandle);
			//Plot the selected text on the graph control with the created meta-font
			PlotText(GraphPanelHandle, PLOTS_GRAPH, X1_Position,
						   Y1_Position,TextBuffer,NewMetaFont,FontColor,VAL_TRANSPARENT);
			break;
	}
	return 0;
}//TextOKCB

//Exit the project
int CVICALLBACK Shutdown (int panel, int control, int event,
                          void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface (0);
    return 0;
} //Shutdown


//Clear everything on the Graph Plot
int CVICALLBACK ClearPlotCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			DeleteGraphPlot(GraphPanelHandle,PLOTS_GRAPH, -1, VAL_DELAYED_DRAW);
			RefreshGraph(GraphPanelHandle,PLOTS_GRAPH);
			break;
	}
	return 0;
}//ClearPlotCB
