/***************************************************************************

Purpose:  This project reads the DataSocket data and the attributes created
by project3-1Send.prj.  This project also demonstrates reading the data from a "file"
written in project3-1Send.prj instead of the DataSocket Server.  The new data is 
read from the "file" every time the UPDATE command button is selected.

Project Name:   project3-1Receive.prj
File Name: 		project3-1Receive.c  

****************************************************************************/

#include <cvidef.h>
#include <cviauto.h>
#include <dataskt.h>
#include <formatio.h>
#include <ansi_c.h>
#include <cvirte.h>		
#include <userint.h>
#include <utility.h>
#include "project3Receive.h"


// Structure to Read
typedef struct WriteDataStruct
{
    short data;
    short index;
} WriteDataStruct;


int Checkerror(int ); //Checks status of datasocket functions 

static WriteDataStruct WriteData; 
static int PanelHandle;
// DShandle is the handle used by all of the DataSocket Operations
static DSHandle DSReadHandle;    
int hours,minutes, seconds; 
HRESULT errorCode;  
char Source_String[500], errorBuf[256], *ReadString;

//Function prototype
void CVICALLBACK DSCallback(DSHandle DSReadHandle, int event, void *callbackData);

//main function loads the GUI and launches the DataSocket server
int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((PanelHandle = LoadPanel (0, "project3Receive.uir", DS_READ)) < 0)
		return -1;
	DisplayPanel (PanelHandle);
	
	RunUserInterface ();
	return 0;
} //main

////Invoked when the CONNECT button is selected.  
//int CVICALLBACK ConnectReadCB (int panel, int control, int event,
//		void *callbackData, int eventData1, int eventData2)
//{
//	
//	switch (event)
//	{
//		case EVENT_COMMIT:
//			ResetTextBox (PanelHandle,DS_READ_RD_DSP, "");
//			if (DSReadHandle)
//			{
//				errorCode =DS_DiscardObjHandle(DSReadHandle);
//				Checkerror(errorCode);
//				DSReadHandle=0;
//			}
//			
//			//Read the Source
//			GetCtrlVal(PanelHandle, DS_READ_SOURCE, Source_String);
//			
//			//Creates a DataSocket object and connects it to the data source 
//			errorCode=DS_Open (Source_String, DSConst_ReadAutoUpdate, DSCallback, 
//												        NULL, &DSReadHandle);
//			
//			Checkerror(errorCode); 
//			//Display message 
//			SetCtrlVal(PanelHandle, DS_READ_MSG ,"Connected");
//			
//			break;
//	}
//	return 0;
//} //ConnectReadCB

//Invoked when the CONNECT button is selected.  
int CVICALLBACK ConnectReadCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char buf [260]= {"\0"};
	DSEnum_Status Status;

	switch (event)
	{
		case EVENT_COMMIT:
			ResetTextBox (PanelHandle,DS_READ_RD_DSP, "");
			if (DSReadHandle)
			{
				errorCode =DS_DiscardObjHandle(DSReadHandle);
				Checkerror(errorCode);
				DSReadHandle=0;
			}
			
			//Read the Source
			GetCtrlVal(PanelHandle, DS_READ_SOURCE, Source_String);
			
			//Creates a DataSocket object and connects it to the data source 
			errorCode=DS_Open (Source_String, DSConst_ReadAutoUpdate, 
                    				                            DSCallback, NULL, &DSReadHandle);
			
			Checkerror(errorCode); 
			errorCode=DS_GetStatus (DSReadHandle,&Status);
			Checkerror(errorCode);
			switch (Status)
			{
				case 1:
					 sprintf(buf, "Unconnected");
					 break;
				case 2:
					 sprintf(buf, "Connection Active");
					 break;
				case 3:
					 sprintf(buf, "Connection Idle");
					 break;
				case 4:
					 sprintf(buf, "Connection Error");
					 break;
			}
			//Display DS Status 
			SetCtrlVal(PanelHandle, DS_READ_MSG ,buf);
		
	}
	return 0;
} //ConnectReadCB


//This callback function is executed whenever the data or the status changes
void CVICALLBACK DSCallback(DSHandle DSReadHandle, int event, void *callbackData)
{
	char msg[1000];	
	unsigned int StringLen;
	
	switch(event)	   
	{
		case DS_EVENT_DATAUPDATED: 
			errorCode = DS_GetDataType (DSReadHandle, NULL, &StringLen, NULL);
			 
			if (StringLen ==0)  //No data received
			{
				MessagePopup ("DATASOCKET ERROR",
							  "No data received. Connect Write DataSocket first");
				Delay (3.0); //Wait a while for the operator to enable the DataSocket Writer
				//Disable DISCONNECT button
				SetCtrlAttribute (PanelHandle, DS_READ_DISCONNECT, ATTR_DIMMED, 1); 
				//Enable CONNECT button  
				SetCtrlAttribute (PanelHandle, DS_READ_CONNECT , ATTR_DIMMED, 0); 
				return;
			}
			else
				ReadString= malloc(StringLen +1);
				
			//Get the string from DataSocket Writer
			errorCode = DS_GetDataValue (DSReadHandle, CAVT_CSTRING,
											 	ReadString, StringLen+1, 0, 0);
			Checkerror(errorCode);								 	
			//Get the index value from DataSocket Writer
			errorCode = DS_GetAttrValue (DSReadHandle, "Index", CAVT_SHORT,
										 &WriteData.index, sizeof(short), 0, 0);
		    Checkerror(errorCode);
		
			//No errors, write data
			if (errorCode>=0)
			{
				InsertTextBoxLine(PanelHandle,DS_READ_RD_DSP, -1, ReadString); 
				SetCtrlVal(PanelHandle, DS_READ_INDEX,WriteData.index);
			}
			//Enable DISCONNECT button 
			SetCtrlAttribute (PanelHandle, DS_READ_DISCONNECT, ATTR_DIMMED, 0); 
			//Disable CONNECT button  
			SetCtrlAttribute (PanelHandle, DS_READ_CONNECT , ATTR_DIMMED, 1);   
		break;
		
		case DS_EVENT_STATUSUPDATED:
			//DataSocket message
			errorCode=DS_GetLastMessage(DSReadHandle, msg,1000);
			Checkerror(errorCode);
			SetCtrlVal(PanelHandle, DS_READ_ERROR_MSG ,msg);
			break;
	}

 return;
} //DSCallback

//Update Data
int CVICALLBACK UpdateCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			errorCode=DS_Update(DSReadHandle);
			Checkerror(errorCode);
			break;
	}
	return 0;
} //UpdateCB

//Read DataSocket file data
int CVICALLBACK ReadFileCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char ProjectDir[261], SelectedDataSocketPath[261],FileSource[261], dirName[261], 
		 Directory[261], Path[261], SelectedFile[261];
	short FileSelected;
	
	switch (event)
	{
		case EVENT_COMMIT:
			GetProjectDir(ProjectDir);
			sprintf(Directory, "%s\\datafiles", ProjectDir);
			FileSelected = FileSelectPopup (Directory, "*.dsd", "",
								"SELECT DATASOCKET FILE",  VAL_LOAD_BUTTON, 0, 1, 1, 
															0, SelectedDataSocketPath);
		
			if (FileSelected !=0)// file is selected
			{
				//Get the file name
				SplitPath (SelectedDataSocketPath, NULL, dirName, SelectedFile);
				//Create the pathname from where to read the file
				sprintf(Path, "%s\\datafiles",ProjectDir);
				SetDir(Path);  //Set the path
				
				sprintf(FileSource,"file:../datafiles/%s",SelectedFile);
				SetCtrlVal(PanelHandle, DS_READ_SOURCE, FileSource);
			}
			else
				SetCtrlVal(PanelHandle, DS_READ_SOURCE, "");
				
		break;
	}
	return 0;
} //ReadFileCB

//Reset the Source
int CVICALLBACK ResetSourceCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			DefaultCtrl(PanelHandle, DS_READ_SOURCE);
			break;
	}
	return 0;
}//ResetSourceCB

//Invoked when DISCONNECT is selected   
int CVICALLBACK DisconnectReadCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			if (DSReadHandle)
			{
				errorCode =DS_DiscardObjHandle(DSReadHandle);
				Checkerror(errorCode);
				DSReadHandle=0;
			}
			//Display message
			SetCtrlVal(PanelHandle, DS_READ_MSG,"Disconnected"); 
			//Disable DISCONNECT button
			SetCtrlAttribute (PanelHandle, DS_READ_DISCONNECT, ATTR_DIMMED, 1); 
			//Enable CONNECT button 
			SetCtrlAttribute (PanelHandle, DS_READ_CONNECT , ATTR_DIMMED, 0); 
		break;
	}
	return 0;
}//DisconnectReadCB

//Status checking for datasocket functions
int Checkerror(int errorCode )
{
	 if (errorCode <0)   //if problem, display the error message
	{
			DS_GetLibraryErrorString (errorCode, errorBuf, 256);
			SetCtrlVal(PanelHandle, DS_READ_ERROR_MSG ,errorBuf);
			return -1;
	}

 return 0;
}//Checkerror

//Discard the DataSocket objects on Exit
int CVICALLBACK ExitReadCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			if (DSReadHandle)
			{
				errorCode=DS_DiscardObjHandle(DSReadHandle);
				Checkerror(errorCode);
				DSReadHandle=0;
			}
			QuitUserInterface (0);
		break;
	}
	return 0;
}//ExitReadCB

//Exit project by selecting panel close "X" in top right corner
int CVICALLBACK ClosePanelCB (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_CLOSE:
			errorCode=DS_DiscardObjHandle(DSReadHandle);
				Checkerror(errorCode);
				DSReadHandle=0;
				QuitUserInterface (0);
			break;
	}
	return 0;
} //ClosePanelCB


