/*******************************************************************************

Purpose:  This project generates random numbers every second using the CVI timer, and 
writes the string with the time stamp and the generated random number to the DataSocket 
Server.  This application also generates a sequential index for the data each time the data
is created and is also sent as a data attribute to the server.  This data is read by the 
project3-1Receive.prj simultaneously as the data is generated.  This project also demonstrates 
writing the data to a "file" instead of the DataSocket.  This data is them read from the "file" 
every time the UPDATE command button is selected from project3-1Receive.prj.

Project Name: project3-1Send.prj
File Name:    project3-1Send.c 

*********************************************************************************/
#include <cvidef.h>
#include <cvirte.h>    
#include <userint.h>
#include <cviauto.h>
#include <ansi_c.h>
#include <utility.h>
#include "dataskt.h"
#include <formatio.h>
#include "Project3Send.h"

/* This is the callback function prototype for  DataSocket */
void CVICALLBACK UpdateDSCallback (DSHandle DSWriteHandle, int event, void *callbackData);
int Checkerror(int ); //Checks error of datasocket functions 

static int WritepanelHandle;
static  DSHandle DSWriteHandle = 0;
HRESULT error;
char errorBuf[260];


//Structure to write data
typedef struct WriteDataStruct
{
    short data;		 	//random number
    short index;		//counter
} WriteDataStruct;

WriteDataStruct WriteData; 

//Main function 
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)   /* Needed if linking in external compiler;
    															harmless otherwise */
        return -1;  /* out of memory */
    if ((WritepanelHandle = LoadPanel (0, "Project3Send.uir", WRT)) < 0)
        return -1;
    DisplayPanel (WritepanelHandle);
	//Start the DataSocket Server
    error =DS_ControlLocalServer (DSConst_ServerLaunch);
    Checkerror(error);
    
    RunUserInterface ();
    //Close the DataSocket Server
    error = DS_ControlLocalServer (DSConst_ServerClose);
    Checkerror(error); 
    return 0;
}// main

//Invoked when CONNECT button is selected  
int CVICALLBACK ConnectCB (int panel, int control, int event,            
        void *callbackData, int eventData1, int eventData2)                    
{  
    char URL[500];
    
    switch (event) {
        case EVENT_COMMIT: 
        	ResetTextBox(WritepanelHandle,WRT_DSP_DATA, "");
            //Read the connection from the URL control box
            GetCtrlVal (WritepanelHandle, WRT_SOURCE, URL);
            //Open the DataSocket or File connection (depending on the URL)
			error=DS_Open (URL, DSConst_Write, UpdateDSCallback, NULL, &DSWriteHandle);
			
			Checkerror(error); 
            
            WriteData.index=0; //initialize the index                   									
		    //Start the timer to generate  data 
		    SetCtrlAttribute(WritepanelHandle, WRT_TIMER ,ATTR_ENABLED,TRUE);
		    //Enable DISCONNECT button 
			SetCtrlAttribute (WritepanelHandle, WRT_DISCONNECT, ATTR_DIMMED, 0); 
			//Disable CONNECT button
			SetCtrlAttribute (WritepanelHandle, WRT_WRT_CON , ATTR_DIMMED, 1); 
            break;                                                             
    }                                                                      
    return 0;                                                                  
}//ConnectCB                                                                              

//Called from  DS_Open function when data or error is updated 
void CVICALLBACK UpdateDSCallback (DSHandle DSWriteHandle, int event, void *callbackData)
{
    char message[1000];

    switch (event) {
        case DS_EVENT_STATUSUPDATED: /* on status updated*/
        	//Obtain the last message and update the message box
            error =DS_GetLastMessage (DSWriteHandle, message, 1000);
            Checkerror(error); 
            SetCtrlVal (WritepanelHandle, WRT_STATUS, message);
            break;
    }
}//UpdateDSCallback

// Create new random data and index
int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int hours, minutes, seconds;
    char DataString[50];
    
    switch (event) {
        case EVENT_TIMER_TICK:
        	//Generate the random data and write to the structure member
            WriteData.data = rand() % 100;
            //Get the system time
            GetSystemTime(&hours, &minutes, &seconds);
            
            if (DSWriteHandle)//if handle is valid
            {
            	//Create a string to write 
            	sprintf(DataString, "%-2d:%-2d:%-2d                %-2d",
            										hours, minutes, seconds,WriteData.data);
				//Display string on the Text Box
            	InsertTextBoxLine(WritepanelHandle,WRT_DSP_DATA,-1, DataString);
				//Set the DataSocket object value
				error=DS_SetDataValue (DSWriteHandle, CAVT_CSTRING, &DataString, 50, 0);
				Checkerror(error); 
				//Set the data socket attribute value for index
				error=DS_SetAttrValue (DSWriteHandle, "Index", CAVT_SHORT, 
																&WriteData.index, 0, 0);
				Checkerror(error); 
				//Synchronize the data value and the attribute value
				error =DS_Update(DSWriteHandle);
				Checkerror(error); 
				//Update the Index box 
           		SetCtrlVal (WritepanelHandle, WRT_INDX, WriteData.index);
           		WriteData.index++;
            }
            else
            {
				MessagePopup ("DATA SOCKET HANDLE ERROR",
							  "DataSocket Handle is invalid.\nRe-run Project.");
				exit(-1);  //Terminate project
			}
           break;
    }
    return 0;
} //TimerCallback

//Invoked by the DISCONNECT command button to discard the DataSocket object 
//and free the resources
int CVICALLBACK DisconnectCB (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
        	//Disable timer
            SetCtrlAttribute (WritepanelHandle, WRT_TIMER, ATTR_ENABLED, FALSE);
            if (DSWriteHandle) 
            {
            	//Discard the DataSocket object and free resources
                error =DS_DiscardObjHandle(DSWriteHandle); 
                Checkerror(error); 
                DSWriteHandle = 0;
            }
            
            SetCtrlVal (WritepanelHandle, WRT_STATUS, "Unconnected.");
            //Disable DISCONNECT button
            SetCtrlAttribute (WritepanelHandle, WRT_DISCONNECT, ATTR_DIMMED, 1); 
            //Enable CONNECT button  
			SetCtrlAttribute (WritepanelHandle, WRT_WRT_CON , ATTR_DIMMED, 0); 
            break;
    }
    return 0;
} //DisconnectCB

//Write data to file.  Invoked when the "WRITE FILE" command button is selected
int CVICALLBACK WriteToFileCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	char  prjDir[261], FileURL[261], dirName[261], DSFileName[261];
	short len;
	int nPrevState;
	switch (event)
	{
		case EVENT_COMMIT:	
			len =0;
			do
			{
				//Enter file name to write data
				PromptPopup ("DATASOCKET FILE NAME", 
			           "Enter the file name to write data (no extension)", 	
			           					DSFileName, 260);
				len = strlen (DSFileName);
				//Check file name entered is at least one character long
			} while (len ==0 );
			
			SetCtrlVal(WritepanelHandle, WRT_SOURCE, "");
			GetProjectDir(prjDir);			
			nPrevState= SetBreakOnLibraryErrors(0);
			sprintf(dirName, "%s\\datafiles",prjDir);
			//Make the directory 
			if ( SetDir(dirName) !=0)  MakeDir (dirName);  
			SetBreakOnLibraryErrors(nPrevState);
			SetDir(dirName);  //Set the path
			//Create the path name to write to ".dsd" file
			sprintf(FileURL,"file:../datafiles/%s.dsd",DSFileName);
			SetCtrlVal(WritepanelHandle, WRT_SOURCE, FileURL);		
	}
	return 0;
} //WriteToFileCB

//Reset the URL indicator invoked by the "RESET" command button
int CVICALLBACK ResetCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			DefaultCtrl (WritepanelHandle, WRT_SOURCE);
			break;
	}
	return 0;
}//ResetCB

//error checking for datasocket functions
int Checkerror(int error )
{
	 if (error <0)   //if problem, display the error message
	{
			DS_GetLibraryErrorString (error, errorBuf, 256);
			SetCtrlVal(WritepanelHandle, WRT_STATUS ,errorBuf);
			return -1;
	}

 return 0;
}

//Exit project
int CVICALLBACK ExitCB (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            SetCtrlAttribute (WritepanelHandle, WRT_TIMER, ATTR_ENABLED, FALSE);
            if (DSWriteHandle) {
                error= DS_DiscardObjHandle(DSWriteHandle);
                Checkerror(error); 
                DSWriteHandle = 0;
            }
            QuitUserInterface (0);
            break;
    }
    return 0;
} //ExitCB
