/**************************************************************

Purpose:  This project  illustrates some of the basic 
features of the Table Control. Some data is read from a file and 
the other generated randomly and displayed on the table control.
Any selected column is sorted in ascending or descending direction 
as selected by the user.  By selecting a cell and performing a SEARCH, 
all the rows with the matching values in that column are highlighted.  
These rows are simultaneously pasted to a system clipboard and to a 
table when you select the DISPLAY command button.

Project Name: project4.prj
File Name:    project4.c

******************************************************************/

#include <formatio.h>
#include <ansi_c.h>
#include <cvirte.h>		
#include <userint.h>
#include "project4.h"
#include "Project4SearchTable.h" 
#define NMBR_OF_ROWS 15   //Total number of rows in the table

//Define the columns of the table control
#define 	CURRENT_COL 	1
#define		VOLTAGE_COL		2
#define		PRESSURE_COL	3
#define		TEMP_COL		4
#define		RESULTS_COL		5
#define		PICTURE_COL		6

static int TablePanel,SearchedPanel, SearchedRows=0;
int status, DataLoaded=1, visible, smilePicture, 
            frownPicture,  SelectedColumnIndex;  


//Function prototype 
int HighlightRow(Point NewCell, int SearchNumber);

//Main
int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((TablePanel = LoadPanel (0, "project4.uir", PNL)) < 0)
		return -1;
	
	//Hide the right click option to Find item.  
	//We will do the search programmatically
	HideBuiltInCtrlMenuItem (TablePanel, PNL_TAB, VAL_SEARCH);
	
	DisplayPanel (TablePanel);
	//Panel to display searched data	
	if ((SearchedPanel = LoadPanel (PNL, "project4SearchTable.uir", SRCH_PNL )) < 0)
		return -1;

	//Load the bitmap files for happy and sad faces
	GetBitmapFromFile ("smiley.ico", &smilePicture);
	GetBitmapFromFile ("frown.ico", &frownPicture); 
	
	RunUserInterface ();
	DiscardPanel (TablePanel);
	return 0;
}//main

//Create and display data on Table. Invoked from "LOAD DATA" command button
int CVICALLBACK LoadDataCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	short int index, result;
	double volts_array[ NMBR_OF_ROWS+1], pressure, temperature, 
		   current_array[ NMBR_OF_ROWS+1];
	switch (event)
	{
		case EVENT_COMMIT:
			//If rows exit then delete them before loading the data again
			if (DataLoaded==0) DeleteTableRows (TablePanel, PNL_TAB, 1, -1);
			//Set background color of "Results" column
			SetTableColumnAttribute (TablePanel, PNL_TAB, RESULTS_COL, 
									ATTR_TEXT_BGCOLOR, VAL_LT_GRAY);
	
			//Create and load data						  
			for (index=1; index <= NMBR_OF_ROWS; index++)
			{
				
			 	InsertTableRows (TablePanel, PNL_TAB, -1, 1, 
			 							VAL_USE_MASTER_CELL_TYPE);
			 
				//Read the simulated data from the ASCII files 
				//The data from the file is input to current_array
				FileToArray ("Current.txt", current_array, VAL_DOUBLE, 
										NMBR_OF_ROWS, 1,  VAL_GROUPS_TOGETHER, 
										    VAL_GROUPS_AS_COLUMNS, VAL_ASCII);
				//The data from the file is input to volts_array 
				FileToArray ("Volts.txt", volts_array, VAL_DOUBLE, NMBR_OF_ROWS, 1,
							 			VAL_GROUPS_TOGETHER, VAL_GROUPS_AS_COLUMNS, 
							 											VAL_ASCII);
								 
				//Create random data for pressure and temperature
				 pressure = rand ()/10.0;
				 temperature = rand ()/250.0;
			 
				 //Display results on the table
				 SetTableCellVal(TablePanel, PNL_TAB,MakePoint(CURRENT_COL,index), 
				 										current_array[index-1]);
				 SetTableCellVal(TablePanel, PNL_TAB,MakePoint(VOLTAGE_COL,index),
				 											volts_array[index-1]);
				 SetTableCellVal(TablePanel, PNL_TAB,MakePoint(PRESSURE_COL,index),
				 															pressure);
				 SetTableCellVal(TablePanel, PNL_TAB,MakePoint(TEMP_COL,index),
				 														temperature);
				 result=rand() % 2;  //generate odd and even number
				 if (result == 0)//pass
				 {
				 		//Display the "Pass" string in green and bold attributes
				 		SetTableCellVal(TablePanel, PNL_TAB,MakePoint(RESULTS_COL,index),		
				 															    	"Pass");  
						SetTableCellAttribute (TablePanel, PNL_TAB, MakePoint(RESULTS_COL,index),
											  					 ATTR_TEXT_COLOR, VAL_DK_GREEN);
						SetTableCellAttribute (TablePanel, PNL_TAB, MakePoint(RESULTS_COL,index),
											  								 ATTR_TEXT_BOLD, 1);
						SetTableCellVal(TablePanel, PNL_TAB,MakePoint(PICTURE_COL,index),
																					smilePicture);
				 }
				 else  //fail
				 {
				 		//Display the "Fail" string in red and bold attributes 
				 		SetTableCellVal(TablePanel, PNL_TAB,MakePoint(RESULTS_COL,index),"Fail");
				 		SetTableCellAttribute (TablePanel, PNL_TAB, MakePoint(RESULTS_COL,index),
											   				ATTR_TEXT_COLOR, VAL_RED);
						SetTableCellAttribute (TablePanel, PNL_TAB, MakePoint(RESULTS_COL,index),
											   						ATTR_TEXT_BOLD, 1);
						SetTableCellVal(TablePanel, PNL_TAB,MakePoint(PICTURE_COL,index),
																				frownPicture); 					   						
				 }
			 
			} //for
			
			//Make horizontal grid control visible
			SetCtrlAttribute (TablePanel, PNL_H_GRID, ATTR_VISIBLE, 1);
			SetCtrlVal(TablePanel, PNL_H_GRID,0);
			
			//Dim the "LOAD" command button  
			SetCtrlAttribute (TablePanel, PNL_LOAD, ATTR_DIMMED, 1);
			DataLoaded =0;  //Rows are created on the table 
			break;
			
	}
	
	return 0;
} //LoadDataCB

//Select the column to sort
int CVICALLBACK SortColumnSelectCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	
	switch (event)
	{
		case EVENT_COMMIT:
			//Check if data loaded
			if (DataLoaded==1)
			{
				//Inform the user
				MessagePopup ("LOAD DATA FIRST",
							"No data loaded.  Load data before sorting");
				return 0;
			}
			
			//Un-dim Ascending/Descending control  
			SetCtrlAttribute (TablePanel, PNL_RNG, ATTR_DIMMED, 0);
			GetCtrlIndex (TablePanel, PNL_SEL_COL, &SelectedColumnIndex);

			break;
	}
	return 0;
} //SortColumnSelectCB

//Sort using the Ring selection
int CVICALLBACK SortResultsCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int direction[5]; 
	static int SortFlag[5]={2,2,2,2,2};
	switch (event)
	{
		case EVENT_COMMIT:
			//Get the ring index for ascending or descending sort							
			GetCtrlIndex (TablePanel, PNL_RNG, &direction[SelectedColumnIndex]);
			//Check if already sorted in the direction selected
			if( (direction[SelectedColumnIndex] ==0 && SortFlag[SelectedColumnIndex]==0)) 
			{   //Ascending
				//Inform the user
				MessagePopup ("ALREADY SORTED IN THIS DIRECTION",
							"Sorted in this direction. Try a different direction");
				SortFlag[SelectedColumnIndex]=0;
				return 0;
			 }
			 else if ( direction[SelectedColumnIndex] ==1 && 
			 											SortFlag[SelectedColumnIndex]==1 ) 
			 {  //Descending
			   //Inform the user
				MessagePopup ("ALREADY SORTED THIS DIRECTION",
							"Sorted in this direction. Try a different direction");
				SortFlag[SelectedColumnIndex]=1;
				return 0;
			 }
		
			//Sort in the direction specified in the ring control
			SortTableCells (TablePanel, PNL_TAB, VAL_TABLE_ENTIRE_RANGE,
							       VAL_COLUMN_MAJOR, SelectedColumnIndex + 1, 
							      		 direction[SelectedColumnIndex], 0, 0);
							
			//Set flag to determine the direction presently sorted
			if (direction[SelectedColumnIndex] ==0)
				SortFlag[SelectedColumnIndex]=0; //Ascend sort Flag
			else 
				SortFlag[SelectedColumnIndex]=1;  //Descend sort flag
		
			break;
	}
	return 0;
} //SortResultsCB

//Invoked when left-clicked on Table
int CVICALLBACK TableCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
		if (event == EVENT_LEFT_CLICK) 
		{	
			if (DataLoaded != 0)   //No data loaded on the table
			{
				MessagePopup ("LOAD DATA",
							"Load data before selecting cell on table.");
				return 0;
			}
		} //event
	
	return 0;
} //TableCB

//Search on selected cell value 
int CVICALLBACK SearchCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	Point NewCell, SearchCell; 
	double SearchValue, NewValue,value;
	int RowsInTable,row, SearchFlag=0;
	static int SearchNumber=0;
	char SearchString[10], NewStringValue[10];
	
	switch (event)
	{
		case EVENT_COMMIT:
		  	//Get active cell 
			GetActiveTableCell (TablePanel, PNL_TAB, &SearchCell);

			//Get Cell value
			GetTableCellVal (TablePanel, PNL_TAB, SearchCell, &value);
			
			if (value ==0.0) //no active cell
			//Popup message if no cell selected
			{
				MessagePopup ("NO SEARCH CELL SELECTED",
							"Select cell on table before searching");
				return 0;
			}
			
			//Set the selected cell and highlight it
			GetActiveTableCell (TablePanel, PNL_TAB, &SearchCell);
			SetTableCellAttribute (TablePanel, PNL_TAB, SearchCell,
								   ATTR_TEXT_BOLD, 1);


			if (SearchCell.x == PICTURE_COL)	//Searching on picture value
			{
				MessagePopup ("PICTURE SEARCH DISABLED",
				   "Code not set up to search on picture cell.\nSelect another column.");
				return 0;
			}
			if (SearchCell.x == RESULTS_COL)	//Searching on string value				   
				//Get the selected cell value
				GetTableCellVal(TablePanel, PNL_TAB,SearchCell,SearchString);
			else
				//Get the selected cell value
				GetTableCellVal(TablePanel, PNL_TAB,SearchCell,&SearchValue);
			
			//Get number of rows in table
			GetNumTableRows(TablePanel, PNL_TAB,&RowsInTable);
			
			
			//Search on all the values in that column
			for (row=1; row <= RowsInTable; row++)
			{
				SetActiveTableCell(TablePanel, PNL_TAB, MakePoint(SearchCell.x,row));
				GetActiveTableCell (TablePanel, PNL_TAB, &NewCell);
				if (SearchCell.x != RESULTS_COL)		
					//Get the selected cell value                                   
					GetTableCellVal(TablePanel, PNL_TAB,NewCell,&NewValue);
				else   //Searching on string value
					//Get the selected cell value                                   
					GetTableCellVal(TablePanel, PNL_TAB,NewCell,NewStringValue);
					
				//If value found and is not string
				if ( (SearchCell.x != RESULTS_COL) && (SearchValue == NewValue)) 
				{
					//Highlight the row and put on clipboard
					HighlightRow(NewCell, SearchNumber);
					SearchFlag=1;
					
				}//if 
				else if ( (SearchCell.x == RESULTS_COL) && (strcmp (SearchString,NewStringValue) ==0 )) 
				{
					//Highlight the row and put on clipboard
					HighlightRow(NewCell, SearchNumber);
					SearchFlag=1;
				}//else if 
				
			}//for
			
			//Enable the DISPLAY command button if there is data on clipboard
			if (SearchFlag)  
				SetCtrlAttribute (TablePanel, PNL_DISPLAY, ATTR_DIMMED, 0);
			
			SearchNumber++;  //increment the search color index
			if (SearchNumber== RESULTS_COL) SearchNumber=0;  //Reset SearchColor number
			
			break;
	}
	return 0;
} //SearchCB

//Highlight the selections and place on system clipboard
int HighlightRow(Point NewCell, int SearchNumber) 
{

//Hex Values for colors
int SearchColor[5]= {0xCCCCCC,    0x00FF00,    0x00FFFF,   0xFF00FF,    0xFFFF00}; 
//					 VAL_LT_GRAY  VAL_GREEN    VAL_CYAN	   VAL_MAGENTA  VAL_YELLOW
																 
	int *ColorPtr;
	ColorPtr=SearchColor; 
	
	////Color the row containing the  found cell 
	SetTableCellRangeAttribute (TablePanel, PNL_TAB,
								VAL_TABLE_ROW_RANGE(NewCell.y),
							ATTR_TEXT_BGCOLOR, *(ColorPtr+SearchNumber));
												
	SearchedRows++; //increment number of rows with searched values  
				
	//Paste selection to the clipboard
	ClipboardPutTableVals (TablePanel, PNL_TAB, VAL_TABLE_ROW_RANGE(NewCell.y));
	
	//Create rows on the Searched Table control
	InsertTableRows (SearchedPanel, SRCH_PNL_TAB, -1, 1,
								 VAL_USE_MASTER_CELL_TYPE);
	//Paste selection from the clipboard
	status = ClipboardGetTableVals (SearchedPanel, SRCH_PNL_TAB,
									VAL_TABLE_ROW_RANGE(SearchedRows), 0);
	return 0;							  					
} //HighlightRow

//Paste Clipboard data.  Invoked from DISPLAY command button
int CVICALLBACK DisplayFromClipboardCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	switch (event)
	{
		case EVENT_COMMIT:
			
			//Make the grid visible
			SetTableCellRangeAttribute (SearchedPanel, SRCH_PNL_TAB,VAL_TABLE_ENTIRE_RANGE,
													 ATTR_HORIZONTAL_GRID_VISIBLE,visible);
			//Color the horizontal grid 
			SetTableCellRangeAttribute (SearchedPanel, SRCH_PNL_TAB,
											VAL_TABLE_ENTIRE_RANGE, 
												ATTR_HORIZONTAL_GRID_COLOR,
																VAL_PANEL_GRAY);
			//Display the panel
			DisplayPanel (SearchedPanel); 
		
			break;
	}
	return 0;
}//PasteFromClipboardCB

//Return from Selection panel
int CVICALLBACK ReturnCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			HidePanel(SearchedPanel);
			break;
	}
	return 0;
} //ReturnCB


//Restore the data on the table to its original form
int CVICALLBACK ResetTableCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int row;
	switch (event)
	{
		case EVENT_COMMIT:
			for (row=1; row <= NMBR_OF_ROWS; row++)
			{
				//Remove the highlight from the row with the cell found 
				SetTableCellRangeAttribute (TablePanel, PNL_TAB,
											VAL_TABLE_ROW_RANGE(row),
											ATTR_TEXT_BGCOLOR, VAL_WHITE);
				
				//Remove the bolded cells
				SetTableCellRangeAttribute (TablePanel, PNL_TAB,
											VAL_TABLE_ROW_RANGE(row), 
											ATTR_TEXT_BOLD,0);
											
				//Put the default color of "Results" column
				SetTableCellRangeAttribute(TablePanel, PNL_TAB,
										   VAL_TABLE_COLUMN_RANGE(5),
											ATTR_TEXT_BGCOLOR,VAL_LT_GRAY);
											
				//Set the Results column to Bold
				SetTableCellRangeAttribute (TablePanel, PNL_TAB,
											VAL_TABLE_COLUMN_RANGE(5), ATTR_TEXT_BOLD,1);
			}//for
			break;
	}
	return 0;
} //ResetTableCB

//Invoked from the radio button to set/clear the horizontal grid
int CVICALLBACK HorizontalGridCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			//Get the radio button to show/remove horizontal grid
			GetCtrlVal(TablePanel, PNL_H_GRID,&visible);
			//Make the grid visible
			SetTableCellRangeAttribute (TablePanel, PNL_TAB,VAL_TABLE_ENTIRE_RANGE,
										ATTR_HORIZONTAL_GRID_VISIBLE,visible);
			//Color the horizontal grid 
			SetTableCellRangeAttribute (TablePanel, PNL_TAB,
										VAL_TABLE_ENTIRE_RANGE, ATTR_HORIZONTAL_GRID_COLOR,
																			VAL_PANEL_GRAY);
			break;
	}
	return 0;
} //HorizontalGridCB


//Close panel 
int CVICALLBACK ClosePanelCB (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_CLOSE:
			QuitUserInterface(0);
			break;
	}
	return 0;
} //ClosePanelCB

//You are done
int CVICALLBACK QuitCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			QuitUserInterface (0);
			break;
	}
	return 0;
}  //QuitCB
