/***************************************************
 Purpose:   Demonstrates acquiring analog input data 
            and plotting to a strip chart.

Project Name: project 6-1.prj 
File Name:    project 6-1.c 

***************************************************/

#include <utility.h>
#include <cvirte.h>		
#include <userint.h>
#include <easyio.h>
#include "project6-1.h"
#include <ansi_c.h>


static int DAQHandle;
long TaskID, daq_error, NumOfChannels;
char *ErrorString, Channel_Selected[100]; 
static double AcquiredData[2000];
int StartFlag=0;

int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((DAQHandle = LoadPanel (0, "project6-1.uir", PANEL)) < 0)
		return -1;
	DisplayPanel (DAQHandle);
	RunUserInterface ();
	DiscardPanel (DAQHandle);
	return 0;
} //main

//Invoked from START command button
int CVICALLBACK StartAcquisitionCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	switch (event)
	{
        case EVENT_COMMIT:
			//Obtain the channel string from the control box
            GetCtrlVal (DAQHandle, PANEL_CHL_STR, Channel_Selected);
			ClearStripChart (DAQHandle, PANEL_STRIP_CHART);  //Clear the strip chart
            
			//Step 1 - Create data acquisition task
			daq_error = nidaqAICreateTask (Channel_Selected, kNidaqPointByPoint,
										                  &NumOfChannels, &TaskID);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
								   
			//Step 2 - Start data acquisition
			daq_error=nidaqAIStart(TaskID);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
			if (NumOfChannels)
                SetCtrlAttribute (DAQHandle, PANEL_STRIP_CHART, 
                                ATTR_NUM_TRACES, NumOfChannels);
            
            if (NumOfChannels > 1)
            	 SetTraceAttribute (DAQHandle, PANEL_STRIP_CHART, 2,
                               			ATTR_TRACE_COLOR, VAL_RED);
            
            if (NumOfChannels > 2)
				SetTraceAttribute (DAQHandle, PANEL_STRIP_CHART, 3, 
									ATTR_TRACE_COLOR,  VAL_DK_GREEN);
			
			StartFlag=1;
			break;
	}
	return 0;
}//StartAcquisitionCB

//Run Every second on timer tick
int CVICALLBACK TimerTickCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	
		if  (( event == EVENT_TIMER_TICK) && (StartFlag) )
		{
			//Step 3 - Read data
			daq_error = nidaqAIRead (TaskID, Channel_Selected, 1, -1.0,	   
									 					AcquiredData);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
			
			//Step 4 - Plot data
			PlotStripChart (DAQHandle, PANEL_STRIP_CHART, AcquiredData,
												NumOfChannels, 0, 0,  VAL_DOUBLE);
		}
	return 0;
}//TimerTickCallback

int CVICALLBACK StopAcquisitionCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			//Step 5 - Stop data acquisition    
			daq_error =nidaqAIStop(TaskID);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
			daq_error =nidaqAIDestroyTask (TaskID);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
			 
			StartFlag=0;  //Stop timer - disable flag
			break;
	}
	return 0;
}


//Invoked from QUIT command button  
int CVICALLBACK QuitAcquisitionCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
		
			/* Stop data acquisition either in this function or in
			   the "StopAcquisitionCB" function. If you try to stop
			   a task that is already stopped, a library error occurs.*/
			   
			//Step 6 - Relinquish the resources
			daq_error =nidaqAIDestroyTask (TaskID);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
			StartFlag=0;  //Stop timer - disable flag
			QuitUserInterface (0);
			break;
	}
	return 0;
} //QuitAcquisitionCB
