#include "PROJECT6-2.h"

/*************************************************************
 
 Purpose:   Demonstrates otaining finite number of scans
            from analog input channels and plotting to a graph.

Project Name: project 6-2.prj
File Name:    project 6-2.c

***************************************************************/

#include <utility.h>
#include <cvirte.h>		
#include <userint.h>
#include <easyio.h>
#include "project6-2.h"
#include <ansi_c.h>

#define UPPER_LIMIT 6.0
#define LOWER_LIMIT -6.0

static int DAQHandle, GRAPH_CTRL_ID;;
long TaskID, daq_error, NumOfChannels, NmbrScans;
char *ErrorString, Channel_Selected[100]; 
static double AcquiredData[2000], ActualRate, ScanRate, *ScanData=0;
int StartFlag=0, ScanFlag=0, Switch_Selection;


int CreateGraphControl(void);

int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((DAQHandle = LoadPanel (0, "project6-2.uir", PANEL)) < 0)
		return -1;
	DisplayPanel (DAQHandle);
	RunUserInterface ();
	DiscardPanel (DAQHandle);
	return 0;
} //main

//Invoked from START command button
int CVICALLBACK StartAcquisitionCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

int i;
	switch (event)
	{
        case EVENT_COMMIT:
			//Obtain the channel string from the control box
            GetCtrlVal (DAQHandle, PANEL_CHL_STR, Channel_Selected);
			ClearStripChart (DAQHandle, PANEL_STRIP_CHART);  //Clear the strip chart
            
			
			daq_error = nidaqAICreateTask (Channel_Selected, kNidaqPointByPoint,
										                  &NumOfChannels, &TaskID);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
			//Get switch setting
			GetCtrlVal(DAQHandle,PANEL_SCAN_SW, &Switch_Selection);				   
			
			if (Switch_Selection==0 ) //For Continuous Mode
			{
				SetInputMode (DAQHandle, PANEL_STOP_CMD, 1);  //Enable Stop Button   
				SetCtrlAttribute (DAQHandle,PANEL_STRIP_CHART, ATTR_VISIBLE, 1);
				daq_error=nidaqAIStart(TaskID);
				if ( daq_error !=0)
				{
					ErrorString = nidaqGetErrorString (TaskID, daq_error);
					InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
				}
				if (NumOfChannels)
	                SetCtrlAttribute (DAQHandle, PANEL_STRIP_CHART, 
	                                ATTR_NUM_TRACES, NumOfChannels);
            
	            if (NumOfChannels > 1)
	            	 SetTraceAttribute (DAQHandle, PANEL_STRIP_CHART, 2,
	                               			ATTR_TRACE_COLOR, VAL_RED);
	            if (NumOfChannels > 2)
					SetTraceAttribute (DAQHandle, PANEL_STRIP_CHART, 3, 
										ATTR_TRACE_COLOR,  VAL_DK_GREEN);
				
				StartFlag=1;
				
				if (ScanFlag==1)
					SetCtrlAttribute (DAQHandle, GRAPH_CTRL_ID, ATTR_VISIBLE, 0);
				SetInputMode (DAQHandle, PANEL_START_BUTTON, 0);  //Disable Start Button
				ScanFlag=0;
			}
			else   //Scan Selection
			{
				SetInputMode (DAQHandle, PANEL_START_BUTTON, 1);  //Enable Start Button
				SetInputMode (DAQHandle, PANEL_STOP_CMD, 0);  //Disable Stop Button
				
				SetCtrlAttribute (DAQHandle,PANEL_STRIP_CHART, ATTR_VISIBLE, 0);
				CreateGraphControl(); //Create graph programmatically
				GetCtrlVal(DAQHandle,PANEL_SCAN_NMBR,&NmbrScans); 
				GetCtrlVal(DAQHandle,PANEL_SCAN_RATE, &ScanRate); 
				
				ScanData= (double *) malloc(NmbrScans* NumOfChannels * sizeof(double) );
				daq_error = nidaqAIScanOp (Channel_Selected, NmbrScans, ScanRate,
										   UPPER_LIMIT, LOWER_LIMIT,
										   kNidaqGroupByChannel, &ActualRate,
										   ScanData);
				if ( daq_error !=0)
				{
					ErrorString = nidaqGetErrorString (TaskID, daq_error);
					InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
				}
				
				for (i=0; i < NumOfChannels; i++)
				   if (i==0)
						PlotY (DAQHandle, GRAPH_CTRL_ID, &ScanData[NmbrScans*i], NmbrScans,
							   VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1,
							   														VAL_RED);
					else
					   	PlotY (DAQHandle, GRAPH_CTRL_ID, &ScanData[NmbrScans*i],
							  NmbrScans, VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE,
							  									VAL_SOLID, 1, VAL_DK_GREEN);
				ScanFlag =1;				  									
			}
			break;
	}
	return 0;
}//StartAcquisitionCB

//Create graph programmatically 
int CreateGraphControl(void)
{

	GRAPH_CTRL_ID = NewCtrl (DAQHandle, CTRL_GRAPH, "SCAN DATA", 40, 150);
	SetCtrlAttribute (DAQHandle, GRAPH_CTRL_ID, ATTR_WIDTH, 270);
	SetCtrlAttribute (DAQHandle, GRAPH_CTRL_ID, ATTR_HEIGHT, 200);
	SetCtrlAttribute (DAQHandle, GRAPH_CTRL_ID, ATTR_XNAME, "Scans"); 
	SetCtrlAttribute (DAQHandle, GRAPH_CTRL_ID, ATTR_YNAME, "Volts");
	SetAxisRange (DAQHandle, GRAPH_CTRL_ID, VAL_NO_CHANGE, 0.0, NmbrScans,
				  								VAL_MANUAL, -5.0, 5.0);
				  
return 0;
} //CreateGraphControl


//Invoked when "Number of Scans" value is entered
int CVICALLBACK NumberOfScanCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			GetCtrlVal(DAQHandle,PANEL_SCAN_NMBR,&NmbrScans);
			SetAxisRange (DAQHandle, GRAPH_CTRL_ID, VAL_MANUAL, 0.0,NmbrScans ,
						  VAL_NO_CHANGE, 0.0, 1.0);
			break;
	}
	return 0;
}

//Get Scan Rate 
int CVICALLBACK ScanRateCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			GetCtrlVal(DAQHandle,PANEL_SCAN_RATE, &ScanRate);
			break;
	}
	return 0;
}

//Run Every second on timer tick
int CVICALLBACK TimerTickCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	
		if  (( event == EVENT_TIMER_TICK) && (StartFlag) )
		{	
			daq_error = nidaqAIRead (TaskID, Channel_Selected, 1, -1.0,	   
									 					AcquiredData);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
			
			PlotStripChart (DAQHandle, PANEL_STRIP_CHART, AcquiredData,
												NumOfChannels, 0, 0,  VAL_DOUBLE);
		}
	return 0;
}//TimerTickCallback

//Invoked from STOP command button
int CVICALLBACK StopAcquisitionCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
		    if (ScanFlag==0)  //For continuous mode
		    {
			    //Step 5 - Stop data acquisition 
				daq_error =nidaqAIStop(TaskID);
				if ( daq_error !=0)
				{
					ErrorString = nidaqGetErrorString (TaskID, daq_error);
					InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
				}
			}
			daq_error =nidaqAIDestroyTask (TaskID);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
			SetInputMode (DAQHandle, PANEL_STOP_CMD, 0);  //Disable Stop Button
			SetInputMode (DAQHandle, PANEL_START_BUTTON, 1);  //Enable Start Button 
			StartFlag=0;  //Stop timer - disable flag
			break;
	}
	return 0;
}


//Invoked from QUIT command button  
int CVICALLBACK QuitAcquisitionCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			/* Stop data acquisition either in this function or in
			   the "StopAcquisitionCB" function. If you try to stop
			   a task that is already stopped, a library error occurs.*/
			   
			//Step 6 - Relinquish the resources
			daq_error =nidaqAIDestroyTask (TaskID);
			if ( daq_error !=0)
			{
				ErrorString = nidaqGetErrorString (TaskID, daq_error);
				InsertTextBoxLine (DAQHandle, PANEL_ERROR_BOX, -1, ErrorString);
			}
			StartFlag=0;  //Stop timer - disable flag
			QuitUserInterface (0);
			break;
	}
	return 0;
} //QuitAcquisitionCB


//Mode control callback
int CVICALLBACK Continuous_ScansCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			//Get switch setting
			GetCtrlVal(DAQHandle,PANEL_SCAN_SW, &Switch_Selection);				   
			
			if (Switch_Selection==0 ) //For Continuous Mode
			{
				SetInputMode (DAQHandle, PANEL_SCAN_NMBR, 0);  //Disable "Number of Scans" control
				SetInputMode (DAQHandle, PANEL_SCAN_RATE, 0);  //Disable "Scan Rate" control 
			}
			else  //For Scan mode
			{
				SetInputMode (DAQHandle, PANEL_SCAN_NMBR, 1);  //Enable "Number of Scans" control
				SetInputMode (DAQHandle, PANEL_SCAN_RATE, 1);  //Enable "Scan Rate" control 
			}
			break;
		}
	return 0;
}//Continuous_ScansCB
