/***************************************************************************** 
 *  Copyright 1998 National Instruments Corporation.  All Rights Reserved.   * 
 *****************************************************************************/

    /*=CHANGE:===============================================================*
                INSTRUCTIONS TO THE INSTRUMENT DRIVER DEVELOPER
                         READ THESE INSTRUCTIONS FIRST!
     *=======================================================================*
                                                                             
        Congratulations!  You have successfully created all the files needed
        for your instrument driver - *.c,  *.h, *.fp, and *.sub.  These     
        files give you a framework to build your complete instrument driver 
        with the following features:                                    
            - IVI compliance                                             
            - VXIplug&play compliance                                    
            - Attribute model                                           
            - State caching                                             
            - Simulation                                                
            - Multithread safety                                         
                                                                        
        Before you can begin using this driver, you must perform the following 
        manual changes.                                 
                                                                        
        - If you are developing a driver for a register-based device, 
          remove the function panels for the E3631A_WriteInstrData 
          and E3631A_ReadInstrData functions.  These functions
          are for use with message-based instruments.                                       
                                                                            
        - Device-specific changes are marked with the comment "CHANGE".      
          Search for all occurrences of the word "CHANGE" and make the changes 
          for your specific instrument.  Delete the comments after you complete 
          the changes.
          
        - The IVI wizard might not properly align the code it generates.
          Review the resulting *.h and *.c files and reformat them as necessary.
          
        - Read the help text for the functions and attributes to understand
          the functionality you must implement in this driver.  The easiest
          way to view the help text is by using the E3631A.fp 
          function panels in "operate" mode.  You can view the help text for
          the attributes in any of the E3631A_GetAttribute 
          function panels.

        - Many of the functions contain extensive code examples within 
          comment blocks.  These code examples contain embedded comments.
          The only way to embed comments is by using double slashes (//).  
          Some C compilers do not accept double slashes.  If you are developing 
          this driver for general use, convert the double slashes to 
          traditional C comment markers.
        
        - Edit the Function Panel help.  Pay close attention to the default
          values and valid ranges for each parameter.  If you change the 
          default value for a control, you also need to change the default
          value in the help for the control.  Also, search for the comment 
          "CHANGE" in the help text, and make the appropriate changes.
          
        - Edit the help for all attributes with the attribute editor.  Pay close
          attention to the valid range information.  The Set/Get/Check attribute
          function panels automatically display valid range information for 
          attributes that have range tables of type IVI_VAL_DISCRETE or 
          IVI_VAL_COERCED.  You must edit/create the valid range information 
          in the description of an attribute if any of the following conditions 
          is true:
            * The attribute does not have a range table.
            * The attribute has a range table of type IVI_VAL_RANGED.
            * The attribute uses multiple static range tables or a dynamic
              range table.
            
          Also, look for "CHANGE" comments in the help text, and make the 
          appropriate changes.
           
        - Verify that all values that the header file defines are valid for 
          your instrument.  Delete the values that your instrument does not 
          use.
            
        - Delete these instructions and save the modified program.         

	 *=======================================================================*
	                                                                             
	    This driver has functions and attributes that are common to most
	    DC power supplies.  The functions and attributes have example code with
	    instructions on how you can modify the code for your specific DC power
	    supply.  This section provides additional modification instructions
	    regarding special cases you must address.  The instructions in the
	    example code sections refer to these comments when applicable.
	
	 *=======================================================================*
	    
	    The following sections list groups of attributes and functions that are 
	    optional.  This driver has these functions and attributes, but you do not 
	    have to implement them in the driver for your specific instrument.
	    If you choose to implement one of the attributes or functions listed below, 
	    you must implement all of the attributes and functions that belong to 
	    the same group, unless otherwise noted.
	
	    - Trigger Group:  This group controls DC power supplies capable of 
	      changing their voltage and current when a trigger event occurs.
	      
	      Attributes:  E3631A_ATTR_TRIGGER_SOURCE
	                   E3631A_ATTR_TRIGGERED_VOLTAGE_LEVEL
	                   E3631A_ATTR_TRIGGER_CURRENT_LIMIT
	                   
	      Functions:   E3631A_ConfigureTriggerSource
	                   E3631A_ConfigureTriggeredVoltageLevel
	                   E3631A_ConfigureTriggeredCurrentLimit
	                   E3631A_Initiate                   
	                   E3631A_Abort
	
	    - Software Trigger Group: This group controls DC power supplies 
	      capable of triggering on a software trigger.
	      
	      Functions:   E3631A_SendSoftwareTrigger
	      
	    - Measure Group:  This group controls power supplies capable of taking
	      measurements on an output channel.
	
	      Functions:   E3631A_Measure
     *============================================================END=CHANGE=*/

/*****************************************************************************
 *  Triple Output DC Power Supply Instrument Driver                               
 *  LabWindows/CVI 5.0 Instrument Driver                                     
 *  Original Release: Friday, January 26, 2001                                  
 *  By: MyName, MyCompany                              
 *      PH. 999-555-1111   Fax 999-555-1112                               
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       Friday, January 26, 2001 - Instrument Driver Created.                  
 *                                                                           
 *****************************************************************************/

#include <string.h>
#include <stdio.h>  
#include <formatio.h>
#include <ctype.h>
#include "E3631A.h"

/*****************************************************************************
 *--------------------- Hidden Attribute Declarations -----------------------*
 *****************************************************************************/

#define E3631A_ATTR_OPC_TIMEOUT      (IVI_SPECIFIC_PRIVATE_ATTR_BASE + 1L)   /* ViInt32 */

/*****************************************************************************
 *---------------------------- Useful Macros --------------------------------*
 *****************************************************************************/

    /*- I/O buffer size -----------------------------------------------------*/
#define BUFFER_SIZE                             512L        
#define E3631A_IO_SESSION_TYPE   IVI_VAL_VISA_SESSION_TYPE

    /*- 488.2 Event Status Register (ESR) Bits ------------------------------*/
#define IEEE_488_2_QUERY_ERROR_BIT              0x04
#define IEEE_488_2_DEVICE_DEPENDENT_ERROR_BIT   0x08
#define IEEE_488_2_EXECUTION_ERROR_BIT          0x10
#define IEEE_488_2_COMMAND_ERROR_BIT            0x20


    /*- List of channels passed to the Ivi_BuildChannelTable function -------*/ 
#define CHANNEL_LIST                            "1,2,3"

/*****************************************************************************
 *-------------- Utility Function Declarations (Non-Exported) ---------------*
 *****************************************************************************/
static ViStatus E3631A_InitAttributes (ViSession vi);
static ViStatus E3631A_DefaultInstrSetup (ViSession openInstrSession);
static ViStatus E3631A_CheckStatus (ViSession vi);
static ViStatus E3631A_WaitForOPC (ViSession vi, ViInt32 maxTime);
static ViStatus E3631A_GetDriverSetupOption (ViSession vi,
                                               ViConstString optionTag,
                                               ViInt32 optionValueSize,
                                               ViChar optionValue[]);

    /*- File I/O Utility Functions -*/
static ViStatus E3631A_ReadToFile (ViSession vi, ViConstString filename, 
                                     ViInt32 maxBytesToRead, ViInt32 fileAction, ViInt32 *totalBytesWritten);
static ViStatus E3631A_WriteFromFile (ViSession vi, ViConstString filename, 
                                        ViInt32 maxBytesToWrite, ViInt32 byteOffset, 
                                        ViInt32 *totalBytesWritten);

/*****************************************************************************
 *----------------- Callback Declarations (Non-Exported) --------------------*
 *****************************************************************************/

    /*- Global Session Callbacks --------------------------------------------*/
    
static ViStatus _VI_FUNC E3631A_CheckStatusCallback (ViSession vi, ViSession io);
static ViStatus _VI_FUNC E3631A_WaitForOPCCallback (ViSession vi, ViSession io);

    /*- Attribute callbacks -------------------------------------------------*/

static ViStatus _VI_FUNC E3631AAttrDriverRevision_ReadCallback (ViSession vi,
                                                                  ViSession io, 
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId, 
                                                                  const ViConstString cacheValue);
static ViStatus _VI_FUNC E3631AAttrInstrumentManufacturer_ReadCallback (ViSession vi, 
                                                                          ViSession io,
                                                                          ViConstString channelName, 
                                                                          ViAttr attributeId,
                                                                          const ViConstString cacheValue);
static ViStatus _VI_FUNC E3631AAttrInstrumentModel_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   const ViConstString cacheValue);

static ViStatus _VI_FUNC E3631AAttrFirmwareRevision_ReadCallback (ViSession vi, 
                                                                    ViSession io,
                                                                    ViConstString channelName, 
                                                                    ViAttr attributeId,
                                                                    const ViConstString cacheValue);
static ViStatus _VI_FUNC E3631AAttrIdQueryResponse_ReadCallback (ViSession vi,
                                                                   ViSession io, 
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId, 
                                                                   const ViConstString cacheValue);
static ViStatus _VI_FUNC E3631AAttrOutputEnabled_WriteCallback (ViSession vi, 
                                                                 ViSession io,
                                                                 ViConstString channelName, 
                                                                 ViAttr attributeId,
                                                                 ViBoolean value);
static ViStatus _VI_FUNC E3631AAttrOutputEnabled_ReadCallback (ViSession vi, 
                                                                ViSession io,
                                                                ViConstString channelName, 
                                                                ViAttr attributeId,
                                                                ViBoolean* value);
static ViStatus _VI_FUNC E3631AAttrVoltageLevel_WriteCallback (ViSession vi, 
                                                                ViSession io,
                                                                ViConstString channelName, 
                                                                ViAttr attributeId,
                                                                ViReal64 value);
static ViStatus _VI_FUNC E3631AAttrVoltageLevel_ReadCallback (ViSession vi, 
                                                               ViSession io,
                                                               ViConstString channelName, 
                                                               ViAttr attributeId,
                                                               ViReal64* value);
static ViStatus _VI_FUNC E3631AAttrOVPEnabled_WriteCallback (ViSession vi, 
                                                              ViSession io,
                                                              ViConstString channelName, 
                                                              ViAttr attributeId,
                                                              ViBoolean value);
static ViStatus _VI_FUNC E3631AAttrOVPEnabled_ReadCallback (ViSession vi, 
                                                             ViSession io,
                                                             ViConstString channelName, 
                                                             ViAttr attributeId,
                                                             ViBoolean* value);
static ViStatus _VI_FUNC E3631AAttrOVPLimit_WriteCallback (ViSession vi, 
                                                            ViSession io,
                                                            ViConstString channelName, 
                                                            ViAttr attributeId,
                                                            ViReal64 value);
static ViStatus _VI_FUNC E3631AAttrOVPLimit_ReadCallback (ViSession vi, 
                                                           ViSession io,
                                                           ViConstString channelName, 
                                                           ViAttr attributeId,
                                                           ViReal64* value);
static ViStatus _VI_FUNC E3631AAttrCurrentLimitBehavior_WriteCallback (ViSession vi, 
                                                                        ViSession io,
                                                                        ViConstString channelName, 
                                                                        ViAttr attributeId,
                                                                        ViInt32 value);
static ViStatus _VI_FUNC E3631AAttrCurrentLimitBehavior_ReadCallback (ViSession vi, 
                                                                       ViSession io,
                                                                       ViConstString channelName, 
                                                                       ViAttr attributeId,
                                                                       ViInt32* value);
static ViStatus _VI_FUNC E3631AAttrCurrentLimit_WriteCallback (ViSession vi, 
                                                                ViSession io,
                                                                ViConstString channelName, 
                                                                ViAttr attributeId,
                                                                ViReal64 value);
static ViStatus _VI_FUNC E3631AAttrCurrentLimit_ReadCallback (ViSession vi, 
                                                               ViSession io,
                                                               ViConstString channelName, 
                                                               ViAttr attributeId,
                                                               ViReal64* value);
static ViStatus _VI_FUNC E3631AAttrTriggerSource_WriteCallback (ViSession vi, 
                                                                ViSession io,
                                                                ViConstString channelName, 
                                                                ViAttr attributeId,
                                                                ViInt32 value);
static ViStatus _VI_FUNC E3631AAttrTriggerSource_ReadCallback (ViSession vi, 
                                                                ViSession io,
                                                                ViConstString channelName, 
                                                                ViAttr attributeId,
                                                                ViInt32* value);
static ViStatus _VI_FUNC E3631AAttrTrigVoltageLevel_WriteCallback (ViSession vi, 
                                                                    ViSession io,
                                                                    ViConstString channelName, 
                                                                    ViAttr attributeId,
                                                                    ViReal64 value);
static ViStatus _VI_FUNC E3631AAttrTrigVoltageLevel_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   ViReal64* value);
static ViStatus _VI_FUNC E3631AAttrTrigCurrentLimit_WriteCallback (ViSession vi, 
                                                                    ViSession io,
                                                                    ViConstString channelName, 
                                                                    ViAttr attributeId,
                                                                    ViReal64 value);
static ViStatus _VI_FUNC E3631AAttrTrigCurrentLimit_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   ViReal64* value);

/*****************************************************************************
 *------------ User-Callable Functions (Exportable Functions) ---------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: E3631A_init   
 * Purpose:  VXIplug&play required function.  Calls the   
 *           E3631A_InitWithOptions function.   
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newVi)
{   
    ViStatus    error = VI_SUCCESS;

    if (newVi == VI_NULL)
        {
        Ivi_SetErrorInfo (VI_NULL, VI_FALSE, IVI_ERROR_INVALID_PARAMETER, 
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( IVI_ERROR_INVALID_PARAMETER);
        }

    checkErr( E3631A_InitWithOptions (resourceName, IDQuery, resetDevice, 
                                        "", newVi));

Error:  
    return error;
}

/*****************************************************************************
 * Function: E3631A_InitWithOptions                                       
 * Purpose:  This function creates a new IVI session and calls the 
 *           IviInit function.                                     
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                            ViBoolean resetDevice, ViString optionString, 
                                            ViSession *newVi)
{   
    ViStatus    error = VI_SUCCESS;
    ViSession   vi = VI_NULL;
    ViChar      newResourceName[IVI_MAX_MESSAGE_BUF_SIZE];
    ViChar      newOptionString[IVI_MAX_MESSAGE_BUF_SIZE];
    ViBoolean   isLogicalName;
    
    if (newVi == VI_NULL)
        {
        Ivi_SetErrorInfo (VI_NULL, VI_FALSE, IVI_ERROR_INVALID_PARAMETER, 
                          VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( IVI_ERROR_INVALID_PARAMETER);
        }

    *newVi = VI_NULL;
    
    checkErr( Ivi_GetInfoFromResourceName (resourceName, optionString, newResourceName,
                                           newOptionString, &isLogicalName));
    
        /* create a new interchangeable driver */
    checkErr( Ivi_SpecificDriverNew ("E3631A", newOptionString, &vi));  
    
        /* init the driver */
    checkErr( E3631A_IviInit (newResourceName, IDQuery, resetDevice, vi)); 
    if (isLogicalName)
        checkErr( Ivi_ApplyDefaultSetup (vi));
    *newVi = vi;
    
Error:
    if (error < VI_SUCCESS) 
        Ivi_Dispose (vi);
        
    return error;
}

/*****************************************************************************
 * Function: E3631A_IviInit                                                       
 * Purpose:  This function is called by E3631A_InitWithOptions  
 *           or by an IVI class driver.  This function initializes the I/O 
 *           interface, optionally resets the device, optionally performs an
 *           ID query, and sends a default setup to the instrument.                
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_IviInit (ViRsrc resourceName, ViBoolean IDQuery,
                                    ViBoolean reset, ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
    ViSession   io = VI_NULL;
    
    checkErr( Ivi_BuildChannelTable (vi, CHANNEL_LIST, VI_FALSE, VI_NULL));
    
     /* Add attributes */
    checkErr( E3631A_InitAttributes (vi));

    if (!Ivi_Simulating(vi))
        {
        ViSession   rmSession = VI_NULL;

        /* Open instrument session */
        checkErr( Ivi_GetAttributeViSession (vi, VI_NULL, IVI_ATTR_VISA_RM_SESSION, 0,
                                             &rmSession)); 
        viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &io));
        /* io session owned by driver now */
        checkErr( Ivi_SetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, io));  

		/* Configure VISA Formatted I/O */
		viCheckErr( viSetAttribute (io, VI_ATTR_TMO_VALUE, 5000 ));
		viCheckErr( viSetBuf (io, VI_READ_BUF | VI_WRITE_BUF, 4000));
		viCheckErr( viSetAttribute (io, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
		viCheckErr( viSetAttribute (io, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
        }
        
    /*- Reset instrument ----------------------------------------------------*/
    if (reset) 
        checkErr( E3631A_reset (vi));
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( E3631A_DefaultInstrSetup (vi));
	
	/*- Identification Query ------------------------------------------------*/
	if (IDQuery)                               
	    {
	    ViChar rdBuffer[BUFFER_SIZE];
	
	    #define VALID_RESPONSE_STRING_START        "HEWLETT-PACKARD,E3631A,0,1.4-5.0-1.0"
	
	    checkErr( Ivi_GetAttributeViString (vi, VI_NULL, E3631A_ATTR_ID_QUERY_RESPONSE, 
	                                        0, BUFFER_SIZE, rdBuffer));
	    
	    if (strncmp (rdBuffer, VALID_RESPONSE_STRING_START, 
	                 strlen(VALID_RESPONSE_STRING_START)) != 0)
	        {             
	        viCheckErr( VI_ERROR_FAIL_ID_QUERY);
	        }
	    }

    checkErr( E3631A_CheckStatus (vi));

Error:
    if (error < VI_SUCCESS)
        {
        if (!Ivi_Simulating (vi) && io)
            viClose (io);
        }
    return error;
}

/*****************************************************************************
 * Function: E3631A_close                                                           
 * Purpose:  This function closes the instrument.                            
 *
 *           Note:  This function must unlock the session before calling
 *           Ivi_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_close (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    checkErr( E3631A_IviClose (vi));

Error:    
    Ivi_UnlockSession (vi, VI_NULL);
    Ivi_Dispose (vi);  

    return error;
}

/*****************************************************************************
 * Function: E3631A_IviClose                                                        
 * Purpose:  This function performs all of the drivers clean-up operations   
 *           except for closing the IVI session.  This function is called by 
 *           E3631A_close or by an IVI class driver. 
 *
 *           Note:  This function must close the I/O session and set 
 *           IVI_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_IviClose (ViSession vi)
{
    ViStatus error = VI_SUCCESS;
    ViSession io = VI_NULL;

        /* Do not lock here.  The caller manages the lock. */

    checkErr( Ivi_GetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, &io));

Error:
    Ivi_SetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, VI_NULL);
    if(io)                                                      
        {
        viClose (io);
        }
    return error;   
}

/*****************************************************************************
 * Function: E3631A_reset                                                         
 * Purpose:  This function resets the instrument.                          
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_reset (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    viCheckErr( viPrintf (io, "*RST"));
	    }
	
	checkErr( E3631A_DefaultInstrSetup (vi));                                
	
	checkErr( E3631A_CheckStatus (vi));                                      

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: E3631A_self_test                                                       
 * Purpose:  This function executes the instrument self-test and returns the 
 *           result.                                                         
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_self_test (ViSession vi, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    if (testResult == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    viCheckErr( viPrintf (io, "*TST?"));
	        
		viCheckErr( viScanf (io, "%hd", testResult));
		
		testMessage[0] = 0;
	
	    }
	else
	    {
	        /* Simulate Self Test */
	    *testResult = 0;
	    strcpy (testMessage, "No error.");
	    }
	
	checkErr( E3631A_CheckStatus (vi));

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: E3631A_error_query                                                     
 * Purpose:  This function queries the instrument error queue and returns   
 *           the result.                                                     
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_error_query (ViSession vi, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (errCode == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    viCheckErr( viPrintf(io, ":SYST:ERR?"));
	
		viCheckErr( viScanf (io, "%ld,\"%256[^\"]", errCode, errMessage));
	
	    }
	else
	    {
	        /* Simulate Error Query */
	    *errCode = 0;
	    strcpy (errMessage, "No error.");
	    }

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: E3631A_error_message                                                  
 * Purpose:  This function translates the error codes returned by this       
 *           instrument driver into user-readable strings.  
 *
 *           Note:  The caller can pass VI_NULL for the vi parameter.  This 
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_error_message (ViSession vi, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error = VI_SUCCESS;
    
        static      IviStringValueTable errorTable = 
        {
            {E3631A_ERROR_RESET_PROT_NOT_SUPPORTED,    "The Reset Output Protection is not supported."},
            {E3631A_ERROR_TRIGGER_ABORT_NOT_SUPPORTED, "The Abort trigger is not supported."},
            IVIDCPWR_ERROR_CODES_AND_MSGS,
            {VI_NULL,                               VI_NULL}
        };


        
    if (vi)
        Ivi_LockSession(vi, VI_NULL);

        /* all VISA and IVI error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Ivi_GetSpecificDriverStatusDesc(vi, errorCode, errorMessage, errorTable));

Error:
    if (vi)
        Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: E3631A_revision_query                                                  
 * Purpose:  This function returns the driver and instrument revisions.      
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_revision_query (ViSession vi, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Ivi_GetAttributeViString (vi, VI_NULL, E3631A_ATTR_SPECIFIC_DRIVER_REVISION, 
                                        0, -1, driverRev));
    checkErr( Ivi_GetAttributeViString (vi, "", E3631A_ATTR_FIRMWARE_REVISION, 
                                        0, -1, instrRev));
    checkErr( E3631A_CheckStatus (vi));

Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: E3631A_ConfigureOutputEnabled                                   
 * Purpose:  This function controls whether or not the signal the power supply
 *           generates is present at the output terminal.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_ConfigureOutputEnabled (ViSession vi,
                                                   ViConstString channelName,
                                                   ViBoolean enabled)
{
    return Ivi_SetAttributeViBoolean (vi, channelName, E3631A_ATTR_OUTPUT_ENABLED,
                                      IVI_VAL_DIRECT_USER_CALL, enabled);
}

/*****************************************************************************
 * Function: E3631A_ConfigureOutputRange                                   
 * Purpose:  This function determines the output range in which the power
 *           supply operates.
 *****************************************************************************/
static IviRangeTableEntry rangeTypeRangeTableEntries[] = 
    {   
        {E3631A_VAL_RANGE_CURRENT, 0, 0, "CURR", E3631A_ATTR_CURRENT_LIMIT},
        {E3631A_VAL_RANGE_VOLTAGE, 0, 0, "VOLT", E3631A_ATTR_VOLTAGE_LEVEL},
        {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable rangeTypeRangeTable = 
    {
        IVI_VAL_DISCRETE,
        VI_FALSE,
        VI_FALSE,
        VI_NULL,
        rangeTypeRangeTableEntries
    };

ViStatus _VI_FUNC E3631A_ConfigureOutputRange (ViSession vi,
                                                 ViConstString channelName,
                                                 ViInt32 rangeType,
                                                 ViReal64 range)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrId;
    ViString cmd;
    
    checkErr( Ivi_LockSession(vi, VI_NULL));
    
    /*=CHANGE:=============================================================*
        NOTE TO THE DEVELOPER: Some power supplies have multiple output ranges,
        and you must explicitly specify the output range in which you wish to
        operate.  Other power supplies automatically alter the output range,
        and you cannot explicitly specify the output range.
        The two blocks of example code below cover these two possibilities.
        
        - You should remove the code that is not appropriate for your instrument.
        - The rangeTypeRangeTable is used by both example code blocks.  This
          range table should be modified to contain only the information needed
          by the block of code you choosed to use.
          
        If your instrument has only one output range, then this function
        simply verifies that the range specified is within the valid range
        of the instrument.
        
        If the instrument's output range must be explicitly specified, this
        function places the instrument in the smallest output range that
        can generate the requested value.
     *==========================================================END=CHANGE=*/
    /*=CHANGE:=============================================================*
        This code is an example of what to do if the instrument has only one
        range.  */
    viCheckParm( Ivi_GetViInt32EntryFromValue (rangeType,
                                            &rangeTypeRangeTable, 
                                            VI_NULL, VI_NULL, VI_NULL,
                                            VI_NULL, VI_NULL, &attrId),
											3, "Range Type");
    viCheckParm( Ivi_CheckAttributeViReal64 (vi, channelName, attrId, 0, range),
											 4, "Output Range");
    /*==========================================================END=CHANGE=*/

    /*=CHANGE:=============================================================*
        This code is an example of what to do if the instrument's range
        must be set explicitly.
     */   
    viCheckParm( Ivi_GetViInt32EntryFromValue (rangeType,
                                            &rangeTypeRangeTable, 
                                            VI_NULL, VI_NULL, VI_NULL,
                                            VI_NULL, &cmd, VI_NULL),
											3, "Range Type");
    if (Ivi_RangeChecking(vi))
        {
        checkErr( Ivi_CoerceChannelName (vi, channelName, VI_NULL));
        
        //Insert code here to verify that the range parameter is a valid value
        }
    if (!Ivi_Simulating (vi))
        {
        ViSession io = Ivi_IOSession(vi);
        
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
        
        viCheckErr( viPrintf (io, ":OUTP:%s:RANGE %lf", cmd, range));
        }
     
     checkErr( Ivi_InvalidateAttribute (vi, channelName, E3631A_ATTR_CURRENT_LIMIT));
     checkErr( Ivi_InvalidateAttribute (vi, channelName, E3631A_ATTR_VOLTAGE_LEVEL));
    /*==========================================================END=CHANGE=*/

    checkErr( E3631A_CheckStatus (vi));

Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
            
}

/*****************************************************************************
 * Function: E3631A_ConfigureCurrentLimit                                   
 * Purpose:  This function configures the behavior and level of the
 *           current limit.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_ConfigureCurrentLimit (ViSession vi,
                                                  ViConstString channelName,
                                                  ViInt32 behavior,
                                                  ViReal64 limit)
{
    ViStatus error = VI_SUCCESS;
    ViBoolean behaviorFirst = VI_FALSE;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /*=CHANGE:=============================================================*
        NOTE TO THE DEVELOPER: Since the dependency of these attributes 
        varies between instruments, you might have to rearrange the order
        in which they are set so as to achieve expected behavior.
     *==========================================================END=CHANGE=*/

    //If you are setting the instrument to trip, set the limit and then the behavior
    //to prevent a premature over-current condition.
    
    behaviorFirst = (behavior != E3631A_VAL_CURRENT_TRIP);
    
    if (behaviorFirst)
        {
        viCheckParm( Ivi_SetAttributeViInt32 (vi, channelName, 
                                              E3631A_ATTR_CURRENT_LIMIT_BEHAVIOR,
                                              0, behavior), 3,
                                              "Current Limit Behavior");
        }
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channelName, 
                                               E3631A_ATTR_CURRENT_LIMIT,
                                               0, limit), 4,
                                               "Current Limit");
    if (!behaviorFirst)        
        {
        viCheckParm( Ivi_SetAttributeViInt32 (vi, channelName, 
                                              E3631A_ATTR_CURRENT_LIMIT_BEHAVIOR,
                                              0, behavior), 3,
                                              "Current Limit Behavior");
        }
Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: E3631A_ConfigureOVP                                   
 * Purpose:  This function enables or disables over-voltage protection and
 *           configures the over-voltage limit.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_ConfigureOVP (ViSession vi,
                                         ViConstString channelName,
                                         ViBoolean enabled,
                                         ViReal64 limit)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    /*=CHANGE:=============================================================*
        NOTE TO THE DEVELOPER: Since the dependency of these attributes 
        varies between instruments, you might have to rearrange the order
        in which they are set so as to achieve expected behavior.
     *==========================================================END=CHANGE=*/
    
    //If you are enabling over-voltage protection, set the limit first
    //to avoid a premature over-voltage condition.
    if (enabled)
        {
        viCheckParm( Ivi_SetAttributeViReal64 (vi, channelName, 
                                           E3631A_ATTR_OVP_LIMIT,
                                           0, limit), 4, "OVP Limit");
        }
    viCheckParm( Ivi_SetAttributeViBoolean (vi, channelName, 
                                            E3631A_ATTR_OVP_ENABLED,
                                            0, enabled), 3, "OVP Enabled");
            
Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: E3631A_ConfigureVoltageLevel                                   
 * Purpose:  This function configures the voltage level the power supply
 *           attempts to generate.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_ConfigureVoltageLevel (ViSession vi,
                                                  ViConstString channelName,
                                                  ViReal64 level)
{
    return Ivi_SetAttributeViReal64 (vi, channelName, E3631A_ATTR_VOLTAGE_LEVEL,
                                     IVI_VAL_DIRECT_USER_CALL, level);
}

/*****************************************************************************
 * Function: E3631A_QueryOutputState                                   
 * Purpose:  This function returns a value indicating if the power supply
 *           is in a particular output state.
 *****************************************************************************/
static IviRangeTableEntry outputStateRangeTableEntries[] =
    {
        {E3631A_VAL_OUTPUT_CONSTANT_VOLTAGE, 0, 0, "CV",  0},
        {E3631A_VAL_OUTPUT_CONSTANT_CURRENT, 0, 0, "CC",  0},
        {E3631A_VAL_OUTPUT_OVER_VOLTAGE,     0, 0, "OV",  0},
        {E3631A_VAL_OUTPUT_OVER_CURRENT,     0, 0, "OC",  0},
        {E3631A_VAL_OUTPUT_UNREGULATED,      0, 0, "UNR", 0},
        {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable outputStateRangeTable = 
    {
        IVI_VAL_DISCRETE,
        VI_FALSE,
        VI_FALSE,
        VI_NULL,
        outputStateRangeTableEntries
    };

ViStatus _VI_FUNC E3631A_QueryOutputState (ViSession vi,
                                             ViConstString channelName,
                                             ViInt32 outputState,
                                             ViBoolean* inState)
{
    ViStatus error = VI_SUCCESS;
    ViBoolean value = VI_FALSE;
    
    checkErr( Ivi_LockSession(vi, VI_NULL));
    
    /*=CHANGE:=============================================================*
        Modify this code to return whether the instrument is in the
        requested state.  The example code verifies the channelName
        parameter, and then returns a whether the instrument is in the
        state specified by the outputState parameter.
	*/
    if (inState == VI_NULL)
			viCheckParm (IVI_ERROR_INVALID_PARAMETER, 2, "Null value for inState.");
	
	if (Ivi_RangeChecking(vi))
        {
        viCheckParm( Ivi_CoerceChannelName (vi, channelName, VI_NULL),
											2, "Channel Name");
        }
    
	if (!Ivi_Simulating (vi))
        {
        ViSession io = Ivi_IOSession(vi);
        ViString cmd;
        
        viCheckParm( Ivi_GetViInt32EntryFromValue (outputState,
                                                  &outputStateRangeTable,
                                                  VI_NULL, VI_NULL, VI_NULL, 
                                                  VI_NULL, &cmd, VI_NULL),
												  3, "Output State");
                                                  
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
        viCheckErr( viQueryf (io, "OUTP:%s:COND?", "%hd", cmd, &value));
        }

     /*==========================================================END=CHANGE=*/
    checkErr( E3631A_CheckStatus (vi));

Error:    
    if (inState) *inState = value;
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
            
}

/*****************************************************************************
 * Function: E3631A_ResetOutputProtection                                   
 * Purpose:  This function resets the power supply's output protection.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_ResetOutputProtection (ViSession vi,
                                                  ViConstString channelName)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession(vi, VI_NULL));
    
    /*=CHANGE:=============================================================*
        Modify this code to reset the output protection.  The example
        code below sends a command that attempts to return the power supply
        to the state it was in before the protection system disabled the output
	 */
    if (Ivi_RangeChecking(vi))
        {
        viCheckParm( Ivi_CoerceChannelName (vi, channelName, VI_NULL),
										    2, "Channel Name");
        }
    if (!Ivi_Simulating (vi))
        {
        ViSession io = Ivi_IOSession(vi);
        
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
        
        viCheckErr( viPrintf (io, "OUTP:PROT:CLE"));
        }
    checkErr( E3631A_CheckStatus (vi));
    
     /*==========================================================END=CHANGE=*/

Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
            
}

/*****************************************************************************
 * Function: E3631A_ConfigureTriggerSource                                   
 * Purpose:  This function configures the trigger source.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_ConfigureTriggerSource (ViSession vi,
                                                  ViConstString channelName,
                                                  ViInt32 source)
{
    return Ivi_SetAttributeViInt32 (vi, channelName, E3631A_ATTR_TRIGGER_SOURCE,
                                    IVI_VAL_DIRECT_USER_CALL, source);
}

/*****************************************************************************
 * Function: E3631A_ConfigureTriggeredVoltageLevel                                   
 * Purpose:  This function configures the voltage level the power supply
 *           attempts to generate after it receives a trigger.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_ConfigureTriggeredVoltageLevel (ViSession vi,
                                                           ViConstString channelName,
                                                           ViReal64 level)
{
    return Ivi_SetAttributeViReal64 (vi, channelName,
                                     E3631A_ATTR_TRIGGERED_VOLTAGE_LEVEL,
                                     IVI_VAL_DIRECT_USER_CALL, level);
}

/*****************************************************************************
 * Function: E3631A_ConfigureTriggeredCurrentLimit                                   
 * Purpose:  This function configures the current limit the power supply
 *           attempts to generate after it receives a trigger.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_ConfigureTriggeredCurrentLimit (ViSession vi,
                                                           ViConstString channelName,
                                                           ViReal64 limit)
{
    return Ivi_SetAttributeViReal64 (vi, channelName,
                                     E3631A_ATTR_TRIGGERED_CURRENT_LIMIT,
                                     IVI_VAL_DIRECT_USER_CALL, limit);
}

/*****************************************************************************
 * Function: E3631A_Abort                                   
 * Purpose:  This function causes the instrument to no longer wait for a
 *           trigger.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_Abort (ViSession vi)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession(vi, VI_NULL));
    
    /*=CHANGE:=============================================================*
        Modify this code to abort the trigger system.  The example
        code below invalidates the triggered voltage level and triggered
        current limit attributes, as these values might not be valid after
        aborting the trigger system.
	*/
    checkErr( Ivi_InvalidateAttribute (vi, IVI_VAL_ALL_CHANNELS,
                                       E3631A_ATTR_TRIGGERED_VOLTAGE_LEVEL));
    checkErr( Ivi_InvalidateAttribute (vi, IVI_VAL_ALL_CHANNELS,
                                       E3631A_ATTR_TRIGGERED_CURRENT_LIMIT));
    if (!Ivi_Simulating (vi))
        {
        ViSession io = Ivi_IOSession(vi);
        
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
        
        viCheckErr( viPrintf (io, "ABOR"));
        }
    
     /*==========================================================END=CHANGE=*/

    /* 
        Do not invoke the E3631A_CheckStatus function here.  It 
        is invoked by the high-level driver functions when this function is 
        used internally.  After the user calls this function, the user can check
        for errors by calling the E3631A_error_query function.
    */

Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
            
}

/*****************************************************************************
 * Function: E3631A_Initiate                                   
 * Purpose:  This function causes the instrument to wait for a trigger from
 *           the trigger source.  After it receives a trigger, it updates the
 *           triggered current limit and triggered voltage level.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_Initiate (ViSession vi)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession(vi, VI_NULL));
    
    /*=CHANGE:=============================================================*
        Modify this code to initiate the trigger system.  The example
        code below invalidates the voltage level and current limit attributes,
        as these values change to the triggered values after a trigger occurs.
	 */
    checkErr( Ivi_InvalidateAttribute (vi, IVI_VAL_ALL_CHANNELS,
                                       E3631A_ATTR_VOLTAGE_LEVEL));
    checkErr( Ivi_InvalidateAttribute (vi, IVI_VAL_ALL_CHANNELS,
                                       E3631A_ATTR_CURRENT_LIMIT));
    if (!Ivi_Simulating (vi))
        {
        ViSession io = Ivi_IOSession(vi);
        
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
        
        viCheckErr( viPrintf (io, "INIT"));
        }
    
     /*==========================================================END=CHANGE=*/

    /* 
        Do not invoke the E3631A_CheckStatus function here.  It 
        is invoked by the high-level driver functions when this function is 
        used internally.  After the user calls this function, the user can check
        for errors by calling the E3631A_error_query function.
    */

Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
            
}

/*****************************************************************************
 * Function: E3631A_SendSoftwareTrigger                                   
 * Purpose:  This function sends a software trigger to the instrument.  If the
 *           instrument is not configured to receive a software trigger, this
 *           function returns an error.
 *****************************************************************************/
    /*=CHANGE:=============================================================*
        NOTE TO THE DEVELOPER: Do not implement this function unless your
        DC power supply allows the trigger source to be set to respond
        exclusively to software triggers.  If your DC power supply cannot
        support an exclusive software trigger, but does support a software
        trigger that overrides the specified trigger source, rename this
        function to "E3631A_ForceTrigger" and remove the code
        that checks if the trigger source is set to software.
     *==========================================================END=CHANGE=*/
ViStatus _VI_FUNC E3631A_SendSoftwareTrigger (ViSession vi)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession(vi, VI_NULL));
    
    /*=CHANGE:=============================================================*
        Modify this code to send a trigger command to the instrument when
        the trigger source is set to software, and return an error when it 
        is not.  The example code verifies that at least one channel's
        trigger source is set to the software trigger value, and then sends
        a trigger command if the driver is not simulating.
	*/
    if (Ivi_RangeChecking (vi))
        {
        ViBoolean isSoftware;
        ViInt32 numChans;
        ViString channel;
        ViInt32 i;
        ViInt32 source;
        
        checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, 
                                           E3631A_ATTR_NUM_CHANNELS,
                                           0, &numChans));
        for (i = 1; i <= numChans && !isSoftware; i++)
            {
            checkErr( Ivi_GetNthChannelString (vi, i, &channel));
            checkErr( Ivi_GetAttributeViInt32 (vi, channel,
                                               E3631A_ATTR_TRIGGER_SOURCE,
                                               0, &source));
            isSoftware = (source == E3631A_VAL_SOFTWARE_TRIG);
            }
        if (!isSoftware)
            {
            viCheckErr (E3631A_ERROR_TRIGGER_NOT_SOFTWARE);
            }
        }
    if (!Ivi_Simulating (vi))
        {
        ViSession io = Ivi_IOSession(vi);
        
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
        
        viCheckErr( viPrintf (io, "TRIG"));
        }
     /*==========================================================END=CHANGE=*/
	 
    /* 
        Do not invoke the E3631A_CheckStatus function here.  It 
        is invoked by the high-level driver functions when this function is 
        used internally.  After the user calls this function, the user can check
        for errors by calling the E3631A_error_query function.
    */

Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
            
}


/*****************************************************************************
 * Function: E3631A_Measure                                   
 * Purpose:  This function measures either the current or voltage that the
 *           power supply is generating.
 *****************************************************************************/
static IviRangeTableEntry measurementTypeRangeTableEntries[] = 
    {   
        {E3631A_VAL_MEASURE_CURRENT, 0, 0, "CURR", E3631A_ATTR_CURRENT_LIMIT},
        {E3631A_VAL_MEASURE_VOLTAGE, 0, 0, "VOLT", E3631A_ATTR_VOLTAGE_LEVEL},
        {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable measurementTypeRangeTable = 
    {
        IVI_VAL_DISCRETE,
        VI_FALSE,
        VI_FALSE,
        VI_NULL,
        measurementTypeRangeTableEntries
    };

ViStatus _VI_FUNC E3631A_Measure (ViSession vi,
                                   ViConstString channelName,
                                   ViInt32 measurementType,
                                   ViReal64* measurement)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 value = 0.0;
    ViAttr   attrId;
    ViString cmd;
    
    checkErr( Ivi_LockSession(vi, VI_NULL));
    
    /*=CHANGE:=============================================================*
        Modify this code to take the requested measurement.  The example
        code verifies the channelName parameter, and then returns
        a measurement.
	 */
    if (measurement == VI_NULL)
		viCheckParm (IVI_ERROR_INVALID_PARAMETER, 4, "Null value for measurement.");

	if (Ivi_RangeChecking (vi))
        {
        viCheckParm( Ivi_CoerceChannelName (vi, channelName, VI_NULL),
											2, "Channel Name");
        }
    viCheckParm( Ivi_GetViInt32EntryFromValue (measurementType,
											   &measurementTypeRangeTable, VI_NULL,
											   VI_NULL, VI_NULL, VI_NULL, &cmd,&attrId),
											   3, "measurementType");
											        
	 if (!Ivi_Simulating (vi))
	 {
	 	ViSession io = Ivi_IOSession (vi);
	 	checkErr(Ivi_SetNeedToCheckStatus(vi, VI_TRUE));
	 	viCheckErr(viQueryf (io,":MEAS:%s?", "%lf", cmd, &value));
	 }
	 else
	 {
	 	checkErr(Ivi_GetAttributeViReal64(vi, channelName, attrId,0, &value));
	 	value *= 0.95;
	 }
    

     /*==========================================================END=CHANGE=*/

    checkErr( E3631A_CheckStatus (vi));

Error:    
    if (measurement) *measurement = value;
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
            
}

/*****************************************************************************
 * Function: E3631A_GetAttribute<type> Functions                                    
 * Purpose:  These functions enable the instrument driver user to get 
 *           attribute values directly.  There are typesafe versions for 
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.                                         
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_GetAttributeViInt32 (ViSession vi, ViConstString channelName, 
                                                ViAttr attributeId, ViInt32 *value)
{                                                                                                           
    return Ivi_GetAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                    value);
}                                                                                                           
ViStatus _VI_FUNC E3631A_GetAttributeViReal64 (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViReal64 *value)
{                                                                                                           
    return Ivi_GetAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     value);
}                                                                                                           
ViStatus _VI_FUNC E3631A_GetAttributeViString (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViInt32 bufSize, 
                                                 ViChar value[]) 
{   
    return Ivi_GetAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     bufSize, value);
}   
ViStatus _VI_FUNC E3631A_GetAttributeViBoolean (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViBoolean *value)
{                                                                                                           
    return Ivi_GetAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           
ViStatus _VI_FUNC E3631A_GetAttributeViSession (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViSession *value)
{                                                                                                           
    return Ivi_GetAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           

/*****************************************************************************
 * Function: E3631A_SetAttribute<type> Functions                                    
 * Purpose:  These functions enable the instrument driver user to set 
 *           attribute values directly.  There are typesafe versions for 
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.                                         
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_SetAttributeViInt32 (ViSession vi, ViConstString channelName, 
                                                ViAttr attributeId, ViInt32 value)
{                                                                                                           
    return Ivi_SetAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                    value);
}                                                                                                           
ViStatus _VI_FUNC E3631A_SetAttributeViReal64 (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViReal64 value)
{                                                                                                           
    return Ivi_SetAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     value);
}                                                                                                           
ViStatus _VI_FUNC E3631A_SetAttributeViString (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViConstString value) 
{   
    return Ivi_SetAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     value);
}   
ViStatus _VI_FUNC E3631A_SetAttributeViBoolean (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViBoolean value)
{                                                                                                           
    return Ivi_SetAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           
ViStatus _VI_FUNC E3631A_SetAttributeViSession (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViSession value)
{                                                                                                           
    return Ivi_SetAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           

/*****************************************************************************
 * Function: E3631A_CheckAttribute<type> Functions                                  
 * Purpose:  These functions enable the instrument driver user to check  
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the E3631A_ATTR_RANGE_CHECK 
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32, 
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.                         
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_CheckAttributeViInt32 (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViInt32 value)
{                                                                                                           
    return Ivi_CheckAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}
ViStatus _VI_FUNC E3631A_CheckAttributeViReal64 (ViSession vi, ViConstString channelName, 
                                                   ViAttr attributeId, ViReal64 value)
{                                                                                                           
    return Ivi_CheckAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                       value);
}                                                                                                           
ViStatus _VI_FUNC E3631A_CheckAttributeViString (ViSession vi, ViConstString channelName, 
                                                   ViAttr attributeId, ViConstString value)  
{   
    return Ivi_CheckAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                       value);
}   
ViStatus _VI_FUNC E3631A_CheckAttributeViBoolean (ViSession vi, ViConstString channelName, 
                                                    ViAttr attributeId, ViBoolean value)
{                                                                                                           
    return Ivi_CheckAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                        value);
}                                                                                                           
ViStatus _VI_FUNC E3631A_CheckAttributeViSession (ViSession vi, ViConstString channelName, 
                                                    ViAttr attributeId, ViSession value)
{                                                                                                           
    return Ivi_CheckAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                        value);
}                                                                                                           

/*****************************************************************************
 * Function: E3631A_GetNextCoercionRecord                        
 * Purpose:  This function enables the instrument driver user to obtain
 *           the coercion information associated with the IVI session.                                                              
 *           This function retrieves and clears the oldest instance in which 
 *           the instrument driver coerced a value the instrument driver user
 *           specified to another value.                     
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_GetNextCoercionRecord (ViSession vi,
                                                  ViInt32 bufferSize,
                                                  ViChar  record[])
{
    return Ivi_GetNextCoercionString (vi, bufferSize, record);
}

/*****************************************************************************
 * Function: E3631A_LockSession and E3631A_UnlockSession Functions                        
 * Purpose:  These functions enable the instrument driver user to lock the 
 *           session around a sequence of driver calls during which other
 *           execution threads must not disturb the instrument state.
 *                                                                          
 *           NOTE:  The callerHasLock parameter must be a local variable 
 *           initialized to VI_FALSE and passed by reference, or you can pass 
 *           VI_NULL.                     
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_LockSession (ViSession vi, ViBoolean *callerHasLock)  
{                                              
    return Ivi_LockSession(vi,callerHasLock);      
}                                              
ViStatus _VI_FUNC E3631A_UnlockSession (ViSession vi, ViBoolean *callerHasLock) 
{                                              
    return Ivi_UnlockSession(vi,callerHasLock);    
}   

/*****************************************************************************
 * Function: E3631A_GetErrorInfo and E3631A_ClearErrorInfo Functions                       
 * Purpose:  These functions enable the instrument driver user to  
 *           get or clear the error information the driver associates with the
 *           IVI session.                                                        
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_GetErrorInfo (ViSession vi, ViStatus *primaryError, 
                                         ViStatus *secondaryError, ViChar errorElaboration[256])  
{                                                                                                           
    return Ivi_GetErrorInfo(vi, primaryError, secondaryError, errorElaboration);                                
}                                                                                                           
ViStatus _VI_FUNC E3631A_ClearErrorInfo (ViSession vi)                                                        
{                                                                                                           
    return Ivi_ClearErrorInfo (vi);                                                                             
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions                      
 * Purpose:  These functions enable the instrument driver user to  
 *           write and read commands directly to and from the instrument.            
 *                                                                           
 *           Note:  These functions bypass the IVI attribute state caching.  
 *                  WriteInstrData invalidates the cached values for all 
 *                  attributes.
 *****************************************************************************/
ViStatus _VI_FUNC E3631A_WriteInstrData (ViSession vi, ViConstString writeBuffer)   
{   
    return Ivi_WriteInstrData (vi, writeBuffer);    
}   
ViStatus _VI_FUNC E3631A_ReadInstrData (ViSession vi, ViInt32 numBytes, 
                                          ViChar rdBuf[], ViInt32 *bytesRead)  
{   
    return Ivi_ReadInstrData (vi, numBytes, rdBuf, bytesRead);   
} 

/*****************************************************************************
 *-------------------- Utility Functions (Not Exported) ---------------------*
 *****************************************************************************/
/*****************************************************************************
 *  Function: E3631A_GetDriverSetupOption
 *  Purpose:  Get the value of a DriverSetup option.  The driver setup string
 *            is part of the option string the user passes to the
 *            E3631A_InitWithOptions function.  Specifically it is the part
 *            that follows "DriverSetup=".  This function assumes the 
 *            DriverSetup string has the following format:
 *              "OptionTag1 : OptionValue1; OptionTag2 : OptionValue2"
 *            Leading and trailing white space is removed from the option
 *            value and both the option tag and option value may contain
 *            white space.  The option tag is case sensitive.
 *
 *            Example:
 *
 *              "DriverSetup= Model : InstrX ; Special Option:Feature Y "
 *
 *              optionTag "Model"          returns optionValue "InstrX"
 *
 *              optionTag "Special Option" returns optionValue "Feature Y"
 *****************************************************************************/
static ViStatus E3631A_GetDriverSetupOption (ViSession vi,
                                               ViConstString optionTag,
                                               ViInt32 optionValueSize,
                                               ViChar optionValue[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   setup[BUFFER_SIZE];
    ViChar*  searchOrigin;
    ViChar*  start;
    ViChar*  end;
    ViUInt32 n;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    if ((optionValue != VI_NULL) && (optionValueSize > 0))
        optionValue[0] = 0;

    if ((optionTag != VI_NULL) && (optionValue != VI_NULL) && (optionValueSize > 1))
        {
        checkErr( Ivi_GetAttributeViString (vi, "", IVI_ATTR_DRIVER_SETUP,
                                            0, BUFFER_SIZE, setup));
        searchOrigin = setup;
        while (searchOrigin != VI_NULL)
            {
            searchOrigin = strstr(searchOrigin, optionTag);
            if (searchOrigin)
                {
                searchOrigin += strlen(optionTag);
                start = searchOrigin;
                while (isspace(*start)) start++;
                if (*start == ':')
                    {
                    start++;
                    while (isspace(*start)) start++;
                    if (*start)
                        {
                        end = strchr (start, ';');
                        if (start != end)
                            {
                            end = (end) ? end-1 : &setup[strlen(setup)-1];
                            while (isspace(*end)) end--;
                            n = end - start + 1;
                            if (n > optionValueSize - 1)
                                n = optionValueSize - 1;
                            strncpy(optionValue, start, n);
                            optionValue[n] = 0;
                            break;
                            }
                        }
                    }
                }
            }
        }
        
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: E3631A_CheckStatus                                                 
 * Purpose:  This function checks the status of the instrument to detect 
 *           whether the instrument has encountered an error.  This function  
 *           is called at the end of most exported driver functions.  If the    
 *           instrument reports an error, this function returns      
 *           IVI_ERROR_INSTR_SPECIFIC.  The user can set the 
 *           IVI_ATTR_QUERY_INSTR_STATUS attribute to VI_FALSE to disable this 
 *           check and increase execution speed.                                   
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus E3631A_CheckStatus (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;

    if (Ivi_QueryInstrStatus (vi) && Ivi_NeedToCheckStatus (vi) && !Ivi_Simulating (vi))
        {
        checkErr( E3631A_CheckStatusCallback (vi, Ivi_IOSession(vi)));
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_FALSE));
        }
        
Error:
    return error;
}

/*****************************************************************************
 * Function: E3631A_WaitForOPC                                                  
 * Purpose:  This function waits for the instrument to complete the      
 *           execution of all pending operations.  This function is a        
 *           wrapper for the WaitForOPCCallback.  It can be called from 
 *           other instrument driver functions. 
 *
 *           The maxTime parameter specifies the maximum time to wait for
 *           operation complete in milliseconds.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus E3631A_WaitForOPC (ViSession vi, ViInt32 maxTime) 
{
    ViStatus    error = VI_SUCCESS;

    if (!Ivi_Simulating(vi))
        {
        ViInt32  oldOPCTimeout; 
        
        checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, E3631A_ATTR_OPC_TIMEOUT, 
                                           0, &oldOPCTimeout));
        Ivi_SetAttributeViInt32 (vi, VI_NULL, E3631A_ATTR_OPC_TIMEOUT,        
                                 0, maxTime);

        error = E3631A_WaitForOPCCallback (vi, Ivi_IOSession(vi));

        Ivi_SetAttributeViInt32 (vi, VI_NULL, E3631A_ATTR_OPC_TIMEOUT, 
                                 0, oldOPCTimeout);
        viCheckErr( error);
        }
Error:
    return error;
}

/*****************************************************************************
 * Function: E3631A_DefaultInstrSetup                                               
 * Purpose:  This function sends a default setup to the instrument.  The    
 *           E3631A_reset function calls this function.  The 
 *           E3631A_IviInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter.  This function is 
 *           useful for configuring settings that other instrument driver 
 *           functions require.    
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus E3631A_DefaultInstrSetup (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
        
    /* Invalidate all attributes */
    checkErr( Ivi_InvalidateAllAttributes (vi));
    
    if (!Ivi_Simulating(vi))
        {
        ViSession   io = Ivi_IOSession(vi); /* call only when locked */

        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
		
		/*=CHANGE:===============================================================*
		   Change the following command string so that it executes the default 
		   setup for your instrument.  The example does the following: 
		  
		   *CLS     clears the event/status registers                                      
		   *ESE 1   sets the standard event status enable register to recognize  
		            operation complete.                                          
		   *SRE 32  sets the service request register to enable a service        
		            request on operation complete                                
		
		   These settings are required for the default implementation of the      
		   WaitForOPCCallback to work correctly.                                 
		*/
		viCheckErr( viPrintf (io, "*CLS;*ESE 1;*SRE 32"));
		 /*============================================================END=CHANGE=*/
		
		viCheckErr( viPrintf (io, "APPL P6V,3.5,1.5"));
        }
Error:
    return error;
}

/*****************************************************************************
 * Function: ReadToFile and WriteFromFile Functions                          
 * Purpose:  Functions for instrument driver developers to read/write        
 *           instrument data to/from a file.                                 
 *****************************************************************************/
static ViStatus E3631A_ReadToFile (ViSession vi, ViConstString filename, 
                                     ViInt32 maxBytesToRead, ViInt32 fileAction, 
                                     ViInt32 *totalBytesWritten)  
{   
    return Ivi_ReadToFile (vi, filename, maxBytesToRead, fileAction, totalBytesWritten);  
}   
static ViStatus E3631A_WriteFromFile (ViSession vi, ViConstString filename, 
                                        ViInt32 maxBytesToWrite, ViInt32 byteOffset, 
                                        ViInt32 *totalBytesWritten) 
{   
    return Ivi_WriteFromFile (vi, filename, maxBytesToWrite, byteOffset, totalBytesWritten); 
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: E3631A_CheckStatusCallback                                               
 * Purpose:  This function queries the instrument to determine if it has 
 *           encountered an error.  If the instrument has encountered an 
 *           error, this function returns the IVI_ERROR_INSTRUMENT_SPECIFIC 
 *           error code.  This function is called by the 
 *           E3631A_CheckStatus utility function.  The 
 *           Ivi_SetAttribute and Ivi_GetAttribute functions invoke this 
 *           function when the optionFlags parameter includes the
 *           IVI_VAL_DIRECT_USER_CALL flag.
 *           
 *           The user can disable calls to this function by setting the 
 *           IVI_ATTR_QUERY_INSTR_STATUS attribute to VI_FALSE.  The driver can 
 *           disable the check status callback for a particular attribute by 
 *           setting the IVI_VAL_DONT_CHECK_STATUS flag.
 *****************************************************************************/
static ViStatus _VI_FUNC E3631A_CheckStatusCallback (ViSession vi, ViSession io)
{
    ViStatus    error = VI_SUCCESS;
	
	
	/* Query instrument status */
	/*=CHANGE:===============================================================*
	    Change the *ESR? to the query command that requests the instrument to
	    indicate whether it has encountered an error.  The following example 
	    code is based on IEEE 488.2 Common System Commands.  Example:
	   */   
	    ViInt16     esr = 0; 
	    viCheckErr( viWrite (io, "*ESR?", 5, VI_NULL));
	
	    viCheckErr( viScanf (io, "%hd", &esr));
	
	    if ((esr & IEEE_488_2_QUERY_ERROR_BIT) ||
	        (esr & IEEE_488_2_DEVICE_DEPENDENT_ERROR_BIT) ||
	        (esr & IEEE_488_2_EXECUTION_ERROR_BIT) ||
	        (esr & IEEE_488_2_COMMAND_ERROR_BIT))
	            {     
	            viCheckErr( IVI_ERROR_INSTR_SPECIFIC);
	            }
	 /*============================================================END=CHANGE=*/
            
Error:
    return error;
}

/*****************************************************************************
 * Function: E3631A_WaitForOPCCallback                                               
 * Purpose:  This function waits until the instrument has finished processing 
 *           all pending operations.  This function is called by the 
 *           E3631A_WaitForOPC utility function.  The IVI engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the 
 *             IVI_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the 
 *             IVI_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *****************************************************************************/
static ViStatus _VI_FUNC E3631A_WaitForOPCCallback (ViSession vi, ViSession io)
{
    ViStatus    error = VI_SUCCESS;
    /*=CHANGE:===============================================================*
       Change this function to wait for operation complete for your specific 
       instrument.  This example function is based on the IEEE 488.2 Common  
       System Commands.  This example shows how to use the 
       E3631A_ATTR_OPC_TIMEOUT hidden attribute to control the
       length of time this function waits.  This function assumes that the 
       instrument does not generate service requests for other reasons.  This 
       example performs the following operations:
       
       - Enables the service request event.
       - Sends a command that forces the instrument to generate a service 
         request.
       - Waits for the service request.
       - Cleans up after the service request.
	*/
    ViInt32     opcTimeout;
    ViUInt16    statusByte;

    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, E3631A_ATTR_OPC_TIMEOUT, 
                                       0, &opcTimeout));

    viCheckErr( viEnableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE, VI_NULL));

    viCheckErr( viPrintf (io, "*OPC"));

        // wait for SRQ 
    viCheckErr( viWaitOnEvent (io, VI_EVENT_SERVICE_REQ, opcTimeout, VI_NULL, VI_NULL));
    viCheckErr( viDisableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE));

        // clean up after SRQ 
    viCheckErr( viBufWrite (io, "*CLS", 4, VI_NULL));
    viCheckErr( viReadSTB (io, &statusByte));
    
Error:
    viDiscardEvents (io, VI_EVENT_SERVICE_REQ, VI_QUEUE);
     /*============================================================END=CHANGE=*/

    return error;
}

/*****************************************************************************
 *----------------- Attribute Range Tables and Callbacks --------------------*
 *****************************************************************************/

/*- E3631A_ATTR_ID_QUERY_RESPONSE -*/

static ViStatus _VI_FUNC E3631AAttrIdQueryResponse_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rdBuffer[BUFFER_SIZE];
    ViUInt32    retCnt;
    
    viCheckErr( viPrintf (io, "*IDN?"));
    viCheckErr( viRead (io, rdBuffer, BUFFER_SIZE-1, &retCnt));
    rdBuffer[retCnt] = 0;

    checkErr( Ivi_SetValInStringCallback (vi, attributeId, rdBuffer));
    
Error:
    return error;
}
    
/*- E3631A_ATTR_SPECIFIC_DRIVER_REVISION -*/

static ViStatus _VI_FUNC E3631AAttrDriverRevision_ReadCallback (ViSession vi, 
                                                                  ViSession io,
                                                                  ViConstString channelName, 
                                                                  ViAttr attributeId,
                                                                  const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      driverRevision[256];
    
    
    sprintf (driverRevision, 
             "Driver: E3631A %.2f, Compiler: %s %3.2f, "
             "Components: IVIEngine %.2f, VISA-Spec %.2f",
             E3631A_MAJOR_VERSION + E3631A_MINOR_VERSION/1000.0, 
             IVI_COMPILER_NAME, IVI_COMPILER_VER_NUM, 
             IVI_ENGINE_MAJOR_VERSION + IVI_ENGINE_MINOR_VERSION/1000.0, 
             Ivi_ConvertVISAVer(VI_SPEC_VERSION));

    checkErr( Ivi_SetValInStringCallback (vi, attributeId, driverRevision));    
Error:
    return error;
}


/*- E3631A_ATTR_FIRMWARE_REVISION -*/

static ViStatus _VI_FUNC E3631AAttrFirmwareRevision_ReadCallback (ViSession vi, 
                                                                    ViSession io,
                                                                    ViConstString channelName, 
                                                                    ViAttr attributeId,
                                                                    const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      instrRev[BUFFER_SIZE];
    
    viCheckErr( viPrintf (io, "*IDN?"));

	viCheckErr( viScanf (io, "%*[^,],%*[^,],%*[^,],%256[^\n]", instrRev));

    checkErr( Ivi_SetValInStringCallback (vi, attributeId, instrRev));
    
Error:
    return error;
}

/*- E3631A_ATTR_INSTRUMENT_MANUFACTURER -*/

static ViStatus _VI_FUNC E3631AAttrInstrumentManufacturer_ReadCallback (ViSession vi, 
                                                                          ViSession io,
                                                                          ViConstString channelName, 
                                                                          ViAttr attributeId,
                                                                          const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rdBuffer[BUFFER_SIZE];
    ViChar      idQ[BUFFER_SIZE];
    
    checkErr( Ivi_GetAttributeViString (vi, "", E3631A_ATTR_ID_QUERY_RESPONSE,
                                        0, BUFFER_SIZE, idQ));
    sscanf (idQ, "%256[^,]", rdBuffer);

    checkErr( Ivi_SetValInStringCallback (vi, attributeId, rdBuffer));
    
Error:
    return error;
}

    

/*- E3631A_ATTR_INSTRUMENT_MODEL -*/

static ViStatus _VI_FUNC E3631AAttrInstrumentModel_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rdBuffer[BUFFER_SIZE];
    ViChar      idQ[BUFFER_SIZE];
    
    checkErr( Ivi_GetAttributeViString (vi, "", E3631A_ATTR_ID_QUERY_RESPONSE,
                                        0, BUFFER_SIZE, idQ));
    sscanf (idQ, "%*[^,],%256[^,]", rdBuffer);

    checkErr( Ivi_SetValInStringCallback (vi, attributeId, rdBuffer));
    
Error:
    return error;
}
    

/*- E3631A_ATTR_OUTPUT_ENABLED -*/

    /*=CHANGE:===============================================================*
        
        NOTE TO THE DEVELOPER: For some DC power supplies, the output
        is constantly enabled and is not readable or writable.  If this is
        true for your instrument, do the following:
        
        - Pass VI_TRUE as the default value parameter of the Ivi_AddAttribute
          function that creates this attribute.
          
        - Delete the E3631AAttrOutputEnabled_ReadCallback and
          E3631AAttrOutputEnabled_WriteCallback function
          prototypes and definitions.
        
        - Create a CheckCallback function for this attribute that verifies
          that the value the user specifies is VI_TRUE.
     *============================================================END=CHANGE=*/

static ViStatus _VI_FUNC E3631AAttrOutputEnabled_WriteCallback (ViSession vi, 
                                                                 ViSession io,
                                                                 ViConstString channelName, 
                                                                 ViAttr attributeId,
                                                                 ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to enable or disable the output.  
	*/
    viCheckErr( viPrintf (io, "OUTP %hd", value ? 1 : 0));
    
     /*============================================================END=CHANGE=*/
    
Error:
    return error;
}

static ViStatus _VI_FUNC E3631AAttrOutputEnabled_ReadCallback (ViSession vi, 
                                                                ViSession io,
                                                                ViConstString channelName, 
                                                                ViAttr attributeId,
                                                                ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;
        
    /*=CHANGE:===============================================================*
       
         Modify this example code to query whether the output is enabled.
	 */
    viCheckErr( viQueryf (io, "OUTP?","%hd", value));
    
     /*============================================================END=CHANGE=*/
    
Error:
    return error;
}

/*- E3631A_ATTR_VOLTAGE_LEVEL -*/
static IviRangeTableEntry attrVoltageLevelRangeTableEntries[] = 
    {
        {0.0, 25.00, 0, "", 0},
        {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrVoltageLevelRangeTable = 
    {
        IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrVoltageLevelRangeTableEntries,
    };
    
static ViStatus _VI_FUNC E3631AAttrVoltageLevel_WriteCallback (ViSession vi, 
                                                                ViSession io,
                                                                ViConstString channelName, 
                                                                ViAttr attributeId,
                                                                ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to set the voltage level.  
	*/
    viCheckErr( viPrintf (io, "VOLT %lf", value));
    
     /*============================================================END=CHANGE=*/

Error:
    return error;
}

static ViStatus _VI_FUNC E3631AAttrVoltageLevel_ReadCallback (ViSession vi, 
                                                               ViSession io,
                                                               ViConstString channelName, 
                                                               ViAttr attributeId,
                                                               ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to query the voltage level.  
	 */
    viCheckErr( viQueryf (io, "VOLT?", "%lf", value));
    
     /*============================================================END=CHANGE=*/

Error:
    return error;
}

/*- E3631A_ATTR_OVP_ENABLED -*/

    /*=CHANGE:===============================================================*
        
        NOTE TO THE DEVELOPER: For some DC power supplies, the OVP Limit
        is constantly enabled and is not readable or writable.  If this is
        true for your instrument, do the following:
        
        - Pass VI_TRUE as the default value parameter of the Ivi_AddAttribute
          function that creates this attribute.
          
        - Delete the E3631AAttrOutputEnabled_ReadCallback and
          E3631AAttrOutputEnabled_WriteCallback function
          prototypes and definitions.
        
        - Create a CheckCallback function for this attribute that verifies
          that the value the user specifies is VI_TRUE.
     *============================================================END=CHANGE=*/

static ViStatus _VI_FUNC E3631AAttrOVPEnabled_WriteCallback (ViSession vi, 
                                                              ViSession io,
                                                              ViConstString channelName, 
                                                              ViAttr attributeId,
                                                              ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to enable or disable the OVP Limit.  
	*/
    viCheckErr( viPrintf (io, "VOLT:PROT:STAT %hd", value));
    
     /*============================================================END=CHANGE=*/
    
Error:
    return error;
}

static ViStatus _VI_FUNC E3631AAttrOVPEnabled_ReadCallback (ViSession vi, 
                                                           ViSession io,
                                                           ViConstString channelName, 
                                                           ViAttr attributeId,
                                                           ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to query whether the OVP Limit is enabled.
	*/
    viCheckErr( viQueryf (io, "VOLT:PROT:STAT?", "%hd", value));
    
     /*============================================================END=CHANGE=*/

Error:
    return error;
}

/*- E3631A_ATTR_OVP_LIMIT -*/
static IviRangeTableEntry attrOVPLimitRangeTableEntries[] = 
    {
        {0.0, 30.00, 0, "", 0},
        {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrOVPLimitRangeTable = 
    {
        IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrOVPLimitRangeTableEntries,
    };

static ViStatus _VI_FUNC E3631AAttrOVPLimit_WriteCallback (ViSession vi, 
                                                            ViSession io,
                                                            ViConstString channelName, 
                                                            ViAttr attributeId,
                                                            ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to set the OVP Limit.
	 */
    viCheckErr( viPrintf (io, "VOLT:PROT %lf", value));
    
     /*============================================================END=CHANGE=*/
    
Error:
    return error;
}

static ViStatus _VI_FUNC E3631AAttrOVPLimit_ReadCallback (ViSession vi, 
                                                           ViSession io,
                                                           ViConstString channelName, 
                                                           ViAttr attributeId,
                                                           ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to query the OVP Limit.
	*/
    viCheckErr( viQueryf (io, "VOLT:PROT?", "%lf", value));
    
     /*============================================================END=CHANGE=*/

Error:
    return error;
}

/*- E3631A_ATTR_CURRENT_LIMIT_BEHAVIOR -*/

    /*=CHANGE:===============================================================*

        NOTE TO THE DEVELOPER: Some DC power supplies do not support 
        multiple current limit behaviors.  For example, your instrument might
        support a regulatory limit behavior, but not a trip behavior limit.
        If this is true for your instrument, do the following:
        
        - Delete the E3631AAttrCurrentLimitBehavior_ReadCallback
          and E3631AAttrCurrentLimitBehavior_WriteCallback
          function prototypes and definitions.
        
        - Pass the instrument's fixed value as the default value 
          parameter of the Ivi_AddAttribute function that creates this attribute.
  
        - Modify the attrCurrentLimitBehaviorRangeTable range table so that
          the only valid value is the current limit behavior your instrument
          supports.
        
     *============================================================END=CHANGE=*/

static IviRangeTableEntry attrCurrentLimitBehaviorRangeTableEntries[] =
    {
        {E3631A_VAL_CURRENT_REGULATE,  0, 0, "REG", 0},
        {E3631A_VAL_CURRENT_TRIP,      0, 0, "TRP", 0},        
        {IVI_RANGE_TABLE_LAST_ENTRY},
    };
static IviRangeTable attrCurrentLimitBehaviorRangeTable = 
    {
        IVI_VAL_DISCRETE,
        VI_FALSE,
        VI_FALSE,
        VI_NULL,
        attrCurrentLimitBehaviorRangeTableEntries,
    };
    
static ViStatus _VI_FUNC E3631AAttrCurrentLimitBehavior_WriteCallback (ViSession vi, 
                                                                        ViSession io,
                                                                        ViConstString channelName, 
                                                                        ViAttr attributeId,
                                                                        ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*

        Modify this example code to set the current limit behavior.  The 
        following example shows how to use a range table to look up the 
        correct command.
	 */
    ViString cmd;                                                                                                                     
                                                                                                                                
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrCurrentLimitBehaviorRangeTable,
                                              VI_NULL, VI_NULL, VI_NULL,
                                              VI_NULL, &cmd, VI_NULL));
    viCheckErr( viPrintf (io, "CURR:PROT:BEH %s", cmd));

     /*============================================================END=CHANGE=*/

Error:
    return error;
}

static ViStatus _VI_FUNC E3631AAttrCurrentLimitBehavior_ReadCallback (ViSession vi, 
                                                                       ViSession io,
                                                                       ViConstString channelName, 
                                                                       ViAttr attributeId,
                                                                       ViInt32* value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*

        Modify this example code to read the current limit behavior.  The 
        following example shows how to send a query to the instrument, read 
        a response, and look up the correct value via a range table.
	 */
    ViChar cmd[BUFFER_SIZE];
    ViInt32 rdBufferSize = BUFFER_SIZE;
    
    viCheckErr( viQueryf (io, "CURR:PROT:STAT?", "%#s", &rdBufferSize, cmd));
    viCheckErr( Ivi_GetViInt32EntryFromString (cmd, &attrCurrentLimitBehaviorRangeTable,
                                               value, VI_NULL, VI_NULL, VI_NULL,
                                               VI_NULL));

     /*============================================================END=CHANGE=*/

Error:
    return error;
}

/*- E3631A_ATTR_CURRENT_LIMIT -*/
static IviRangeTableEntry attrCurrentLimitRangeTableEntries[] = 
    {
        {0.0, 30.00, 0, "", 0},
        {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrCurrentLimitRangeTable = 
    {
        IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrCurrentLimitRangeTableEntries,
    };

static ViStatus _VI_FUNC E3631AAttrCurrentLimit_WriteCallback (ViSession vi, 
                                                                ViSession io,
                                                                ViConstString channelName, 
                                                                ViAttr attributeId,
                                                                ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to set the Current Limit.
	*/
    viCheckErr( viPrintf (io, "CURR %lf", value));
    
     /*============================================================END=CHANGE=*/
    
Error:
    return error;
}

static ViStatus _VI_FUNC E3631AAttrCurrentLimit_ReadCallback (ViSession vi, 
                                                               ViSession io,
                                                               ViConstString channelName, 
                                                               ViAttr attributeId,
                                                               ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to query the Current Limit.
	 */
    viCheckErr( viQueryf (io, "CURR?", "%lf", value));
    
     /*============================================================END=CHANGE=*/

Error:
    return error;
}

/*- E3631A_ATTR_TRIGGER_SOURCE -*/

    /*=CHANGE:===============================================================*

        NOTE TO THE DEVELOPER: Some DC power supplies do not support 
        multiple trigger sources.  For example, your instrument might
        support only an external trigger.  If this is true for your
        instrument, do the following:
        
        - Delete the E3631AAttrTriggerSource_ReadCallback
          and E3631AAttrTriggerSource_WriteCallback
          function prototypes and definitions.
        
        - Pass the instrument's fixed value as the default value 
          parameter of the Ivi_AddAttribute function that creates this attribute.
  
        - Modify the attrTriggerSourceRangeTable range table so that
          the only valid value is the trigger source your instrument
          supports.
        
     *============================================================END=CHANGE=*/

static IviRangeTableEntry attrTriggerSourceRangeTableEntries[] =
    {
        {E3631A_VAL_TRIG_IMMEDIATE,  0, 0, "IMM",    0},
        {E3631A_VAL_TRIG_EXTERNAL,   0, 0, "EXT",    0},        
        {E3631A_VAL_SOFTWARE_TRIG,   0, 0, "SOFT",   0},
        {E3631A_VAL_TRIG_TTL0,       0, 0, "TTL0",   0},
        {E3631A_VAL_TRIG_TTL1,       0, 0, "TTL1",   0},
        {E3631A_VAL_TRIG_TTL2,       0, 0, "TTL2",   0},
        {E3631A_VAL_TRIG_TTL3,       0, 0, "TTL3",   0},
        {E3631A_VAL_TRIG_TTL4,       0, 0, "TTL4",   0},
        {E3631A_VAL_TRIG_TTL5,       0, 0, "TTL5",   0},
        {E3631A_VAL_TRIG_TTL6,       0, 0, "TTL6",   0},
        {E3631A_VAL_TRIG_TTL7,       0, 0, "TTL7",   0},
        {E3631A_VAL_TRIG_ECL0,       0, 0, "ECL0",   0},
        {E3631A_VAL_TRIG_ECL1,       0, 0, "ECL1",   0},
        {E3631A_VAL_TRIG_PXI_STAR,   0, 0, "STAR",   0},
        {E3631A_VAL_TRIG_RTSI_0,     0, 0, "RTSI 0", 0},
        {E3631A_VAL_TRIG_RTSI_1,     0, 0, "RTSI 1", 0},
        {E3631A_VAL_TRIG_RTSI_2,     0, 0, "RTSI 2", 0},
        {E3631A_VAL_TRIG_RTSI_3,     0, 0, "RTSI 3", 0},
        {E3631A_VAL_TRIG_RTSI_4,     0, 0, "RTSI 4", 0},
        {E3631A_VAL_TRIG_RTSI_5,     0, 0, "RTSI 5", 0},
        {E3631A_VAL_TRIG_RTSI_6,     0, 0, "RTSI 6", 0},
        {IVI_RANGE_TABLE_LAST_ENTRY},
    };
static IviRangeTable attrTriggerSourceRangeTable = 
    {
        IVI_VAL_DISCRETE,
        VI_FALSE,
        VI_FALSE,
        VI_NULL,
        attrTriggerSourceRangeTableEntries,
    };
    
static ViStatus _VI_FUNC E3631AAttrTriggerSource_WriteCallback (ViSession vi, 
                                                                 ViSession io,
                                                                 ViConstString channelName, 
                                                                 ViAttr attributeId,
                                                                 ViInt32 value)
{
    ViStatus error = VI_SUCCESS;

    /*=CHANGE:===============================================================*

        Modify this example code to set the trigger source.  The 
        following example shows how to use a range table to look up the 
        correct command.
	*/
    ViString cmd;                                                                                                                     
                                                                                                                                
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrTriggerSourceRangeTable,
                                              VI_NULL, VI_NULL, VI_NULL,
                                              VI_NULL, &cmd, VI_NULL));
    viCheckErr( viPrintf (io, "TRIG:SOUR %s", cmd));

     /*============================================================END=CHANGE=*/
    
Error:
    return error;
}

static ViStatus _VI_FUNC E3631AAttrTriggerSource_ReadCallback (ViSession vi, 
                                                                ViSession io,
                                                                ViConstString channelName, 
                                                                ViAttr attributeId,
                                                                ViInt32* value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*

        Modify this example code to read the trigger source.  The 
        following example shows how to send a query to the instrument, read 
        a response, and look up the correct value via a range table.
	  */
    ViChar cmd[BUFFER_SIZE];
    ViInt32 rdBufferSize = BUFFER_SIZE;
    
    viCheckErr( viQueryf (io, "TRIG:SOUR?", "%#s", &rdBufferSize, cmd));
    viCheckErr( Ivi_GetViInt32EntryFromString (cmd, &attrTriggerSourceRangeTable,
                                               value, VI_NULL, VI_NULL,
                                               VI_NULL, VI_NULL));

     /*============================================================END=CHANGE=*/

Error:
    return error;
}

/*- E3631A_ATTR_TRIGGERED_VOLTAGE_LEVEL -*/
static ViStatus _VI_FUNC E3631AAttrTrigVoltageLevel_WriteCallback (ViSession vi, 
                                                                    ViSession io,
                                                                    ViConstString channelName, 
                                                                    ViAttr attributeId,
                                                                    ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to set the Triggered Voltage Level.
	*/
    viCheckErr( viPrintf (io, "VOLT:TRIG %lf", value));
    
     /*============================================================END=CHANGE=*/
    
Error:
    return error;
}

static ViStatus _VI_FUNC E3631AAttrTrigVoltageLevel_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to query the Triggered Voltage Level.

    viCheckErr( viQueryf (io, "VOLT:TRIG?", "%lf", value));
    
     *============================================================END=CHANGE=*/

Error:
    return error;
}

/*- E3631A_ATTR_TRIGGERED_CURRENT_LIMIT -*/
static ViStatus _VI_FUNC E3631AAttrTrigCurrentLimit_WriteCallback (ViSession vi, 
                                                                    ViSession io,
                                                                    ViConstString channelName, 
                                                                    ViAttr attributeId,
                                                                    ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to set the Triggered Current Limit.
	*/
    viCheckErr( viPrintf (io, "CURR:TRIG %lf", value));
    
     /*============================================================END=CHANGE=*/
    
Error:
    return error;
}

static ViStatus _VI_FUNC E3631AAttrTrigCurrentLimit_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    
    /*=CHANGE:===============================================================*
        
        Modify this example code to query the Triggered Current Limit.
	 */
    viCheckErr( viQueryf (io, "CURR:TRIG?", "%lf", value));
    
     /*============================================================END=CHANGE=*/

Error:
    return error;
}

/*****************************************************************************
 * Function: E3631A_InitAttributes                                                  
 * Purpose:  This function adds attributes to the IVI session, initializes   
 *           instrument attributes, and sets attribute invalidation          
 *           dependencies.                                                   
 *****************************************************************************/
static ViStatus E3631A_InitAttributes (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     flags = 0;
        /*=CHANGE:=============================================================*

            NOTE TO THE DEVELOPER:  You can add additional parameters to the
            prototype of this function.  This is useful when you want to pass
            information from the initialization functions.  The Attribute Editor 
            in LabWindows/CVI requires that the name of this function be 
            E3631A_InitAttributes.

         *==========================================================END=CHANGE=*/
    
        /*- Initialize instrument attributes --------------------------------*/            

    checkErr( Ivi_SetAttributeViInt32 (vi, "", E3631A_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION,
                                       0, E3631A_MAJOR_VERSION));
    checkErr( Ivi_SetAttributeViInt32 (vi, "", E3631A_ATTR_SPECIFIC_DRIVER_MINOR_VERSION,
                                       0, E3631A_MINOR_VERSION));
    checkErr( Ivi_SetAttrReadCallbackViString (vi, E3631A_ATTR_SPECIFIC_DRIVER_REVISION,
                                               E3631AAttrDriverRevision_ReadCallback));
    checkErr( Ivi_SetAttributeViString (vi, "", E3631A_ATTR_SUPPORTED_CLASSES,
                                        0, E3631A_SUPPORTED_CLASSES));
    checkErr( Ivi_SetAttributeViInt32 (vi, "", E3631A_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION,
                                       0, E3631A_CLASS_SPEC_MAJOR_VERSION));
    checkErr( Ivi_SetAttributeViInt32 (vi, "", E3631A_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION,
                                       0, E3631A_CLASS_SPEC_MINOR_VERSION));
    checkErr( Ivi_SetAttributeViString (vi, "", E3631A_ATTR_IO_SESSION_TYPE,
                                        0, E3631A_IO_SESSION_TYPE));
    checkErr( Ivi_SetAttributeViString (vi, "", E3631A_ATTR_SUPPORTED_INSTRUMENT_MODELS,
                                        0, E3631A_SUPPORTED_INSTRUMENT_MODELS));


    checkErr( Ivi_SetAttrReadCallbackViString (vi, E3631A_ATTR_FIRMWARE_REVISION,
                                               E3631AAttrFirmwareRevision_ReadCallback));

	/*=CHANGE:===============================================================*
	    This driver uses the id query attribute (or vxi manufacturer id
	    attribute in register-based instrument case) to get the information
	    about the instrument manufacturer. Therefore, callbacks do not perform
	    any I/O and in order to simulate correctly, you must flag this
	    attribute as "use callbacks for simulation".  If you implement this 
	    attribute so that it uses instrument I/O in the callback, then you 
	    must remove this call from the driver.
	 *============================================================END=CHANGE=*/
	checkErr( Ivi_GetAttributeFlags (vi, E3631A_ATTR_INSTRUMENT_MANUFACTURER, &flags));
	checkErr( Ivi_SetAttributeFlags (vi, E3631A_ATTR_INSTRUMENT_MANUFACTURER, 
	                                 flags | IVI_VAL_USE_CALLBACKS_FOR_SIMULATION));
	checkErr( Ivi_SetAttrReadCallbackViString (vi, E3631A_ATTR_INSTRUMENT_MANUFACTURER,
	                                           E3631AAttrInstrumentManufacturer_ReadCallback));

	/*=CHANGE:===============================================================*
	    This driver uses the id query attribute (or vxi model code
	    attribute in register-based instrument case) to get the information
	    about the instrument model code. Therefore, callbacks do not perform
	    any I/O and in order to simulate correctly, you must flag this
	    attribute as "use callbacks for simulation".  If you implement this 
	    attribute so that it uses instrument I/O in the callback, then you 
	    must remove this call from the driver.
	 *============================================================END=CHANGE=*/
	checkErr( Ivi_GetAttributeFlags (vi, E3631A_ATTR_INSTRUMENT_MODEL, &flags));
	checkErr( Ivi_SetAttributeFlags (vi, E3631A_ATTR_INSTRUMENT_MODEL, 
	                                 flags | IVI_VAL_USE_CALLBACKS_FOR_SIMULATION));
	checkErr( Ivi_SetAttrReadCallbackViString (vi, E3631A_ATTR_INSTRUMENT_MODEL,
	                                           E3631AAttrInstrumentModel_ReadCallback));

    checkErr( Ivi_SetAttributeViString (vi, "", E3631A_ATTR_SPECIFIC_DRIVER_VENDOR,
                                        0, E3631A_DRIVER_VENDOR));
    checkErr( Ivi_SetAttributeViString (vi, "", E3631A_ATTR_SPECIFIC_DRIVER_DESCRIPTION,
                                        0, E3631A_DRIVER_DESCRIPTION));
    checkErr( Ivi_SetAttributeViAddr (vi, VI_NULL, IVI_ATTR_OPC_CALLBACK, 0,
                                      E3631A_WaitForOPCCallback));
    checkErr( Ivi_SetAttributeViAddr (vi, VI_NULL, IVI_ATTR_CHECK_STATUS_CALLBACK, 0,
                                      E3631A_CheckStatusCallback));
	    /*=CHANGE:===============================================================*
	       If your instrument does not support buffered I/O, change VI_TRUE in   
	       following function to VI_FALSE                                        
	     *============================================================END=CHANGE=*/
	checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, IVI_ATTR_SUPPORTS_WR_BUF_OPER_MODE, 
	                                     0, VI_TRUE));


    /*=CHANGE:===============================================================*
        Set the group capabilities attribute, Remove the extension groups
        this driver does not support, and add additional code to dynamically
        determine how the presence of specific instrument options may affect
        this attribute.
     *============================================================END=CHANGE=*/
        
	checkErr( Ivi_SetAttributeViString (vi, "", E3631A_ATTR_GROUP_CAPABILITIES, 0,
	                                    "IviDCPwrBase,"
	                                    "IviDCPwrTrigger,"
	                                    "IviDCPwrSoftwareTrigger,"
	                                    "IviDCPwrMeasurement"));
    
        /*- Add instrument-specific attributes ------------------------------*/            
	    checkErr( Ivi_AddAttributeViBoolean (vi, E3631A_ATTR_OUTPUT_ENABLED,
	                                         "E3631A_ATTR_OUTPUT_ENABLED",
	                                         VI_FALSE, 
	                                         IVI_VAL_MULTI_CHANNEL,
	                                         E3631AAttrOutputEnabled_ReadCallback,
	                                         E3631AAttrOutputEnabled_WriteCallback));
	    
	    checkErr( Ivi_AddAttributeViReal64 (vi, E3631A_ATTR_VOLTAGE_LEVEL,
	                                         "E3631A_ATTR_VOLTAGE_LEVEL",
	                                         0.0, 
	                                         IVI_VAL_MULTI_CHANNEL,
	                                         E3631AAttrVoltageLevel_ReadCallback,
	                                         E3631AAttrVoltageLevel_WriteCallback,
	                                         &attrVoltageLevelRangeTable, 0));
	                                         
	    checkErr( Ivi_AddAttributeViBoolean (vi, E3631A_ATTR_OVP_ENABLED,
	                                         "E3631A_ATTR_OVP_ENABLED",
	                                         VI_TRUE, 
	                                         IVI_VAL_MULTI_CHANNEL,
	                                         E3631AAttrOVPEnabled_ReadCallback, 
	                                         E3631AAttrOVPEnabled_WriteCallback));
	    
	    checkErr( Ivi_AddAttributeViReal64 (vi, E3631A_ATTR_OVP_LIMIT,
	                                         "E3631A_ATTR_OVP_LIMIT",
	                                         20.0, 
	                                         IVI_VAL_MULTI_CHANNEL,
	                                         E3631AAttrOVPLimit_ReadCallback,
	                                         E3631AAttrOVPLimit_WriteCallback,
	                                         &attrOVPLimitRangeTable, 0));
	    
	    checkErr( Ivi_AddAttributeViInt32 (vi, E3631A_ATTR_CURRENT_LIMIT_BEHAVIOR,
	                                         "E3631A_ATTR_CURRENT_LIMIT_BEHAVIOR",
	                                         E3631A_VAL_CURRENT_REGULATE, 
	                                         IVI_VAL_MULTI_CHANNEL,
	                                         E3631AAttrCurrentLimitBehavior_ReadCallback,
	                                         E3631AAttrCurrentLimitBehavior_WriteCallback,
	                                         &attrCurrentLimitBehaviorRangeTable));
	    
	    checkErr( Ivi_AddAttributeViReal64 (vi, E3631A_ATTR_CURRENT_LIMIT,
	                                         "E3631A_ATTR_CURRENT_LIMIT",
	                                         0.08, 
	                                         IVI_VAL_MULTI_CHANNEL,
	                                         E3631AAttrCurrentLimit_ReadCallback,
	                                         E3631AAttrCurrentLimit_WriteCallback,
	                                         &attrCurrentLimitRangeTable, 0));
	    
	    checkErr( Ivi_AddAttributeViInt32 (vi, E3631A_ATTR_TRIGGER_SOURCE,
	                                         "E3631A_ATTR_TRIGGER_SOURCE",
	                                         E3631A_VAL_TRIG_IMMEDIATE, 
	                                         IVI_VAL_MULTI_CHANNEL,
	                                         E3631AAttrTriggerSource_ReadCallback,
	                                         E3631AAttrTriggerSource_WriteCallback,
	                                         &attrTriggerSourceRangeTable));
	      
	    checkErr( Ivi_AddAttributeViReal64 (vi, E3631A_ATTR_TRIGGERED_VOLTAGE_LEVEL,
	                                         "E3631A_ATTR_TRIGGERED_VOLTAGE_LEVEL",
	                                         0.0, 
	                                         IVI_VAL_MULTI_CHANNEL,
	                                         E3631AAttrTrigVoltageLevel_ReadCallback,
	                                         E3631AAttrTrigVoltageLevel_WriteCallback,
	                                         &attrVoltageLevelRangeTable, 0));
	    
	    checkErr( Ivi_AddAttributeViReal64 (vi, E3631A_ATTR_TRIGGERED_CURRENT_LIMIT,
	                                         "E3631A_ATTR_TRIGGERED_CURRENT_LIMIT",
	                                         0.08, 
	                                         IVI_VAL_MULTI_CHANNEL,
	                                         E3631AAttrTrigCurrentLimit_ReadCallback,
	                                         E3631AAttrTrigCurrentLimit_WriteCallback,
	                                         &attrCurrentLimitRangeTable, 0));
	
	checkErr( Ivi_AddAttributeViString (vi, E3631A_ATTR_ID_QUERY_RESPONSE,
	                                    "E3631A_ATTR_ID_QUERY_RESPONSE", 
	                                    "HEWLETT-PACKARD,E3631A,0,1.4-5.0-1.0",
	                                    IVI_VAL_NOT_USER_WRITABLE,
	                                    E3631AAttrIdQueryResponse_ReadCallback,
	                                    VI_NULL));
	                                           
    checkErr( Ivi_AddAttributeViInt32 (vi, E3631A_ATTR_OPC_TIMEOUT,
                                       "E3631A_ATTR_OPC_TIMEOUT",
                                       5000, IVI_VAL_HIDDEN | IVI_VAL_DONT_CHECK_STATUS,
                                       VI_NULL, VI_NULL, VI_NULL)); 
                                       
        /*- Setup attribute invalidations -----------------------------------*/            
	 

    /*=CHANGE:===============================================================*
       
       Set attribute dependencies by calling the additional 
       Ivi_AddAttributeInvalidation functions here.  Remove the dependencies
       that do not apply to your instrument by deleting the calls to
       Ivi_AddAttributeInvalidation.
       When you initially add an attribute, it applies to all channels. 
       If you want it to apply to only a subset, call the 
       Ivi_RestrictAttrToChannels function.

     *============================================================END=CHANGE=*/

Error:
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
