/****************************************************************************
 *                       Triple Output DC Power Supply                           
 *---------------------------------------------------------------------------
 *   Copyright (c) National Instruments 1998.  All Rights Reserved.         
 *---------------------------------------------------------------------------
 *                                                                          
 * Title:    E3631A.h                                        
 * Purpose:  Triple Output DC Power Supply                                       
 *           instrument driver declarations.                                
 *                                                                          
 ****************************************************************************/

#ifndef __E3631A_HEADER
#define __E3631A_HEADER

#include <ivi.h>
#include <ividcpwr.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/
#define E3631A_MAJOR_VERSION                2     /* Instrument driver major version   */
#define E3631A_MINOR_VERSION                0     /* Instrument driver minor version   */
                                                                
#define E3631A_CLASS_SPEC_MAJOR_VERSION     2     /* Class specification major version */
#define E3631A_CLASS_SPEC_MINOR_VERSION     0     /* Class specification minor version */

#define E3631A_SUPPORTED_CLASSES            "IviDCPwr"

        /*=CHANGE:===============================================================
           Add supported instrument models.
           Add your company name.                                                 
           Add useful instrument driver description.                                                 
         *============================================================END=CHANGE=*/
#define E3631A_SUPPORTED_INSTRUMENT_MODELS  "MODEL1,MODEL2,MODEL3,MODEL4"
#define E3631A_DRIVER_VENDOR                "My Company & Co."
#define E3631A_DRIVER_DESCRIPTION           "Description"
                    



/**************************************************************************** 
 *------------------------------ Useful Macros -----------------------------* 
 ****************************************************************************/

    /*- Defined values for rangeType parameter of function -----------------*/
    /*- ConfigureOutputRange -----------------------------------------------*/
#define E3631A_VAL_RANGE_CURRENT               IVIDCPWR_VAL_RANGE_CURRENT              
#define E3631A_VAL_RANGE_VOLTAGE               IVIDCPWR_VAL_RANGE_VOLTAGE              
                                                                                            
    /*=CHANGE:===============================================================*
        Add definitions for range types here starting at
        IVIDCPWR_VAL_RANGE_TYPE_SPECIFIC_EXT_BASE.  Example:

#define E3631A_VAL_NEW_RANGE               (IVIDCPWR_VAL_RANGE_TYPE_SPECIFIC_EXT_BASE + 0L)
     *============================================================END=CHANGE=*/

    /*- Defined values for outputState parameter of function ---------------*/
    /*- QueryOutputState ---------------------------------------------------*/
#define E3631A_VAL_OUTPUT_CONSTANT_VOLTAGE     IVIDCPWR_VAL_OUTPUT_CONSTANT_VOLTAGE
#define E3631A_VAL_OUTPUT_CONSTANT_CURRENT     IVIDCPWR_VAL_OUTPUT_CONSTANT_CURRENT
#define E3631A_VAL_OUTPUT_OVER_VOLTAGE         IVIDCPWR_VAL_OUTPUT_OVER_VOLTAGE       
#define E3631A_VAL_OUTPUT_OVER_CURRENT         IVIDCPWR_VAL_OUTPUT_OVER_CURRENT       
#define E3631A_VAL_OUTPUT_UNREGULATED          IVIDCPWR_VAL_OUTPUT_UNREGULATED    

    /*=CHANGE:===============================================================*
        Add definitions for output states here starting at
        IVIDCPWR_VAL_OUTPUT_STATE_SPECIFIC_EXT_BASE.  Example:

#define E3631A_VAL_NEW_STATE               (IVIDCPWR_VAL_OUTPUT_STATE_SPECIFIC_EXT_BASE + 0L)
     *============================================================END=CHANGE=*/

    /*- Defined values for measurementType parameter of function -----------*/
    /*- Measure ------------------------------------------------------------*/
#define E3631A_VAL_MEASURE_CURRENT             IVIDCPWR_VAL_MEASURE_CURRENT
#define E3631A_VAL_MEASURE_VOLTAGE             IVIDCPWR_VAL_MEASURE_VOLTAGE

    /*=CHANGE:===============================================================*
        Add definitions for measurement types here starting at
        IVIDCPWR_VAL_MEASURE_SPECIFIC_EXT_BASE.  Example:

#define E3631A_VAL_NEW_MEASUREMENT               (IVIDCPWR_VAL_OUTPUT_STATE_SPECIFIC_EXT_BASE + 0L)
     *============================================================END=CHANGE=*/
        /*=CHANGE:===============================================================
           Add useful instrument macros here.                                                 
         *============================================================END=CHANGE=*/

/**************************************************************************** 
 *---------------------------- Attribute Defines ---------------------------* 
 ****************************************************************************/

    /*- IVI Inherent Instrument Attributes ---------------------------------*/    

        /* User Options */
#define E3631A_ATTR_RANGE_CHECK                   IVI_ATTR_RANGE_CHECK                    /* ViBoolean */
#define E3631A_ATTR_QUERY_INSTR_STATUS            IVI_ATTR_QUERY_INSTR_STATUS             /* ViBoolean */
#define E3631A_ATTR_CACHE                         IVI_ATTR_CACHE                          /* ViBoolean */
#define E3631A_ATTR_SIMULATE                      IVI_ATTR_SIMULATE                       /* ViBoolean */
#define E3631A_ATTR_RECORD_COERCIONS              IVI_ATTR_RECORD_COERCIONS               /* ViBoolean */

        /* Instrument Capabilities */
#define E3631A_ATTR_NUM_CHANNELS                  IVI_ATTR_NUM_CHANNELS                   /* ViInt32,  read-only  */

        /* Driver Information  */
#define E3631A_ATTR_SPECIFIC_DRIVER_PREFIX        IVI_ATTR_SPECIFIC_DRIVER_PREFIX         /* ViString, read-only  */
#define E3631A_ATTR_SUPPORTED_CLASSES             IVI_ATTR_SUPPORTED_CLASSES              /* ViString, read-only  */
#define E3631A_ATTR_SUPPORTED_INSTRUMENT_MODELS   IVI_ATTR_SUPPORTED_INSTRUMENT_MODELS    /* ViString, read-only  */
#define E3631A_ATTR_GROUP_CAPABILITIES            IVI_ATTR_GROUP_CAPABILITIES             /* ViString, read-only  */
#define E3631A_ATTR_INSTRUMENT_MANUFACTURER       IVI_ATTR_INSTRUMENT_MANUFACTURER        /* ViString, read-only  */
#define E3631A_ATTR_INSTRUMENT_MODEL              IVI_ATTR_INSTRUMENT_MODEL               /* ViString, read-only  */
#define E3631A_ATTR_FIRMWARE_REVISION             IVI_ATTR_FIRMWARE_REVISION              /* ViString, read-only  */
#define E3631A_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION IVI_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION  /* ViInt32,  read-only  */
#define E3631A_ATTR_SPECIFIC_DRIVER_MINOR_VERSION IVI_ATTR_SPECIFIC_DRIVER_MINOR_VERSION  /* ViInt32,  read-only  */
#define E3631A_ATTR_SPECIFIC_DRIVER_REVISION      IVI_ATTR_SPECIFIC_DRIVER_REVISION       /* ViString, read-only  */
#define E3631A_ATTR_SPECIFIC_DRIVER_VENDOR        IVI_ATTR_SPECIFIC_DRIVER_VENDOR         /* ViString, read-only  */
#define E3631A_ATTR_SPECIFIC_DRIVER_DESCRIPTION   IVI_ATTR_SPECIFIC_DRIVER_DESCRIPTION    /* ViString, read-only  */
#define E3631A_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION IVI_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION /* ViInt32, read-only */
#define E3631A_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION IVI_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION /* ViInt32, read-only */

        /* Error Info */
#define E3631A_ATTR_PRIMARY_ERROR                 IVI_ATTR_PRIMARY_ERROR                  /* ViInt32   */
#define E3631A_ATTR_SECONDARY_ERROR               IVI_ATTR_SECONDARY_ERROR                /* ViInt32   */
#define E3631A_ATTR_ERROR_ELABORATION             IVI_ATTR_ERROR_ELABORATION              /* ViString  */

        /* Advanced Session Information */
#define E3631A_ATTR_LOGICAL_NAME                  IVI_ATTR_LOGICAL_NAME                   /* ViString, read-only  */
#define E3631A_ATTR_RESOURCE_DESCRIPTOR           IVI_ATTR_RESOURCE_DESCRIPTOR            /* ViString, read-only  */
#define E3631A_ATTR_IO_SESSION_TYPE               IVI_ATTR_IO_SESSION_TYPE                /* ViString, read-only  */
#define E3631A_ATTR_IO_SESSION                    IVI_ATTR_IO_SESSION                     /* ViSession, read-only */
    

    /*- Instrument-Specific Attributes -------------------------------------*/

    /*- Basic Instrument Operations ----------------------------------------*/
#define E3631A_ATTR_VOLTAGE_LEVEL             IVIDCPWR_ATTR_VOLTAGE_LEVEL           /* ViReal64,  multi-channel, volts */
#define E3631A_ATTR_OVP_ENABLED               IVIDCPWR_ATTR_OVP_ENABLED             /* ViBoolean, multi-channel        */
#define E3631A_ATTR_OVP_LIMIT                 IVIDCPWR_ATTR_OVP_LIMIT               /* ViReal64,  multi-channel, volts */
#define E3631A_ATTR_CURRENT_LIMIT_BEHAVIOR    IVIDCPWR_ATTR_CURRENT_LIMIT_BEHAVIOR  /* ViInt32,   multi-channel        */
#define E3631A_ATTR_CURRENT_LIMIT             IVIDCPWR_ATTR_CURRENT_LIMIT           /* ViReal64,  multi-channel, amps  */
#define E3631A_ATTR_OUTPUT_ENABLED            IVIDCPWR_ATTR_OUTPUT_ENABLED          /* ViBoolean, multi-channel        */

    /*- Trigger Attributes -------------------------------------------------*/
#define E3631A_ATTR_TRIGGER_SOURCE            IVIDCPWR_ATTR_TRIGGER_SOURCE          /* ViInt32,   multi-channel        */
#define E3631A_ATTR_TRIGGERED_CURRENT_LIMIT   IVIDCPWR_ATTR_TRIGGERED_CURRENT_LIMIT /* ViReal64,  multi-channel, amps  */
#define E3631A_ATTR_TRIGGERED_VOLTAGE_LEVEL   IVIDCPWR_ATTR_TRIGGERED_VOLTAGE_LEVEL /* ViReal64,  multi-channel, volts */

#define E3631A_ATTR_ID_QUERY_RESPONSE      (IVI_SPECIFIC_PUBLIC_ATTR_BASE + 1L)     /* ViString (Read Only) */
    /*=CHANGE:===============================================================*
        Add attribute definitions here.                                       
        #define E3631A_ATTR_MY_ATTRIBUTE     (IVI_SPECIFIC_PUBLIC_ATTR_BASE + 2L) 
     *============================================================END=CHANGE=*/

    
/**************************************************************************** 
 *------------------------ Attribute Value Defines -------------------------* 
 ****************************************************************************/
    /*- Defined values for attribute E3631A_ATTR_CURRENT_LIMIT_BEHAVIOR ---*/
#define E3631A_VAL_CURRENT_REGULATE           IVIDCPWR_VAL_CURRENT_REGULATE
#define E3631A_VAL_CURRENT_TRIP               IVIDCPWR_VAL_CURRENT_TRIP    
    /*=CHANGE:===============================================================*
        Add definitions for current limit behavior here starting at
        IVIDCPWR_VAL_CURRENT_LIMIT_BEHAVIOR_SPECIFIC_EXT_BASE.  Example:

#define E3631A_VAL_NEW_BEHAVIOR               (IVIDCPWR_VAL_CURRENT_LIMIT_BEHAVIOR_SPECIFIC_EXT_BASE + 0L)
     *============================================================END=CHANGE=*/

    /*- Defined values for attribute E3631A_ATTR_TRIGGER_SOURCE -----------*/
#define E3631A_VAL_TRIG_IMMEDIATE             IVIDCPWR_VAL_TRIG_IMMEDIATE   
#define E3631A_VAL_TRIG_EXTERNAL              IVIDCPWR_VAL_TRIG_EXTERNAL    
#define E3631A_VAL_SOFTWARE_TRIG              IVIDCPWR_VAL_SOFTWARE_TRIG     
#define E3631A_VAL_TRIG_TTL0                  IVIDCPWR_VAL_TRIG_TTL0
#define E3631A_VAL_TRIG_TTL1                  IVIDCPWR_VAL_TRIG_TTL1
#define E3631A_VAL_TRIG_TTL2                  IVIDCPWR_VAL_TRIG_TTL2
#define E3631A_VAL_TRIG_TTL3                  IVIDCPWR_VAL_TRIG_TTL3
#define E3631A_VAL_TRIG_TTL4                  IVIDCPWR_VAL_TRIG_TTL4
#define E3631A_VAL_TRIG_TTL5                  IVIDCPWR_VAL_TRIG_TTL5
#define E3631A_VAL_TRIG_TTL6                  IVIDCPWR_VAL_TRIG_TTL6
#define E3631A_VAL_TRIG_TTL7                  IVIDCPWR_VAL_TRIG_TTL7
#define E3631A_VAL_TRIG_ECL0                  IVIDCPWR_VAL_TRIG_ECL0
#define E3631A_VAL_TRIG_ECL1                  IVIDCPWR_VAL_TRIG_ECL1
#define E3631A_VAL_TRIG_PXI_STAR              IVIDCPWR_VAL_TRIG_PXI_STAR
#define E3631A_VAL_TRIG_RTSI_0                IVIDCPWR_VAL_TRIG_RTSI_0
#define E3631A_VAL_TRIG_RTSI_1                IVIDCPWR_VAL_TRIG_RTSI_1
#define E3631A_VAL_TRIG_RTSI_2                IVIDCPWR_VAL_TRIG_RTSI_2
#define E3631A_VAL_TRIG_RTSI_3                IVIDCPWR_VAL_TRIG_RTSI_3
#define E3631A_VAL_TRIG_RTSI_4                IVIDCPWR_VAL_TRIG_RTSI_4
#define E3631A_VAL_TRIG_RTSI_5                IVIDCPWR_VAL_TRIG_RTSI_5
#define E3631A_VAL_TRIG_RTSI_6                IVIDCPWR_VAL_TRIG_RTSI_6
    /*=CHANGE:===============================================================*
        Add definitions for trigger source here starting at
        IVIDCPWR_VAL_TRIG_SRC_BEHAVIOR_SPECIFIC_EXT_BASE.  Example:

#define E3631A_VAL_NEW_TRIGGER_SOURCE         (IVIDCPWR_VAL_TRIG_SRC_BEHAVIOR_SPECIFIC_EXT_BASE + 0L)
     *============================================================END=CHANGE=*/

        /* Instrument specific attribute value definitions */

    /*=CHANGE:==============================================================*
        Add value defines for instrument specific attributes here.  Example

    // Defined values for attribute E3631A_ATTR_EXAMPLE 

#define E3631A_VAL_EXAMPLE_1             1
#define E3631A_VAL_EXAMPLE_2             2
#define E3631A_VAL_EXAMPLE_3             3
     *===========================================================END=CHANGE=*/

/**************************************************************************** 
 *---------------- Instrument Driver Function Declarations -----------------* 
 ****************************************************************************/

    /*- Init and Close Functions -------------------------------------------*/
ViStatus _VI_FUNC  E3631A_init (ViRsrc resourceName, ViBoolean IDQuery,
                                  ViBoolean resetDevice, ViSession *vi);
ViStatus _VI_FUNC  E3631A_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                             ViBoolean resetDevice, ViString optionString, 
                                             ViSession *newVi);
ViStatus _VI_FUNC  E3631A_close (ViSession vi);   

    /*- Coercion Info Functions --------------------------------------------*/
ViStatus _VI_FUNC  E3631A_GetNextCoercionRecord (ViSession vi,
                                                   ViInt32 bufferSize,
                                                   ViChar record[]);

    /*- Locking Functions --------------------------------------------------*/
ViStatus _VI_FUNC  E3631A_LockSession (ViSession vi, ViBoolean *callerHasLock);   
ViStatus _VI_FUNC  E3631A_UnlockSession (ViSession vi, ViBoolean *callerHasLock);


    /*- Basic Instrument Operation -----------------------------------------*/
ViStatus _VI_FUNC E3631A_ConfigureOutputEnabled (ViSession vi,
                                                  ViConstString channelName,
                                                  ViBoolean enabled);

ViStatus _VI_FUNC E3631A_ConfigureOutputRange (ViSession vi,
                                                ViConstString channelName,
                                                ViInt32 rangeType,
                                                ViReal64 range);

ViStatus _VI_FUNC E3631A_ConfigureCurrentLimit (ViSession vi,
                                                 ViConstString channelName,
                                                 ViInt32 behavior,
                                                 ViReal64 limit);
ViStatus _VI_FUNC E3631A_ConfigureOVP (ViSession vi,
                                        ViConstString channelName,
                                        ViBoolean enabled,
                                        ViReal64 limit);

ViStatus _VI_FUNC E3631A_ConfigureVoltageLevel (ViSession vi,
                                                 ViConstString channelName,
                                                 ViReal64 level);

ViStatus _VI_FUNC E3631A_QueryOutputState (ViSession vi,
                                            ViConstString channelName,
                                            ViInt32 outputState,
                                            ViBoolean* inState);

ViStatus _VI_FUNC E3631A_ResetOutputProtection (ViSession vi,
                                                 ViConstString channelName);

    /*- Triggering ---------------------------------------------------------*/
ViStatus _VI_FUNC E3631A_ConfigureTriggerSource (ViSession vi,
                                                  ViConstString channelName,
                                                  ViInt32 source);

ViStatus _VI_FUNC E3631A_ConfigureTriggeredVoltageLevel (ViSession vi,
                                                          ViConstString channelName,
                                                          ViReal64 level);

ViStatus _VI_FUNC E3631A_ConfigureTriggeredCurrentLimit (ViSession vi,
                                                          ViConstString channelName,
                                                          ViReal64 limit);

ViStatus _VI_FUNC E3631A_Abort (ViSession vi);

ViStatus _VI_FUNC E3631A_Initiate (ViSession vi);

    /*- Software Triggering ------------------------------------------------*/
ViStatus _VI_FUNC E3631A_SendSoftwareTrigger (ViSession vi);

    /*- Measuring ----------------------------------------------------------*/
ViStatus _VI_FUNC E3631A_Measure (ViSession vi,
                                   ViConstString channelName,
                                   ViInt32 measurementType,
                                   ViReal64* measurement);
        /*=CHANGE:===============================================================*
           Add instrument-specific public function declarations here.                           
         *============================================================END=CHANGE=*/
    
    /*- Error Functions ----------------------------------------------------*/
ViStatus _VI_FUNC  E3631A_error_query (ViSession vi, ViInt32 *errorCode,
                                         ViChar errorMessage[]);
ViStatus _VI_FUNC  E3631A_GetErrorInfo (ViSession vi, ViStatus *primaryError, 
                                          ViStatus *secondaryError, 
                                          ViChar errorElaboration[256]);
ViStatus _VI_FUNC  E3631A_ClearErrorInfo (ViSession vi);
ViStatus _VI_FUNC  E3631A_error_message (ViSession vi, ViStatus errorCode,
                                           ViChar errorMessage[256]);
    
    /*- Utility Functions --------------------------------------------------*/
ViStatus _VI_FUNC  E3631A_reset (ViSession vi);
ViStatus _VI_FUNC  E3631A_self_test (ViSession vi, ViInt16 *selfTestResult,
                                       ViChar selfTestMessage[]);
ViStatus _VI_FUNC  E3631A_revision_query (ViSession vi, 
                                            ViChar instrumentDriverRevision[],
                                            ViChar firmwareRevision[]);
ViStatus _VI_FUNC  E3631A_WriteInstrData (ViSession vi, ViConstString writeBuffer); 
ViStatus _VI_FUNC  E3631A_ReadInstrData  (ViSession vi, ViInt32 numBytes, 
                                            ViChar rdBuf[], ViInt32 *bytesRead);

    /*- Set, Get, and Check Attribute Functions ----------------------------*/
ViStatus _VI_FUNC  E3631A_GetAttributeViInt32 (ViSession vi, ViConstString channelName, ViAttr attribute, ViInt32 *value);
ViStatus _VI_FUNC  E3631A_GetAttributeViReal64 (ViSession vi, ViConstString channelName, ViAttr attribute, ViReal64 *value);
ViStatus _VI_FUNC  E3631A_GetAttributeViString (ViSession vi, ViConstString channelName, ViAttr attribute, ViInt32 bufSize, ViChar value[]); 
ViStatus _VI_FUNC  E3631A_GetAttributeViSession (ViSession vi, ViConstString channelName, ViAttr attribute, ViSession *value);
ViStatus _VI_FUNC  E3631A_GetAttributeViBoolean (ViSession vi, ViConstString channelName, ViAttr attribute, ViBoolean *value);

ViStatus _VI_FUNC  E3631A_SetAttributeViInt32 (ViSession vi, ViConstString channelName, ViAttr attribute, ViInt32 value);
ViStatus _VI_FUNC  E3631A_SetAttributeViReal64 (ViSession vi, ViConstString channelName, ViAttr attribute, ViReal64 value);
ViStatus _VI_FUNC  E3631A_SetAttributeViString (ViSession vi, ViConstString channelName, ViAttr attribute, ViConstString value); 
ViStatus _VI_FUNC  E3631A_SetAttributeViSession (ViSession vi, ViConstString channelName, ViAttr attribute, ViSession value);
ViStatus _VI_FUNC  E3631A_SetAttributeViBoolean (ViSession vi, ViConstString channelName, ViAttr attribute, ViBoolean value);

ViStatus _VI_FUNC  E3631A_CheckAttributeViInt32 (ViSession vi, ViConstString channelName, ViAttr attribute, ViInt32 value);
ViStatus _VI_FUNC  E3631A_CheckAttributeViReal64 (ViSession vi, ViConstString channelName, ViAttr attribute, ViReal64 value);
ViStatus _VI_FUNC  E3631A_CheckAttributeViString (ViSession vi, ViConstString channelName, ViAttr attribute, ViConstString value); 
ViStatus _VI_FUNC  E3631A_CheckAttributeViSession (ViSession vi, ViConstString channelName, ViAttr attribute, ViSession value);
ViStatus _VI_FUNC  E3631A_CheckAttributeViBoolean (ViSession vi, ViConstString channelName, ViAttr attribute, ViBoolean value);

    /*********************************************************
        Functions reserved for class driver use only.
        End-users should not call these functions.  
     *********************************************************/
ViStatus _VI_FUNC  E3631A_IviInit (ViRsrc resourceName, ViBoolean IDQuery, 
                                     ViBoolean reset, ViSession vi);
ViStatus _VI_FUNC  E3631A_IviClose (ViSession vi);   

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/
#define E3631A_ERROR_TRIGGER_NOT_SOFTWARE         IVIDCPWR_ERROR_TRIGGER_NOT_SOFTWARE
#define E3631A_ERROR_RESET_PROT_NOT_SUPPORTED    (IVI_SPECIFIC_ERROR_BASE + 1L)
#define E3631A_ERROR_TRIGGER_ABORT_NOT_SUPPORTED (IVI_SPECIFIC_ERROR_BASE + 2L)
/**************************************************************************** 
 *---------------------------- End Include File ----------------------------* 
 ****************************************************************************/
#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __E3631A_HEADER */




